; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Soumen Basak
;   Jean-Philippe Bernard
;   Marc Betoule
;   Guillaume Castex
;   Jacques Delabrouille
;   Clive Dickinson
;   Gianfranco de Zotti
;   Fabrice Dodu
;   Joaquin Gonzalez-Nuevo
;   Frederic Guilloux
;   Samuel Leach
;   Maude Le Jeune
;   Marcella Massardi
;   Jean-Baptiste Melin
;   Marc-Antoine Miville-Deschenes
;   Roberta Paladini
;   Etienne Pointecouteau
;   Mathieu Remazeilles
;   Matthieu Roman
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	BANDINTEGRATED_EMLAW
;
; PURPOSE:
;	Function to compute the band integrated value of an emission law
;
; CATEGORY:
;	...
;
; CALLING SEQUENCE:
;	Result = BANDINTEGRATED_EMLAW(Freqband, Paramstruct, emission_name=, nuref=, numin=, numax=, temp=, specind=, $'
;				curvnuref=, curvampl=, dlcomp=, redshift=, n_integ_pts=, approx=, /double)
;
; INPUTS:
;	Freqband:	The input frequency band (created with GET_BAND_STRUCT), 
;			or a single value of a frequency in GHz.
;
; OPTIONAL INPUTS:
;	Paramstruct:	A structure of parameters for the emission law, that can
;			replace all the band emission parameter keywords (see EMLAW_PARAMSTRUCT).
;	
; KEYWORD PARAMETERS:
;	EMISSION_NAME:	 	Name of the emission law (see PSM_EMLAW)
;
;	NUREF:   	 	Reference frequency (Hz), at which the emission is
;	                 	normalised to 1. Required input keyword.
;
;	NUMIN:  	 	Minimum frequency of the emission. Default = 0
;	                 	(No minimum frequency). Excluded, i.e. the
;	                 	emission DOES vanish at Nu = NUMAX.
;
;	NUMAX:  	 	Maximum frequency of the emission. Default = Inf
;	                 	(No maximum frequency). Included, i.e. the
;	                 	emission does NOT vanish at Nu = NUMAX.
;
;	TEMP:    	 	Temperature. Used by 'BLACKBODY', 'GREYBODY',
;                	 	'FREEFREE' (in which case the temperature is
;                	 	in K) and by 'THERMALSZ' for relativistic
;                	 	corrections, (in which case the temperature is
;                	 	in keV). It is recommended to set this
;                	 	keyword, as default values are different for
;                	 	different emissions, and also may change. For
;                	 	SZ effect, a temperature of 0, or undefined,
;                	 	means that the non-relativistic limit is computed.
;
;	SPECIND:	 	Spectral index. Used by 'GREYBODY',
;	                 	'POWERLAW', 'CURVPOWERLAW'. Required input
;	                 	keyword. Warning, brightness units are assumed
;	                 	for the output, NOT brightness temperature.
;
;	CURVNUREF:	 	Value of the reference frequency (Hz) for the
;	                 	spectral index curvature. Used by 'CURVPOWERLAW'. Required input.
;
;	CURVAMPL:	 	Value of the amplitude of the spectral index
;	                 	curvature. Used by 'CURVPOWERLAW'. Required input.  
;
;	DLCOMP: 	 	Composition of the spinning dust according to
;	                 	Draine-Lazarian model. Optional (do not set it
;	                 	unless you know what you are doing).
;
;	REDSHIFT: 	 	Redshift at which the emisison law is computed.
;	                 	Used by 'SUBMMLAPI'
;
;	N_INTEG_POINTS: 	Number of points used to perform band
;				integration for tophat bands (default=1000).
;
;	APPROX: 		Set this keyword to compute the sum of the emission law 
;				at tabulated nu values instead of using INT_TABULATED. 
;
;	DOUBLE: 		Set this keyword to force double precision. 
;
; OUTPUTS:
;	Result is the band-integrated emission.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Numerical integration.
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	01/12/2008:	Jacques Delabrouille, First version.
;-
FUNCTION BANDINTEGRATED_EMLAW, freqband, paramstruct, emission_name=emission_name, $
                               nuref=nuref, numin=numin, numax=numax, $
                               temp=temp, specind=specind, curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, $                              
                               n_integ_pts=n_integ_pts, approx=approx, double=double, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() LT 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'BANDINTEGRATED_EMLAW: function to integrate in a band the flux from an emission'
     PSM_INFO_MESSAGE, 'SYNTAX: result = BANDINTEGRATED_EMLAW(freqband, paramstruct, emission_name=, $'
     PSM_INFO_MESSAGE, '                                      nuref=, numin=, numax=, temp=, specind=, $'
     PSM_INFO_MESSAGE, '                                      curvnuref=, curvampl=, dlcomp=, redshift=, $'
     PSM_INFO_MESSAGE, '                                      n_integ_pts=, approx=, /double)'
     PSM_INFO_MESSAGE, "'freqband' is the input band (created with GET_BAND_STRUCT)"
     PSM_INFO_MESSAGE, "'paramstruct' is an optional structure of emission law parameters (created with EMLAW_PARAMSTRUCT)"
     PSM_INFO_MESSAGE, "alternatively, parameters of the emission laws can be passed using the keywords."
     PSM_INFO_MESSAGE, "set 'n_integ_pts' to the number of points used to perform band integration"
     PSM_INFO_MESSAGE, "for tophat bands (1000 is the default)"
     PSM_INFO_MESSAGE, "set 'approx' to compute the sum of the emission law at tabulated nu values instead of using INT_TABULATED"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


  IF ISPOINTER(freqband) THEN band=*freqband ELSE band=freqband
  IF KEYWORD_SET(double) NE 0 THEN zero=0d0 ELSE zero=0.

  IF DEFINED(paramstruct) THEN BEGIN
     type = SIZE(paramstruct, /type)
     IF type NE 8 THEN MESSAGE, 'call to BANDINTEGRATED_EMLAW has changed, please fix your codes'
     tagnames = TAG_NAMES(paramstruct)
     wh = WHERE(tagnames EQ 'NUMIN', nwh) & IF nwh GE 1 THEN include_nurange=1 ELSE include_nurange=0
     ;wh = WHERE(tagnames EQ 'AMPL', nwh) & IF nwh GE 1 THEN include_ampl=1 ELSE include_ampl=0
     tagnames = TAG_NAMES(paramstruct)
     paramlist = EMLAW_PARAMLIST(paramstruct.law, include_nurange=include_nurange, include_ampl=include_ampl)
     nparam = N_ELEMENTS(paramlist)
     FOR i=0, nparam-1 DO BEGIN        
        ok = EXECUTE('param = paramstruct.'+paramlist[i]+'.value')
        IF ISFITSFILE(*param) THEN BEGIN
           MESSAGE, 'CASE NOT IMPLEMENTED'
        ENDIF ELSE BEGIN 
           ok = EXECUTE(paramlist[i] +' = *param')
        ENDELSE
     ENDFOR
     IF NOT KEYWORD_SET(emission_name) THEN emission_name = paramstruct.law
  ENDIF ELSE BEGIN
     paramlist = EMLAW_PARAMLIST(emission_name)
     nparam = N_ELEMENTS(paramlist)
  ENDELSE


IF NOT KEYWORD_SET(n_integ_pts) THEN npts=1000 ELSE npts=n_integ_pts

CASE band.shape OF 

   ;; Monochromatic ('DIRAC') band ---------------------------------------
   'DIRAC': BEGIN
      value = PSM_EMLAW(band.nu_c, emission_name=emission_name, nuref=nuref, temp=temp, specind=specind, $
                        curvampl=curvampl, curvnuref=curvnuref, dlcomp=dlcomp, redshift=redshift, double=double, numin=numin, numax=numax)
      RETURN, value / band.integ_dnu     ; division to normalise the band
   END

   ;; Square ('TOPHAT') band ---------------------------------------------
   'TOPHAT': BEGIN
      IF NOT KEYWORD_SET(double) THEN zero=0. else zero=0D0
      IF DEFINED(numin) THEN usenumin = MAX([numin,band.nu_min]) ELSE usenumin = band.nu_min
      IF DEFINED(numax) THEN usenumax = MIN([numax,band.nu_max]) ELSE usenumax = band.nu_max
      IF usenumax LE usenumin THEN RETURN, zero
      deltanu = band.nu_max - band.nu_min
      CASE STRUPCASE(emission_name) OF
         'POWERLAW': BEGIN                          ; integration performed analytically
            IF NOT KEYWORD_SET(nuref) THEN MESSAGE, 'reference frequency value needed for computing power law emission'
            IF NOT DEFINED(specind) THEN MESSAGE, 'spectral index value needed for computing power law emission'
            IF ISARRAY(specind) THEN BEGIN
               whm1 = WHERE(specind EQ -1, nwhm1, complement=whnm1, ncomplement=nwhnm1)
               IF KEYWORD_SET(double) NE 0 THEN res = DBLARR(N_ELEMENTS(specind)) ELSE res = FLTARR(N_ELEMENTS(specind))
               IF nwhm1 NE 0 THEN res[whm1] = nuref * ALOG(usenumax/usenumin) /band.integ_dnu
               IF nwhnm1 NE 0 THEN res[whnm1] = nuref/(specind[whnm1]+1.) * $
                                                ( (usenumax/nuref)^(specind[whnm1]+1) - (usenumin/nuref)^(specind[whnm1]+1) ) / deltanu
            ENDIF ELSE BEGIN
               IF specind NE (-1) THEN RETURN, nuref/(specind+1.) * ( (usenumax/nuref)^(specind+1) - (usenumin/nuref)^(specind+1) ) / deltanu $
               ELSE RETURN, nuref * ALOG(usenumax/usenumin) /band.integ_dnu
            ENDELSE
         END
         'DIRAC': BEGIN
            IF NOT KEYWORD_SET(nuref) THEN MESSAGE, 'reference frequency value needed for computing power law emission'
            RETURN, (nuref GT usenumin) * (nuref LE usenumax) /band.integ_dnu
         END
         ELSE: BEGIN                                ; integration performed numerically by simple summation"
            dnu = deltanu/npts
            nu = band.nu_min + (band.nu_max-band.nu_min)*(FINDGEN(npts)+0.5)/npts
            emission = PSM_EMLAW(nu, emission_name=emission_name, nuref=nuref, temp=temp, specind=specind, $
                                 curvampl=curvampl, curvnuref=curvnuref, dlcomp=dlcomp, redshift=redshift, numin=numin, numax=numax)
            RETURN, TOTAL(emission)*dnu / band.integ_dnu     ; division to normalise the band
         END         
      ENDCASE
   END

   ;; Tabulated ('TABLE') band -------------------------------------------
   'TABLE': BEGIN
      CASE STRUPCASE(emission_name) OF
         'DIRAC': BEGIN
            IF NOT KEYWORD_SET(nuref) THEN MESSAGE, 'reference frequency value needed for computing power law emission'
            IF N_ELEMENTS(nuref) NE 1 THEN STOP 
            IF KEYWORD_SET(double) NE 0 THEN zero=0d0 ELSE zero=0.
            whlonu = WHERE(band.nu LT nuref, nlonu, complement=whhinu, ncomplement=nhinu)
            IF nlonu EQ 0 THEN RETURN, zero
            IF nhinu EQ 0 THEN RETURN, zero
            nu1 = MAX(band.nu[whlonu],i1) & val1 = band.value[whlonu[i1]]
            nu2 = MIN(band.nu[whhinu],i2) & val2 = band.value[whhinu[i2]]
            IF KEYWORD_SET(double) NE 0 THEN RETURN, (val1 + (DOUBLE(nuref)-nu1)*(DOUBLE(val2)-val1)/(DOUBLE(nu2)-nu1)) / band.integ_dnu $
            ELSE RETURN, FLOAT((val1 + ((nuref)-nu1)*((val2)-val1)/((nu2)-nu1)) / band.integ_dnu)
         END
         ELSE: BEGIN                                
            nnu = N_ELEMENTS(band.nu)            
            dimparam=1
            IF KEYWORD_SET(approx) NE 0 THEN BEGIN
               IF DEFINED(nuref) THEN dimparam = dimparam > N_ELEMENTS(nuref)
               IF DEFINED(temp) THEN dimparam = dimparam > N_ELEMENTS(temp)
               IF DEFINED(specind) THEN dimparam = dimparam > N_ELEMENTS(specind)
               IF DEFINED(curvnuref) THEN STOP
               IF DEFINED(curvampl) THEN STOP
               IF DEFINED(dlcomp) THEN STOP
            ENDIF
            IF dimparam GT 1 THEN BEGIN              ; integration performed numerically by simple summation"
               nu_approx = band.nu
               dnu_approx = [0.5*(nu_approx[1]-nu_approx[0]), 0.5*(nu_approx[2:nnu-1]-nu_approx[0:nnu-3]),0.5*(nu_approx[nnu-1]-nu_approx[nnu-2])]
               integ = REPLICATE(zero,dimparam)
               FOR i = 0L, nnu-1 DO integ = integ + band.value[i] * dnu_approx[i] * $
                  PSM_EMLAW((band.nu)[i], emission_name=emission_name, nuref=nuref, temp=temp, specind=specind, $
                            curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, double=double, numin=numin, numax=numax)
            ENDIF ELSE BEGIN
               integ = INT_TABULATED(band.nu, band.value $
                                     * CALL_FUNCTION('PSM_EMLAW', band.nu, emission_name=emission_name, nuref=nuref, temp=temp, specind=specind, $
                                                  curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, double=double, numin=numin, numax=numax))
            ENDELSE
            RETURN, integ / band.integ_dnu
         END         
      ENDCASE
   END

   ;; Instrument ('INSTR') band ------------------------------------------
   'INSTR': BEGIN
      IF NOT PTR_VALID(band.data) THEN FILL_BAND_DATA, band
      RETURN, BANDINTEGRATED_EMLAW(*band.data, paramstruct, emission_name=emission_name, double=double, n_integ_pts = n_integ_pts, $
                               nuref=nuref, temp=temp, specind=specind, curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, $
                               numin=numin, numax=numax,approx=approx)
   END

ENDCASE

END
