; Copyright 2009-2011 ---- Marc Betoule, Jacques Delabrouille, Maude Le Jeune, Sam Leach, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Generate a cmb map
;
; @categories pipe
;   
; @keyword test {in}{optional}{type=boolean} If set no computation is
; done, the routine only test the validity of the parameters it receives
; @keyword debug {in}{optional}{type=boolean} If set no catch on error
; done, subdirectory components/cmb is not removed when an error occurs
;
; @uses cmb_prediction, cmb_constrained, cmb_simulation, writemap
; 
; @history <p>30/01/2007: First version, Marc Betoule </p>
; @history <p>31/01/2007: Split prediction and simulation, Marc Betoule </p>
; @history <p>06/02/2007: Model structure creation (parameters reading becomes conditional), Marc Betoule </p>
; @history <p>08/02/2007: Propagate WRITEMAP procedure improvements, Marc Betoule </p>
; @history <p>25/03/2007: WRITEMAP receives its data by pointer, Marc Betoule </p>
; @history <p>12/07/2007: Seed is an input parameter and is set at psm_main, Maude Le Jeune </p>
; @history <p>17/10/2007: Changed to and for IDL 5.5 compatibility, Sam Leach </p>
; @history <p>20/01/2009: Complete revision for PSM 1.7, Jacques Delabrouille </p>
; @history <p>05/01/2009: Put back nongaussian_fnl map after revision, Jacques Delabrouille </p>
; @history <p>20/04/2009: Added call of visu_map to display/save maps, Guillaume Castex </p>
; @history <p>20/04/2009: Added lensed_sb model, Guillaume Castex </p>
;-

PRO CMB_PIPE, debug=debug

COMMON psm, param
COMMON sky, model

  ;; error handling
  ;;----------------
  IF PSM_DEBUG_VALUE() EQ 0 THEN BEGIN
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PSM_INFO_MESSAGE, 'An error was encountered while computing CMB map.'
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF 

  ;; Definitions and initialisation
  ;;--------------------------------
  runp = param.run
  SET_PSM_SEEDCHAIN, 'cmb-pipe'
  cmbdir = GETPSMDIR('cmb')
  skynside = SKY_NSIDE()

  ;; Filenames for the theory alm and map 
  ;;--------------------------------------
  cmb_unlensedalm_file = cmbdir + param.names.cmb_unlensed_alm_file
  ;; cmb_unlensedmap_file = cmbdir + param.names.cmb_unlensed_map_file
  cmb_lensedalm_file = cmbdir + param.names.cmb_lensed_alm_file
  ;; cmb_lensedmap_file = cmbdir + param.names.cmb_lensed_map_file
  potential_alm_file = cmbdir + param.names.potential_alm_file 
  potential_map_file = cmbdir + param.names.potential_map_file

  ;; Process the appropriate cmb model
  ;;-----------------------------------
  CASE CMB_MODEL() OF
     'gaussian': BEGIN
        IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING GAUSSIAN CMB"
        GAUSSIAN_CMB, cmb
     END
     'nongaussian_fnl': BEGIN
        IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING NON-GAUSSIAN CMB"
        NONGAUSSIAN_CMB, cmb
     END
     'prediction':BEGIN
        IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CMB PREDICTION"
        PREDICTED_CMB, cmb
     END
     'no_cmb': BEGIN
     END
     ELSE: MESSAGE, 'CMB model not allowed (select among [gaussian, nongaussian_fnl, prediction, no_cmb])'
  ENDCASE

  ;; Lens the CMB map if necessary
  ;;-------------------------------
  IF model.cmb.cmb_lensing EQ 'ilens' THEN BEGIN

     ;; Call ilens to get lensed cmb map
     ;;----------------------------------
     tmp_cmb_map_file = PSM_TMPFILENAME(extension='.fits')
     cmb_map_file = cmbdir + param.names.cmb_lensed_map_file
     fitsfilelmax = FITSFILE_LMAX(cmb_unlensedalm_file)
     usenside = skynside
     IF fitsfilelmax GE 3L*skynside THEN BEGIN
        PSM_INFO_MESSAGE, 'Warning, nside of output lensed map is too low'
        WHILE fitsfilelmax GE 3L*usenside DO usenside = usenside*2L
     ENDIF 
     tmpcmbalmfile = PSM_TMPFILENAME(extension='.fits')
     CHANGE_ALM_RESOLUTION, cmb_unlensedalm_file, tmpcmbalmfile, beam_out=0
     ILENS, tmpcmbalmfile, potential_alm_file, tmp_cmb_map_file, nside=usenside, polar=SKY_POLARISED(), paramfile=GETPSMDIR('cmb')+'ilens_param.ini'
     FILE_DELETE, tmpcmbalmfile
     READ_FITS_MAP, tmp_cmb_map_file, map

     ;; Compute the alm up to lmax and write them
     ;;-------------------------------------------
     PSM_MAP2ALM, TEMPORARY(map), alm_cmb_cut_resol0, lmax=SKY_LMAX()
     CHANGE_ALM_RESOLUTION, TEMPORARY(alm_cmb_cut_resol0), alm_cmb_cut, beam_in=0, beam_out=SKY_RESOLUTION()
     CMB_ALM_HEADER, almhdr, id=cmb.id
     SXADDPAR, almhdr, 'PSM_LMAX', SKY_LMAX()
     almhdr1 = almhdr & SXADDPAR, almhdr1, 'ALM_FLD', 'T'
     IF ISPOLARISED(alm_cmb_cut,/alm) THEN BEGIN
        almhdr2 = almhdr & SXADDPAR, almhdr2, 'ALM_FLD', 'E'
        almhdr3 = almhdr & SXADDPAR, almhdr3, 'ALM_FLD', 'B'
        WRITE_ALM, alm_cmb_cut, cmb_lensedalm_file, beam=SKY_RESOLUTION(), coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1, xhdr2=almhdr2, xhdr3=almhdr3
     ENDIF ELSE BEGIN
        WRITE_ALM, alm_cmb_cut, cmb_lensedalm_file, beam=SKY_RESOLUTION(), coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1
     ENDELSE

     ;; Plot the theoretical Cl and the empirical Cl if necessary
     ;;-----------------------------------------------------
     PSM_DEBUG_MESSAGE, 'Do Cl plots', routine='CMB_PIPE', infolimit=1, stoplimit=2 

     ;; Do the inverse Fourier transform to get the CMB map
     ;;-----------------------------------------------------
     PSM_ALM2MAP, TEMPORARY(alm_cmb_cut), map, lmax=lmax, hpx_nside=skynside

     ;; OBSOLETE, replaced by SHT and SHT^-1 =====================
     ;; PSM_DEBUG_MESSAGE, 'Check ordering of input map', infolimit=2, stoplimit=3
     ;; IF usenside NE skynside THEN BEGIN
     ;;    UD_GRADE, TEMPORARY(map), mapatskynside, nside_out=skyside, order_in='RING', order_out='RING'
     ;;    map = TEMPORARY(mapatskynside)
     ;; ENDIF
     ;; ==========================================================
     FILE_DELETE, tmp_cmb_map_file
     
  ENDIF ELSE BEGIN
     cmb_map_file = cmbdir + param.names.cmb_map_file
     PSM_ALM2MAP, *cmb.e1.ampl.value, map, hpx_nside=SKY_NSIDE(), lmax=SKY_LMAX()
     ;; note that the reference CMB input is redefined from alm to map
     ;; a few lines below
  ENDELSE

  ;; Write the CMB map
  ;;-------------------
  lmax = SKY_LMAX()
  CMB_MAP_HEADER, hdr, id=cmb.id
  SXADDPAR, hdr, 'PSM_LMAX', lmax
  IF ISPOLARISED(map, /map) THEN $
     WRITE_HPXMAP, cmb_map_file, tqustokes=map, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G' $
  ELSE $
     WRITE_HPXMAP, cmb_map_file, tstokes=map, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G'

  ;; Re-define the reference CMB input in the CMB structure
  ;;--------------------------------------------------------
  *cmb.e1.ampl.value = cmb_map_file
  
  ;; Visualise and save the CMB component
  ;;--------------------------------------
  IF cmb_model() NE 'no_cmb' THEN BEGIN
     IF PSM_VISU() GT 0 THEN BEGIN
        VISU_COMP, 'cmb'        
     ENDIF
     SAVE_COMP, cmb
     PRTSTRUCT, cmb, file=GETPSMDIR('cmb')+'cmb.txt', name='CMB'
     DESTROY_DATA, cmb
  ENDIF

END
