; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Guillaume Castex
;   Jacques Delabrouille
;   Fabrice Dodu
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GENERATE_X_LM
;
; PURPOSE:
;	Yet another code to generate random harmonic coefficients a_lm
;
; CATEGORY:
;	Spherical transforms; Random draws.
;
; CALLING SEQUENCE:
;	GENERATE_X_LM, Ell, Xl, Xlm, my_seed=, write_fits_file=
;
; INPUTS:
;	Ell:			An array of values of l, from 0 to lmax. Should contain
;				all l values between 0 and lmax (present in call only
;				for historical reasons.
;	Xl:			An array of power spectrum values (e.g. CMB Cl).
;
; KEYWORD PARAMETERS:
;	MY_SEED:		Input seed for random number generator. If not
;				set, then the code calls the GETSEED function to generate a seed.
;	WRITE_FITS_FILE:	Set this keyword to a valid fits file
;				name to save the output coefficients in a file
;
; OUTPUTS:
;	optional, see below
;
; OPTIONAL OUTPUTS:
;	Xlm:			The array of output harmonic coefficients
;
; SIDE EFFECTS:
;	A file is written on disk.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	21/01/2007:	Fabrice Dodu, First version.
; 	29/10/2009:	Guillaume Castex, Now calls ALM_INDEX function.
; 	09/09/2011:	Jacques Delabrouille, some cleanup, added test for validity of input ell vector.
;-

PRO GENERATE_X_LM, ell, xl, xlm, my_seed=my_seed, write_fits_file = write_fits_file

nb_ell = N_ELEMENTS(ell)
l_max = LONG(MAX(ell))
testbad = 0
IF l_max NE nb_ell-1 THEN testbad=1 ELSE testbad = (MAX(ABS(ell-LINDGEN(l_max+1)))<1)
IF testbad NE 0 THEN MESSAGE, 'Input ell array should be equal to LINDGEN(l_max+1)'

nb_xlm = ((l_max+1L)*(l_max+2L))/2L

xlm = FLTARR(nb_xlm,2)

IF NOT KEYWORD_SET(my_seed) THEN my_seed = GETSEED()
randomnumbers = RANDOMN(my_seed,2L*nb_xlm)

xlm(0:nb_xlm-1,0) = randomnumbers(0:nb_xlm-1)
xlm(0:nb_xlm-1,1) = randomnumbers(nb_xlm:*)

use_ell = LINDGEN(l_max+1)
use_xl = FLTARR(l_max+1)

use_xl(ell) = xl

xlm(0,0) = SQRT(use_xl(0))*xlm(0,0)
xlm(0,1) = 0.

FOR i_ell = 1L, nb_ell-1 DO BEGIN & $
    start_ind = ((i_ell+1l)*(i_ell))/2L & $
    end_ind = start_ind + (i_ell) & $
    xlm(start_ind,0) = SQRT(use_xl(i_ell))*xlm(start_ind,0) & $
    xlm(start_ind,1) = 0. & $
    xlm(start_ind+1:end_ind,0) = SQRT(use_xl(i_ell)/2.)*xlm(start_ind+1:end_ind,0) & $
    xlm(start_ind+1:end_ind,1) = SQRT(use_xl(i_ell)/2.)*xlm(start_ind+1:end_ind,1) & $
ENDFOR

IF  KEYWORD_SET(write_fits_file) THEN BEGIN
   index = ALM_INDEX(0,nb_ell-1) 
   ALM2FITS ,index, xlm, write_fits_file, HDR = hdr, XHDR = xhdr
ENDIF

END

