; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Marc Betoule
;   Jacques Delabrouille
;   Maude Le Jeune
;   Matthieu Roman
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	NONGAUSSIAN_CMB
;
; PURPOSE:
;	Generate a non Gaussian CMB in a PSM run.
;
; CATEGORY:
;	Sky model (CMB)
;
; CALLING SEQUENCE:
;	NONGAUSSIAN_CMB, Cmb
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	Cmb
;
; COMMON BLOCKS:
;	PSM:	param, parameters of the psm run
;	SKY:	model, parameters of the sky model
;
; SIDE EFFECTS:
;	Files are written on disk
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Called only from within a PSM run (where common blocks are defined)
;
; MODIFICATION HISTORY:
; 	22/02/2007:	Marc Betoule, First version.
; 	02/02/2009:	Jacques Delabrouille, General reorganisation.
; 	24/11/2011:	Jacques Delabrouille, Major rewriting.
; 	28/11/2011:	Jacques Delabrouille, yet some reorganisation.
; 	11/06/2012:	Matthieu Roman, modified for lensing with ilens.
; 	13/06/2012:	Jacques Delabrouille, general debugging for
; 			new handling of the cosmology.
; 	18/09/2012:	JD, small bug fix for non-gaussian CMB
; 			when computed in double precision.
;-
PRO NONGAUSSIAN_CMB, cmb, test=test;, split_cmb=split_cmb

  COMMON psm, param
  COMMON sky, model

  ;; IF model.cmb.cmb_lensing EQ 'ilens' THEN MESSAGE, 'Lensing with ILENS not implemented yet for non gaussian CMB maps'
  ;; IF model.cmb.cmb_lensing EQ 'cl' THEN MESSAGE, 'Lensing not implemented yet for non gaussian CMB maps'

  cmbdir = GETPSMDIR('cmb')

  ;; Filenames for the theory alm and map 
  ;;--------------------------------------
  cmb_unlensedalm_file = cmbdir + param.names.cmb_unlensed_alm_file
  cmb_unlensedmap_file = cmbdir + param.names.cmb_unlensed_map_file
  cmb_lensedalm_file = cmbdir + param.names.cmb_lensed_alm_file
  cmb_lensedmap_file = cmbdir + param.names.cmb_lensed_map_file
  potential_alm_file = cmbdir + param.names.potential_alm_file 
  potential_map_file = cmbdir + param.names.potential_map_file

  ;; Read unlensed and lensed Cl
  ;;-----------------------------
  cmb_lensedcl_file = cmbdir + 'cmb' + '_lensed_cl.fits' 
  cmb_unlensedcl_file = cmbdir + 'cmb' + '_unlensed_cl.fits' 
  PSM_DEBUG_MESSAGE, 'Fix problem with the format of the fits file', routine='NONGAUSSIAN_CMB', infolimit=3, stoplimit=3
  READ_CL, cmb_lensedcl_file, lensedcl, hdr=hdrlensed
  READ_CL, cmb_unlensedcl_file, unlensedcl, hdr=hdrunlensed

  ;; Some parameters
  ;;-----------------
  gp = param.global
  cmbp = model.cmb
  pixp = SKY_PIX()
  names = param.names
  verbosity = PSM_VERBOSITY()  
  dbl = PRECISION_ISDOUBLE()
  polar = SKY_POLARISED()
  CASE cmbp.cmb_lensing OF
     'ilens': lmax = SKY_LMAX() + 300
     'cl': lmax = SKY_LMAX()
     'none': lmax = SKY_LMAX()
  ENDCASE
  skynside = SKY_NSIDE()

  ;; Test case: return after checking that parameters are accessible
  ;;-----------------------------------------------------------------
  IF KEYWORD_SET(test) THEN RETURN
  
  ;; Create cmb component structure
  ;;-----------------------------------
  PSM_ID_KEYGEN, cmb_id
  cmb = COMPONENT_STRUCT('cmb',name=param.names.cmb_basename,id=cmb_id)
  cmb.polarised = (model.global.sky_fields NE 'T')
  cmb.e1.ampl.unit = model.units.cmb
  *cmb.e1.numin.value = 0.
  *cmb.e1.numax.value = FLOAT('Inf')
  CASE model.cmb.cmb_lensing OF
     'none': BEGIN
        *cmb.e1.ampl.value = cmb_unlensedalm_file
        cmb_map_file = cmb_unlensedmap_file
     END
     'cl': BEGIN
        *cmb.e1.ampl.value = cmb_lensedalm_file
        cmb_map_file = cmb_lensedmap_file
     END
     'ilens': BEGIN
        *cmb.e1.ampl.value = cmb_lensedalm_file
        cmb_map_file = cmb_lensedmap_file
     END
  ENDCASE
  
  ;; Select the cl to be used to generate the CMB map and the name of
  ;; the output cmb alm 
  ;;------------------------------------------------------------------

  CASE model.cmb.cmb_lensing OF
     'cl': BEGIN
        cl = lensedcl
        alm_writefile = cmb_lensedalm_file
     END
     ELSE: BEGIN
        cl = unlensedcl
        alm_writefile = cmb_unlensedalm_file
     END
  ENDCASE

  ;; Check and get data 
  ;;--------------------
  
    ;; Select NG simulation
    mapnumber = PSM_STRING_TO_LENGTH(NUMBER2STRING(cmbp.ng_map_number),4,/strict,/leading,character='0')
    CASE cmbp.ng_simuset OF
       'elsner3500': BEGIN
          cmbdata = GETPSMDIR('psmdata')+'ancillary/simulations/cmb/nongaussian-elsner/lmax3500/'
          alm_lin = cmbdata+'alm_l_'+mapnumber+'.fits'
          alm_nonlin = cmbdata+'alm_nl_'+mapnumber+'.fits'
          simus = 'elsner'
          elsner_clfile = cmbdata+'cl_scalCls.dat'
       END
       'elsner1024': BEGIN
          cmbdata = GETPSMDIR('psmdata')+'ancillary/simulations/cmb/nongaussian-elsner/lmax1024v3/'
          alm_lin = cmbdata+'alm_l_'+mapnumber+'_v3.fits'
          alm_nonlin = cmbdata+'alm_nl_'+mapnumber+'_v3.fits'
          simus = 'elsner'
          elsner_clfile = GETPSMDIR('psmdata')+'ancillary/simulations/cmb/nongaussian-elsner/lmax3500/cl_scalCls.dat'
       END
    ENDCASE
    
    ;; Get the readme if needed
    ok = TEST_PSM_DATA(cmbdata+'readme.txt', /get)

    ;; Get the theoretical cl used for generating the map, if needed
    ;; cl_wmap7_bao_h0.txt and cl_scalCls.dat agree to better than 5e-3
    ;; (and actually 5e-4 almost everywhere)
    ;; so we use cl_scalCls.dat, which extends to lmax=8000
    ;; ok = TEST_PSM_DATA(cmbdata+'cl_wmap7_bao_h0.txt', /get)
    ok = TEST_PSM_DATA(elsner_clfile, /get)
    IF NOT ok THEN  MESSAGE,'ERROR : NON-GAUSSIAN CL POWER SPECTRUM NOT AVAILABLE'        
    IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, 'READING INPUT CL FILE'

    ;; check that the theoretical cl used in the NG simulations
    ;; matches the cl computed for the PSM cosmology
    cambclfile = FIX_SEPARATOR(GETPSMDIR('cosmo'))+'camb/camb_scalCls.txt'
    ok = FILE_TEST(cambclfile)
    IF ok THEN BEGIN
       READ_CAMB_CL, cambclfile, psmcl, /polar, cmb_outputscale=1
       ;; Convert units
       fields2conv = ['T','E','B']
       units_new = CMB_UNITS()
       FOR i=0, N_ELEMENTS(fields2conv)-1 DO CL_UPDATE_UNITS, psmcl, fields2conv[i], units_new 
    ENDIF
       READ_CAMB_CL, elsner_clfile, ngcl, /polar, cmb_outputscale=1
       ngcl.c_1_1 = ngcl.c_1_1*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.c_1_2 = ngcl.c_1_2*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.c_2_2 = ngcl.c_2_2*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.c_1_3 = ngcl.c_1_3*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.c_2_3 = ngcl.c_2_3*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.c_3_3 = ngcl.c_3_3*CONVERSION_FACTOR('K/KCMB',CMB_UNITS())^2
       ngcl.units[0:2] = CMB_UNITS()

    ok = TEST_PSM_DATA(alm_lin,/get)
    IF NOT ok THEN  MESSAGE,'ERROR : NON-GAUSSIANITY DATA NOT AVAILABLE'
    
    ok = TEST_PSM_DATA(alm_nonlin,/get)
    IF NOT ok THEN  MESSAGE,'ERROR : NON-GAUSSIANITY DATA NOT AVAILABLE'

  ;; Make figures if needed
  ;;------------------------
  IF param.run.visu GE 2 THEN BEGIN
     MAKE_NG_FIGURES, cmbp.ng_map_number, SKY_RESOLUTION(), lmax=SKY_LMAX(), nside=SKY_NSIDE(), output=param.run.output_visu
  ENDIF
  
  ;; Generate the CMB, depending on the simu set
  ;;---------------------------------------------
  CASE simus OF
     
     ;; Elsner and Wandelt
     ;;--------------------
     'elsner': BEGIN

        CASE cmbp.ng_simuset OF
           'elsner3500': lmaxng = 3500 < MAX(psmcl.l)
           'elsner1024': lmaxng = 1024 < MAX(psmcl.l)
        ENDCASE

        CASE model.cmb.cmb_lensing OF 
           'none': BEGIN
              uselmaxng = lmaxng < SKY_LMAX()   
              uselmax = MAX(psmcl.l) < SKY_LMAX()   
           END
           'ilens': BEGIN
              uselmaxng = lmaxng
              uselmax = MAX(psmcl.l)   
           END
           'cl': MESSAGE, 'Case not implemented'
        ENDCASE
        ni = LMAX2NALM(uselmaxng)
        nalm = LMAX2NALM(uselmax)

        ;;====================================================================================
        ;; Scalar modes, linear part
        ;;====================================================================================

        ;; IF polar THEN signal = 'TEB' ELSE signal='T'
        signal = 'TEB'
        READ_ALM, alm_lin, almg, signal=signal;, lmax=lmax     
        almg = almg * CONVERSION_FACTOR('K/KCMB',CMB_UNITS()) 

        ;; Readjust spectrum ??
        ;;----------------------
        IF cmbp.readjust_ng_spectrum EQ 'yes' THEN BEGIN
           index = ALM_INDEX(0, lmaxng)
           INDEX2L_ONLY, TEMPORARY(index), lll
           dopolar = 1B
           IF dopolar THEN BEGIN
              FOR l=2, lmaxng DO BEGIN
                 matrix = [[ngcl.c_1_1[l],ngcl.c_1_2[l]],[ngcl.c_1_2[l],ngcl.c_2_2[l]]]
                 sqrtngcl = MATRIX_SQRT(matrix, status, /check)
                 mat = [[psmcl.c_1_1[l],psmcl.c_1_2[l]],[psmcl.c_1_2[l],psmcl.c_2_2[l]]]
                 sqrtpsmcl = MATRIX_SQRT(mat, status, /check)
                 mat = sqrtpsmcl##INVERT(sqrtngcl)
                 wh = WHERE(lll EQ l, nwh)
                 newt = mat[0,0]*almg[wh,*,0] + mat[1,0]*almg[wh,*,1]
                 newe = mat[0,1]*almg[wh,*,0] + mat[1,1]*almg[wh,*,1]
                 almg[wh,*,0] = newt
                 almg[wh,*,1] = newe
              ENDFOR
           ENDIF ELSE BEGIN
              FOR l=2, lmaxng DO BEGIN
                 ratio = SQRT(psmcl.c_1_1[l]/ngcl.c_1_1[l])
                 wh = WHERE(lll EQ l, nwh)
                 IF nwh NE 0 THEN almg[wh,*] = almg[wh,*] * ratio
              ENDFOR
           ENDELSE
        ENDIF

        ;; lmax bigger than lmaxng
        ;;-------------------------
        ;; IF lmax GT lmaxng THEN BEGIN
        ;;    dopolar = 1B
        ;;    IF dopolar THEN BEGIN
        ;;       cl = CL_STRUCTURE(nfields=3, fieldnames=['T','E','B'], units=CMB_UNITS(), lmin=0, lmax=lmax, double=PRECISION_ISDOUBLE())
        ;;       wht = WHERE(psmcl.field EQ 'T') & wht=wht[0]+1
        ;;       whe = WHERE(psmcl.field EQ 'E') & whe=whe[0]+1
        ;;       whb = WHERE(psmcl.field EQ 'B') & whb=whb[0]+1
        ;;       ok = EXECUTE('cl.c_1_1 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(wht)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;       ok = EXECUTE('cl.c_1_2 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(whe)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;       ok = EXECUTE('cl.c_1_3 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(whb)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;       ok = EXECUTE('cl.c_2_2 = psmcl.c_'+NUMBER2STRING(whe)+'_'+NUMBER2STRING(whe)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;       ok = EXECUTE('cl.c_2_3 = psmcl.c_'+NUMBER2STRING(whe)+'_'+NUMBER2STRING(whb)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;       ok = EXECUTE('cl.c_3_3 = psmcl.c_'+NUMBER2STRING(whb)+'_'+NUMBER2STRING(whb)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;    ENDIF ELSE BEGIN
        ;;       cl = CL_STRUCTURE(nfields=1, fieldnames=['T'], units=CMB_UNITS(), lmin=0, lmax=lmax, double=PRECISION_ISDOUBLE())
        ;;       wht = WHERE(psmcl.field EQ 'T') & wht=wht[0]+1
        ;;       ok = EXECUTE('cl.c_1_1 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(wht)+'[0:lmax]') & IF NOT ok THEN STOP
        ;;    ENDELSE

        ;; lmax smaller than or equal to lmaxng
        ;;--------------------------------------
        ;; ENDIF ELSE BEGIN
        ;;    PSM_DEBUG_MESSAGE, 'Check this', infolimit=2, stoplimi=3, routine='NONGAUSSIAN_CMB'
        ;;    ;; ni = LMAX2NALM(lmax)
        ;;    ni = LMAX2NALM(lmaxng)
        ;;    almg = almg[0:ni-1,*,*]
        ;; ENDELSE

        CASE model.cmb.cmb_lensing OF 
           'none': BEGIN
              ;; Complete the gaussian linear alm with random realisation at high l, 
              ;; or restrict it's l range, so that the maximum l value is SKY_LMAX()
              PSM_CL2ALM, psmcl, alm, lmax=uselmax, seed=GETSEED(), double=PRECISION_ISDOUBLE()
              alm[0:ni-1,*,0:2] = almg[0:ni-1,*,*] 
              almg = alm[0:ni-1,*,0:1]
           END
           'ilens': BEGIN
              ccl = CL_STRUCTURE(nfields=3, fieldnames=['T','E','P'], units=[CMB_UNITS(),CMB_UNITS(),'unitless'], lmin=0, lmax=uselmax, /double)
              wht = WHERE(psmcl.field EQ 'T') & wht=wht[0]+1
              whe = WHERE(psmcl.field EQ 'E') & whe=whe[0]+1
              whp = WHERE(psmcl.field EQ 'P') & whp=whp[0]+1
              ok = EXECUTE('ccl.c_1_1 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(wht)+'[0:uselmax]') & IF NOT ok THEN STOP
              ok = EXECUTE('ccl.c_1_2 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(whe)+'[0:uselmax]') & IF NOT ok THEN STOP
              ok = EXECUTE('ccl.c_2_2 = psmcl.c_'+NUMBER2STRING(whe)+'_'+NUMBER2STRING(whe)+'[0:uselmax]') & IF NOT ok THEN STOP
              ok = EXECUTE('ccl.c_1_3 = psmcl.c_'+NUMBER2STRING(wht)+'_'+NUMBER2STRING(whp)+'[0:uselmax]') & IF NOT ok THEN STOP
              ok = EXECUTE('ccl.c_3_3 = psmcl.c_'+NUMBER2STRING(whp)+'_'+NUMBER2STRING(whp)+'[0:uselmax]') & IF NOT ok THEN STOP
              tags = ccl.field
              
              dopolar = 1B
              IF dopolar THEN BEGIN
                 nlobs = CL_STRUCTURE(nfields=2, fieldnames=['T','E'], units=CMB_UNITS(), lmin=0, lmax=lmaxl, double=PRECISION_ISDOUBLE())
                 PSM_CL2ALM, ccl, alm, lmin=0, lmax=uselmax, seed=GETSEED(), almobs=almg[*,*,0:1], nlobs=nlobs, double=double
              ENDIF ELSE BEGIN
                 STOP
              ENDELSE

              ;; Define and write the potential alm
              ;;------------------------------------
              whp = WHERE(tags EQ 'P', nwh)
              ;; nalm = N_ELEMENTS(alm[*,0,0]) ;; already defined above
              almpot = DBLARR(nalm,2)
              IF nwh EQ 1 THEN almpot[*,*] = alm[*,*,whp[0]]
              
              potential_alm_file = GETPSMDIR('cmb')+'potential_alm.fits'
              CMB_ALM_HEADER, almhdrp, id=cmb_id
              SXADDPAR, almhdrp, 'BEAMSIZE', 0.
              almhdr1p = almhdrp & SXADDPAR, almhdr1p, 'ALM_FLD', 'P'
              
              WRITE_ALM, almpot, potential_alm_file, beam=0., coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1p
              PSM_ALM2MAP, almpot, map_pot, hpx_nside=skynside, double=double
              CMB_MAP_HEADER, hdr, id=cmb.id
              SXADDPAR, hdr, 'PSM_LMAX', lmaxl
              IF ISPOLARISED(map_pot, /map) THEN $
                 WRITE_HPXMAP, potential_map_file, tqustokes=map_pot, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G' $
              ELSE $
                 WRITE_HPXMAP, potential_map_file, tstokes=map_pot, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G'
              
              ;; Define the CMB alm
              ;;--------------------
              ;;alm[0:ni-1,*,*] = almg[*,*,*] 
              almg = alm[0:ni-1,*,0:1]
           END
           'cl': BEGIN
              MESSAGE, 'Case not implemented'
           END
        ENDCASE

        ;; Write scalar gaussian CMB alm (no beam)
        ;;-----------------------------------------
        alm_scalar_linear_file = GETPSMDIR('cmb')+'cmb_alm_scalar_linear.fits'
        CMB_ALM_HEADER, almhdr, id=cmb_id
        SXADDPAR, almhdr, 'PSM_LMAX', uselmaxng
        SXADDPAR, almhdr, 'BEAMSIZE', 0.
        almhdr1 = almhdr & SXADDPAR, almhdr1, 'ALM_FLD', 'T'
        almhdr2 = almhdr & SXADDPAR, almhdr2, 'ALM_FLD', 'E'
        WRITE_ALM, almg, alm_scalar_linear_file, beam=0., coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1, xhdr2=almhdr2
        DELVARX, almg

        ;;====================================================================================
        ;; Scalar modes, nonlinear part
        ;;====================================================================================
        READ_ALM, alm_nonlin, almng, signal=signal    ;, lmax=lmax
        IF NOT PRECISION_ISDOUBLE() THEN almng = CONVERSION_FACTOR('K/KCMB',CMB_UNITS())*FLOAT(TEMPORARY(almng)) $
        ELSE almng = CONVERSION_FACTOR('K/KCMB',CMB_UNITS(),/double)*(TEMPORARY(almng))
        alm_nonlinear_file = GETPSMDIR('cmb')+'cmb_alm_scalar_nonlinear.fits'
        ni = LMAX2NALM(uselmaxng)
        CMB_ALM_HEADER, almhdr, id=cmb_id
        SXADDPAR, almhdr, 'PSM_LMAX', uselmaxng
        SXADDPAR, almhdr, 'BEAMSIZE', 0.
        almhdr1 = almhdr & SXADDPAR, almhdr1, 'ALM_FLD', 'T'
        almhdr2 = almhdr & SXADDPAR, almhdr2, 'ALM_FLD', 'E'
        almhdr3 = almhdr & SXADDPAR, almhdr3, 'ALM_FLD', 'B'
        WRITE_ALM, almng[0:ni-1,*,*], alm_nonlinear_file, beam=0., coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1, xhdr2=almhdr2, xhdr3=almhdr3
        
        ;; Add non-gaussian correction to total alm
        ;;------------------------------------------
        alm[0:ni-1,*,0:2] = alm[0:ni-1,*,0:2] + (cmbp.f_nl*TEMPORARY(almng[0:ni-1,*,*]))

        
        ;;====================================================================================
        ;; Tensor modes, linear part
        ;;====================================================================================
        IF model.cosmo.r NE 0 THEN BEGIN
           READ_CAMB_CL, FIX_SEPARATOR(GETPSMDIR('cosmo'))+'camb/camb_tensCls.txt', tcl, /polar, cmb_outputscale=1
           PSM_CL2ALM, tcl, talm, seed=GETSEED(), double=PRECISION_ISDOUBLE()
           ;; convert units to CMB units
           IF NOT PRECISION_ISDOUBLE() THEN BEGIN
              talm[*,*,0] = CONVERSION_FACTOR(tcl.units[0],CMB_UNITS())*FLOAT(talm[*,*,0])
              talm[*,*,1] = CONVERSION_FACTOR(tcl.units[1],CMB_UNITS())*FLOAT(talm[*,*,1])
              talm[*,*,2] = CONVERSION_FACTOR(tcl.units[2],CMB_UNITS())*FLOAT(talm[*,*,2])
           ENDIF ELSE BEGIN
              talm[*,*,0] = CONVERSION_FACTOR(tcl.units[0],CMB_UNITS(),/double)*talm[*,*,0]
              talm[*,*,1] = CONVERSION_FACTOR(tcl.units[1],CMB_UNITS(),/double)*talm[*,*,1]
              talm[*,*,2] = CONVERSION_FACTOR(tcl.units[2],CMB_UNITS(),/double)*talm[*,*,2]
           ENDELSE
           alm_tensor_file = GETPSMDIR('cmb')+'cmb_alm_tensor_linear.fits'

           IF model.cosmo.r NE 0 THEN BEGIN
              ;; Write tensor alm
              ;;------------------
              CMB_ALM_HEADER, almhdr, id=cmb_id
              SXADDPAR, almhdr, 'PSM_LMAX', (uselmaxng < MAX(tcl.l))
              SXADDPAR, almhdr, 'BEAMSIZE', 0.
              almhdr1 = almhdr & SXADDPAR, almhdr1, 'ALM_FLD', 'T'
              almhdr2 = almhdr & SXADDPAR, almhdr2, 'ALM_FLD', 'E'
              almhdr3 = almhdr & SXADDPAR, almhdr3, 'ALM_FLD', 'B'
              WRITE_ALM, talm, alm_tensor_file, beam=0., coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1, xhdr2=almhdr2, xhdr3=almhdr3
              nni = LMAX2NALM(uselmaxng < MAX(tcl.l))
              ;; Add tensor modes to total alm
              ;;-------------------------------
              alm[0:nni-1,*,0:2] = alm[0:nni-1,*,0:2] + talm[0:nni-1,*,0:2]
           ENDIF
        ENDIF

        ;;====================================================================================
        ;; Put everything into final format
        ;;====================================================================================

        ;; Restrict alm to useful part
        ;;-----------------------------
        IF polar THEN alm = alm[0:nalm-1,*,0:2] ELSE alm = REFORM(alm[0:nalm-1,*,0])

        ;; Change resolution to match the PSM resolution
        ;;-----------------------------------------------
        lmaxalm = NALM2LMAX(nalm)
        CHANGE_ALM_RESOLUTION, TEMPORARY(alm), alm, beam_in=0, beam_out=SKY_RESOLUTION(), lmax=lmaxalm
        
        ;; Get pixel window function and correct if needed
        ;;-------------------------------------------------          
        IF model.global.sky_pixwindow NE 0 THEN BEGIN
           
           pixwinl_hpx = HEALPIXWINDOW(SKY_NSIDE())
           lmaxinfo = ((N_ELEMENTS(pixwinl_hpx[*,0])-1) < lmaxalm)
           pixwinl = FLTARR(lmaxinfo+1,2) & IF PRECISION_ISDOUBLE() THEN pixwinl = DOUBLE(pixwinl)
           pixwinl[0:lmaxinfo,0] = pixwinl_hpx[0:lmaxinfo,0]
           pixwinl[0:lmaxinfo,1] = pixwinl_hpx[0:lmaxinfo,1]
           factor_T = pixwinl[*,0]
           factor_T = pixwinl[*,1]
           ind = ALM_INDEX(0,lmaxalm)
           INDEX2L_ONLY, ind, lll
           
           IF polar THEN BEGIN
              alm[*,0,0] = alm[*,0,0] * factor_T[lll]
              alm[*,1,0] = alm[*,1,0] * factor_T[lll]           
              factor_P = pixwinl[*,1]
              alm[*,0,1] = alm[*,0,1] * factor_P[lll]
              alm[*,1,1] = alm[*,1,1] * factor_P[lll]           
              alm[*,0,2] = alm[*,0,2] * factor_P[lll]
              alm[*,1,2] = alm[*,1,2] * factor_P[TEMPORARY(lll)]           
           ENDIF ELSE BEGIN
              
              alm[*,0] = alm[*,0] * factor_T[lll]
              alm[*,1] = alm[*,1] * factor_T[TEMPORARY(lll)]           
           ENDELSE

        ENDIF
       
        ;; Write unlensed alm file (total, with beam and pwf)
        ;;----------------------------------------------------
        CMB_ALM_HEADER, almhdr, id=cmb_id
        SXADDPAR, almhdr, 'PSM_LMAX', lmaxalm
        ;;SXADDPAR, almhdr, 'BEAMSIZE', 0.
        almhdr1 = almhdr & SXADDPAR, almhdr1, 'ALM_FLD', 'T'
        IF polar THEN BEGIN
           almhdr2 = almhdr & SXADDPAR, almhdr2, 'ALM_FLD', 'E'
           almhdr3 = almhdr & SXADDPAR, almhdr3, 'ALM_FLD', 'B'
           WRITE_ALM, alm, cmb_unlensedalm_file, beam=SKY_RESOLUTION(), coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1, xhdr2=almhdr2, xhdr3=almhdr3
        ENDIF ELSE BEGIN
           WRITE_ALM, alm, cmb_unlensedalm_file, beam=SKY_RESOLUTION(), coordsys='G', units=CMB_UNITS(), xhdr1=almhdr1
        ENDELSE

        ;; Compute and write unlensed map file (total, with beam and pwf)
        ;;----------------------------------------------------------------
        PSM_ALM2MAP, TEMPORARY(alm), map, hpx_nside=skynside, double=double
        CMB_MAP_HEADER, hdr, id=cmb.id
        SXADDPAR, hdr, 'PSM_LMAX', lmaxalm
        IF ISPOLARISED(map, /map) THEN $
           WRITE_HPXMAP, cmb_unlensedmap_file, tqustokes=map, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G' $
        ELSE $
           WRITE_HPXMAP, cmb_unlensedmap_file, tstokes=map, xhdr=hdr, ordering='RING', units=CMB_UNITS(), coordsys='G'
        
     END
     
  ENDCASE
  
  ;; Read and write the Cl
  ;;-----------------------
  ;; cmb_cl_file = outdir + names.cmb_basename + '_cl.fits' ; Name of the cl file to be written
  ;; hbase = PSM_BASE_HDR(datatype='comp',dataform='cl', /setval)
  ;; hcomp = PSM_COMP_HDR(compname='cmb',compid=cmb_id)
  ;; MK_PSMHDR, xhdr, hbase, hcomp 
  ;; WRITE_CL, cl, cmb_cl_file, pxwin=0, beam=0, xhdr=xhdr
  ;; IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, 'CMB Cl file written'
   
  ;; Generate map if needed
  ;;------------------------
  ;; IF STRUPCASE(gp.precision) EQ 'DOUBLE' THEN isdouble=1 ELSE isdouble=0
  ;; IF cmbp.save_cmb_map EQ 'yes' THEN BEGIN        
  ;;    IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, 'COMPUTING THE CMB MAP'
  ;;    file_exists = FILE_TEST(cmb_map_file)
  ;;    IF file_exists THEN BEGIN
  ;;       IF runp.carefulness GE 2 THEN BEGIN
  ;;          PSM_INFO_MESSAGE, 'CMB map file already exists: STOPPING THE PSM RUN'
  ;;          PSM_INFO_MESSAGE, 'Type .c to delete existing file ('+cmb_map_file+') and continue'
  ;;          STOP
  ;;       ENDIF
  ;;       FILE_DELETE, cmb_map_file
  ;;    ENDIF
  ;;    IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN
  ;;       PSM_ALM2MAP, *cmb.e1.ampl.value, cmb_map_file, hpx_nside = pixp.nside, /polar
  ;;    ENDIF ELSE BEGIN
  ;;       PSM_ALM2MAP, *cmb.e1.ampl.value, cmb_map_file, hpx_nside = pixp.nside
  ;;    ENDELSE        
  ;;    IF !debug GE 2 THEN BEGIN
  ;;       PSM_INFO_MESSAGE, 'Implement the map header in NONGAUSSIAN_CMB.PRO ! STOPPING THE PSM RUN'
  ;;       PSM_INFO_MESSAGE, 'Type .c if you wish to continue'
  ;;       STOP
  ;;    ENDIF
  
  PSM_ADD2CITE, 'The CMB model is based on a non-Gaussian simulation of the local type generated according to the method of Elsner & Wandelt (2009).'
  PSM_ADD2BIB, '2009ApJS..184..264E'
  
END
