; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Generates a Gaussian CMB
;
; @categories cmb
;
; @param param {in}{required}{type=string} Parameters of the PSM run
; @uses READ_CL_FILE, WRITE_CAMB_PARAM_FILE, WRITE_PSM_CL (PSM) CAMB executable (CAMB)
; @author Jacques Delabrouille
; @history <p>21/01/2009: First version, Jacques Delabrouille </p>
; @history <p>05/02/2009: Added visualisation, Jacques Delabrouille </p>
; @history <p>05/02/2009: Added output keyword for view_output_cl, Guillaume Castex </p>
; @copyright Jacques Delabrouille, CNRS
;-
PRO PREDICTED_CMB, cmb

COMMON psm, param
COMMON sky, model

  gp = param.global
  cmbp = model.cmb
  names = param.names
  mapp = model.mapping
  pixp = SKY_PIX()

  outdir = GETPSMDIR('cmb')

  ;; Filename in which to write the alms and map 
  ;;---------------------------------------------
  cmb_alm_file = outdir + names.cmb_basename + '_alm.fits'
  cmb_map_file = outdir + names.cmb_basename + '_map.fits'
  
  ;; Create cmb component structure
  ;;-----------------------------------
  PSM_ID_KEYGEN, cmb_id
  cmb = COMPONENT_STRUCT('cmb',name=param.names.cmb_basename,id=cmb_id)
  cmb.polarised = (model.global.sky_fields NE 'T')
  cmb.e1.ampl.unit = model.units.cmb
  *cmb.e1.ampl.value = cmb_alm_file


;;--------------------------------------------------------------------------------------------------------------------------
;; Get C_ell values and write C_ell file
;;--------------------------------------------------------------------------------------------------------------------------

  cmb_cl_file = outdir + names.cmb_basename + '_cl.fits'     ; Name of the cl file to be written

  ;; Input Cl read from file
  ;;-------------------------
  ;; read the input c_ell file and write c_ell in the output
  ;; directory, in PSM standard format

  ;; CASE cmbp.cmb_lensing OF
  ;;    'none': cl = CMB_BESTFIT_CL(/unlensed)
  ;;    'cl': cl = CMB_BESTFIT_CL()
  ;; ENDCASE

  cl = CMB_BESTFIT_CL()
  hbase = PSM_BASE_HDR(datatype='comp',dataform='cl', /setval)
  hcomp = PSM_COMP_HDR(compname='cmb',compid=cmb_id)
  MK_PSMHDR, xhdr, hbase, hcomp 
  WRITE_CL, cl, cmb_cl_file, pxwin=0, beam=0, xhdr=xhdr
  IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, 'CMB Cl file written'

;;--------------------------------------------------------------------------------------------------------------------------
;; Generate CMB alm (and map if necessary)
;;--------------------------------------------------------------------------------------------------------------------------

  CASE pixp.pixtype OF

     'HEALPIX': BEGIN

        ;; Set lmax 
        ;;----------
        lmax = mapp.lmax
        available_lmax = MAX(cl.l)
        IF lmax GE available_lmax THEN BEGIN
           lmax = available_lmax
           MESSAGE, '******************************************'+string(10b)+$
                    'WARNING : Requested lmax larger than available CMB C_ell'+ string(10b)+$
                    'Assuming lmax = '+NUMBER2STRING(lmax)+' for CMB prediction'+string(10b) + $
                    '******************************************', /info
        ENDIF  

        ;; Get gaussian beam 
        ;;------------------- 
        bl = GAUSSBEAM(SKY_RESOLUTION(), lmax, 3)

        ;; Get pixel window function 
        ;;---------------------------
        
        IF model.global.sky_pixwindow NE 0 THEN BEGIN
           pixwinl = HEALPIXWINDOW(SKY_NSIDE())
           pixwinl = pixwinl[0:lmax,*]
        ENDIF

        ;; Generate alm 
        ;;--------------

        ell = LINDGEN(lmax+1)
        l_index = cl.l[WHERE(cl.l LE lmax)]
        tt = FLTARR(lmax+1)
        tt[ell[l_index]] = (CL_VALUE(cl,'T','T'))[l_index] * bl[l_index,0]^2
        wl = NILC5_WIENER_FILTER(ell) < 1
        zl = (1.-wl) > 0

        ;; Read and re-beam the NILC5 map
        nilc5_mapfile = !PSMROOT+'Data/ancillary/observations/WMAP/NILC-CMB-5yr/mapilc5yr.fits'
        ok = TEST_PSM_DATA(nilc5_mapfile, /get)
        globalfactor = CONVERSION_FACTOR('mK_CMB',CMB_UNITS())
        blref = PSM_GET_BL('WMAP5yr-W', lmax=lmax)
        cmbbeam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=SKY_RESOLUTION(), sizeunits='arcmin')
        refbeam = GET_BEAM_STRUCT('INSTR',instr='WMAP',version='5yr',channel='W')
        PSM_MAP2ALM, nilc5_mapfile, nilc5_alm, lmax=lmax, /nohdr
        CHANGE_ALM_RESOLUTION, nilc5_alm, alm_fix, beam_in=refbeam, beam_out=cmbbeam
        alm_fix = alm_fix * globalfactor
        ind = ALM_INDEX(0,lmax)
        INDEX2L_ONLY, ind, lll
        alm_fix[*,0] = alm_fix[*,0] * wl[lll]
        alm_fix[*,1] = alm_fix[*,1] * wl[lll]

        ;; Correct for pixel window function (nside=512 for the
        ;; WMAP NILC5 map, although this is ignored in the NILC5
        ;; analysis)
        ;;-------------------------------------------------------
        nilcpixwin = HEALPIXWINDOW(512)
        nilcpixwin = nilcpixwin[0:lmax,*]
        IF model.global.sky_pixwindow NE 0 THEN BEGIN
           factor = pixwinl[*,0]/nilcpixwin[*,0]
        ENDIF ELSE BEGIN
           factor = 1./nilcpixwin[*,0]
        ENDELSE
        alm_fix[*,0] = alm_fix[*,0] * factor[lll]
        alm_fix[*,1] = alm_fix[*,1] * factor[lll]
        
        ;; Coadd both parts
        ;;------------------
        almt = TEMPORARY(alm_fix)
        IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, 'CMB map is a prediction based on existing observations'

        IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN
           polar = 1
           ee = FLTARR(lmax+1)
           ;;bb = FLTARR(lmax+1)
           te = FLTARR(lmax+1)
           ee[ell[l_index]] = (CL_VALUE(cl,'E','E'))[l_index] *bl[l_index,1]^2
           ;;bb[ell[l_index]] = (CL_VALUE(cl,'B','B'))[l_index] *bl[l_index,2]^2
           te[ell[l_index]] = (CL_VALUE(cl,'T','E'))[l_index] *bl[l_index,0]*bl[l_index,1]
   
           IF model.global.sky_pixwindow NE 0 THEN BEGIN
              ;; apply pixel window function
              ee[ell[l_index]] = ee[ell[l_index]] * pixwinl[l_index,1]^2
              ;; bb[ell[l_index]] = bb[ell[l_index]] * pixwinl[l_index,1]^2
              te[ell[l_index]] = te[ell[l_index]] * pixwinl[l_index,0] * pixwinl[l_index,1]
           ENDIF
           
           PSM_DEBUG_MESSAGE, 'Should use PSM_CL2ALM for multivariate Gaussian number generation', routine='GAUSSIAN_CMB'
           GENERATE_ALM_E_GIVEN_ALM_T, ell, ee, tt, te, almt, alme, /predict
           nb_alm = (lmax+1L)*(lmax+2L)/2L 
           IF KEYWORD_SET(double) NE 0 THEN almspol = DBLARR(nb_alm,2,3) $
           ELSE almspol = FLTARR(nb_alm,2,3)
           almspol[*,*,0] = TEMPORARY(almt)
           almspol[*,*,1] = TEMPORARY(alme)
        ENDIF 

        ;; Correct from pixel window function if needed
        ;;----------------------------------------------
        ;; CASE model.mapping.sky_pixwindow OF
        ;;    '1': BEGIN
        ;;       ;; Correct alm_E from T pixel window and apply E window instead
        ;;       ;; Do nothing to alm_B which is not measured and hence vanishes
        ;;       IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN              
        ;;          hpw = HEALPIXWINDOW(SKY_NSIDE(), 2)              
        ;;          almspol[*,0,1] = almspol[*,0,1] * hpw[lll,1] / hpw[lll,0]
        ;;          almspol[*,1,1] = almspol[*,1,1] * hpw[lll,1] / hpw[lll,0]
        ;;       ENDIF ELSE BEGIN
        ;;          hpw = HEALPIXWINDOW(SKY_NSIDE(), 2)              
        ;;       ENDELSE
        ;;    END
        ;;    '0': BEGIN
        ;;       hpw = HEALPIXWINDOW(SKY_NSIDE(), 2)              
        ;;       ;; Correct alm_E from T pixel window 
        ;;       almspol[*,0,0] = almspol[*,0,1] / hpw[lll,0]
        ;;       almspol[*,1,0] = almspol[*,1,1] / hpw[lll,0]
        ;;       ;; Correct alm_E from T pixel window 
        ;;       IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN              
        ;;          almspol[*,0,1] = almspol[*,0,1] / hpw[lll,0]
        ;;          almspol[*,1,1] = almspol[*,1,1] / hpw[lll,0]
        ;;       ENDIF ELSE BEGIN
        ;;       ENDELSE
        ;;    END
        ;; ENDCASE

        ;; Write the cmb alm file
        ;;------------------------
        skypix = SKY_PIX()
        IF skypix.coordsys NE 'G' THEN PSM_DEBUG_MESSAGE, 'Fix coordinates for predicted CMB', routine='PREDICTED_CMB', infolimit=0, stoplimit=1 
        IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN
           WRITE_CMB_ALM, almspol, cmb_alm_file, cmb
           almspol = 0B
        ENDIF ELSE BEGIN
           WRITE_CMB_ALM, almt, cmb_alm_file, cmb
           almt = 0B
        ENDELSE


  ;; Generate map if needed
  ;;------------------------
     IF STRUPCASE(gp.precision) EQ 'DOUBLE' THEN isdouble=1 ELSE isdouble=0
     IF cmbp.save_cmb_map EQ 'yes' THEN BEGIN        
        IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, 'COMPUTING THE CMB MAP'
        file_exists = FILE_TEST(cmb_map_file)
        IF file_exists THEN BEGIN
           IF runp.carefulness GE 2 THEN BEGIN
              PSM_INFO_MESSAGE, 'CMB map file already exists: STOPPING THE PSM RUN'
              PSM_INFO_MESSAGE, 'Type .c to delete existing file ('+cmb_map_file+') and continue'
              STOP
           ENDIF
           FILE_DELETE, cmb_map_file
        ENDIF
        IF STRUPCASE(gp.fields) EQ 'TP' THEN BEGIN
           PSM_ALM2MAP, cmb_alm_file, cmb_map_file, hpx_nside = pixp.nside, /polar
        ENDIF ELSE BEGIN
           PSM_ALM2MAP, cmb_alm_file, cmb_map_file, hpx_nside = pixp.nside
        ENDELSE        
     ENDIF
     
  END
ENDCASE

END
