; -----------------------------------------------------------------------------
;
; Copyright (C) 2006
;   Antoine Chamballu
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;    DEFINE_CLUSTER
;
; PURPOSE:
;    Create a structure containing the cluster parameters. You can define the parameters one by one by hand gamma, sigma_8, gf, 
;    t_star, ns, z_array, small_delta_c_array, big_delta_c_array OR give a structure cosmo.
;    If you give only a cosmo then t_star and sigma_8 will be taken from Pierpaoli et al. 2001 
;
; CALLING SEQUENCE:
;    cluster = DEFINE_CLUSTER(gamma=gamma, sigma_8=sigma_8, gf=gf, t_star=t_star, ns=ns, z_array=z_array, small_delta_c_array=small_delta_c_array,
;                             big_delta_c_array=big_delta_c_array,
;                             cosmo=cosmo, coherent=coherent,
;                             silent=silent, in_c=in_c)
; INPUTS:
;   
; KEYWORDS:
;    gamma : the slope of the matter power spectrum
;    sigma_8 : normalization of the matter power spectrum
;    gf : the cluster gas fraction at T=6 keV
;    t_star : normalization of the Mass-Temperature relation
;    ns : spectral index (cf. k^ns)
;    z_array : a redshift array (covering the cluster redshift range)
;    small_delta_c : the corresponding linear critical density contrast (delta_c(z))
;    big_delta_c_array : the corresponding critical density contrast (Delta_c(z))
;    cosmo : structure with the cosmological parameters
;    coherent : set this keyword to modify gf and t_star in order to make them coherent with your input value for sigma_8 and
;               the current X-ray observations
;    silent : to avoid comments
;    in_c : to compute big_delta_c with a code based on Horellou & Berge, 2005 (dedicated to Terry Riley ;-) )
;
; OUTPUTS:
;    cluster : the structure containing the cluster parameters
;
; EXAMPLE:
;    
; RESTRICTIONS:
;    Report any bugs to Jean-Baptiste Melin and Jacques Delabrouille
;    jean-baptiste.melin@cea.fr, delabrouille@apc.univ-paris7.fr
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, February 1 2006
;    Antoine's define_cluster.pro included after the "coherent" keyword, JBM, Feb 6 2006
;    FAST_SMALL_DELTA_C and FAST_BIG_DELTA_C included, JBM, Feb 6 2006
;    Keyword in_c introduced, Antoine Chamballu, February 14 2006
;    Keyword coherent changed (gf  and t_star are coherent with sigma_8), AC, February 14 2006
;    The coherent keyword allows to fix sigma_8 and t_star and then determine a coherent gf, AC March 24 2006
;    Spectral index ns added, JBM, Sep 21 2006
;    Adapted to wCDM models, Matthieu Roman, April 2012
;-

;------------------------------------------
FUNCTION COMPUTE_SIGMA_8_2001, gamma, cosmo
;------------------------------------------
; taken from Pierpaoli, Scott and White 2001
; for 0.19 LE gamma LE 0.37
;     0.2  LE omega_m LE 0.8
;     0.3  LE lambda LE 1
; I have extend the formula to outer values...

ome_max = 1.1
lam_max = 1.1
delta_omega_m = 0.1
delta_lambda = 0.1
ome = (FINDGEN(ome_max/delta_omega_m)+1)*delta_omega_m
lam = FINDGEN(lam_max/delta_lambda)*delta_lambda
nome = N_ELEMENTS(ome)
nlam = N_ELEMENTS(lam)
omega_m = ome#REPLICATE(1,nlam)
lambda = REPLICATE(1,nome)#lam

p = [0.505, 0.150, -0.233, 0.048]
q = [1.049, 0.029, 0.546, 0.350, 0.278, -0.312]

sigma_cl = p[0]*omega_m^(-(p[1]+p[2]*omega_m+p[3]*lambda))
sigma8 = sigma_cl*q[0]*lambda^q[1]*omega_m^(-(q[2]+q[3]*omega_m))*gamma^(q[4]+q[5]*omega_m)

ndeg = 3
FOR i=0, nome-1 DO BEGIN
  coef = POLY_FIT(INDGEN(nlam-1)+1,REFORM(sigma8[i,1:*]), ndeg)
  val = POLY(0,coef)
  sigma8[i,0] = val
ENDFOR

sig8 = BILINEAR(sigma8, (cosmo.omega_m-delta_omega_m)/delta_omega_m, cosmo.omega_de/delta_lambda)

RETURN, sig8

END
;--

;---------------------------------------------------------------------------------------------------------------------------------------
FUNCTION DEFINE_CLUSTER, gamma=gamma, gf=gf, t_star=t_star, z_array=z_array, small_delta_c_array=small_delta_c_array, $
                         big_delta_c_array=big_delta_c_array, cosmo=cosmo, coherent=coherent, silent=silent, in_c=in_c

IF ((DEFINED(cosmo) EQ 0) AND (((DEFINED(gamma) EQ 0) OR (DEFINED(sigma_8) EQ 0) OR (DEFINED(gf) EQ 0) OR (DEFINED(t_star) EQ 0) OR $
    (DEFINED(ns) EQ 0) OR (DEFINED(z_array) EQ 0) OR (DEFINED(small_delta_c_array) EQ 0) OR (DEFINED(big_delta_c_array) EQ 0)))) THEN BEGIN
   PRINT,'CALL IS : cluster=DEFINE_CLUSTER()'
   PRINT, 'KEYWORDS : gamma, sigma_8, gf, t_star, ns, z_array, small_delta_c_array, big_delta_c_array, cosmo, coherent, silent'
   PRINT, 'You have to set cosmo OR ALL THE KEYWORDS gamma, sigma_8, gf, t_star, z_array, small_delta_c_array, big_delta_c_array'
   PRINT, 'You can also set coherent in order to force gf and t_star to be coherent with your input sigma_8 (or to force gf to be coherent with your input sigma_8 and t_star) and the current X-ray observations'
   GOTO, closing
ENDIF

IF (DEFINED(gamma) EQ 0) THEN BEGIN
  h = cosmo.h
  omega_m = cosmo.omega_m
  omega_b= cosmo.omega_b
  gamma = omega_m*h
  ;gamma = omega_m*h*EXP(-omega_b*(1+SQRT(2*h)/omega_m)) ; Sugiyama (1995)
ENDIF

;; IF (DEFINED(sigma_8) EQ 0) THEN BEGIN
;;    PSM_DEBUG_MESSAGE, 'Risk of bug on the value of sigma_8', routine='DEFINE_CLUSTER'
;;    ;; Pierpaoli et al. 2001
;;    sigma_8 = COMPUTE_SIGMA_8_2001(gamma, cosmo)
;;    sigma_8 = cosmo.sigma_8
;; ENDIF

IF (DEFINED(gf) EQ 0) THEN BEGIN
  omega_m = cosmo.omega_m
  omega_b = cosmo.omega_b
  xg = 0.9 ; gaz fraction in clusters / baryon fraction in universe ratio
  ; gf is the gaz fraction in clusters
  gf = xg*omega_b/omega_m
ENDIF

;; IF (DEFINED(ns) EQ 0) THEN BEGIN
;;    PSM_DEBUG_MESSAGE, 'Risk of bug on the value of ns', routine='DEFINE_CLUSTER'
;;    ns = 1.
;; ENDIF

IF (DEFINED(z_array) EQ 0) THEN BEGIN
   z_array = FINDGEN(100)/10
   ;small_delta_c_array = SMALL_DELTA_C(z_array, cosmo=cosmo)
   ;big_delta_c_array = BIG_DELTA_C(z_array, cosmo=cosmo)
   IF KEYWORD_SET(in_c) THEN BEGIN
       ;big_delta_c_array = BIG_DELTA_C_IN_C(z_array, cosmo=cosmo)
       PRINT, 'This option is not active. Please, contact jean-baptiste.melin@cea.fr if you want to use it.'
   ENDIF ELSE BEGIN
       big_delta_c_array = BDELTA_C(z_array, cosmo=cosmo)
   ENDELSE
   small_delta_c_array = SDELTA_C(z_array, cosmo=cosmo)
ENDIF

IF KEYWORD_SET(coherent) THEN BEGIN
;  t_star_mohr = 1.451
;  gf_mohr = 0.0256*6^(0.486)*cosmo.h^(-3/2.)
;  t_star = (gf*t_star_mohr^(3/2.)/gf_mohr)^(2/3.)
;  sigma_8 = 0.585*t_star^(-0.8)*cosmo.omega_m^(-0.6) ; deduit de Pierpaoli et al. 2003
    ;IF NOT DEFINED(cosmo) THEN BEGIN
    ;    print, "You can't be coherent without defining a cosmology."
    ;ENDIF ELSE BEGIN
    ;    IF (DEFINED(t_star) EQ 0) THEN t_star = 0.512*sigma_8^(-1.25)*cosmo.omega_m^(-0.75) ; deduit de Pierpaoli et al. 2003
    ;    gf = find_fgas( z_array, big_delta_c_array, t_star, cosmo=cosmo) ; chosen to fit the rc-T relation of Romer et al., 2001 at 6keV
    ;ENDELSE
PRINT, 'This option is not active. Please, contact jean-baptiste.melin@cea.fr if you want to use it.'
ENDIF

IF (DEFINED(t_star) EQ 0) THEN BEGIN
   ; Pierpaoli et al. 2001
   t_star = 1.3
ENDIF

IF NOT KEYWORD_SET(silent) THEN BEGIN
  PRINT, 'gamma='+STRING(gamma, format='(F5.3)')
  ;; PRINT, 'sigma_8='+STRING(sigma_8, format='(F5.3)')
  PRINT, 'gf (T=6keV)='+STRING(gf, format='(F5.3)')
  PRINT, 't_star='+STRING(t_star, format='(F5.3)')
  ;; PRINT, 'ns='+STRING(ns, format='(F5.3)')
  PRINT, 'z_array[0]='+STRING(z_array[0], format='(F5.3)')
  PRINT, 'small_delta_c_array[0]='+STRING(small_delta_c_array[0], format='(F5.3)')
  PRINT, 'big_delta_c_array[0]='+STRING(big_delta_c_array[0], format='(F6.2)')
ENDIF

;; RETURN, {gamma:gamma,sigma_8:sigma_8,gf:gf,t_star:t_star, ns:ns, $
;;          z_array:z_array,small_delta_c_array:small_delta_c_array,big_delta_c_array:big_delta_c_array}
RETURN, {gamma:gamma,gf:gf,t_star:t_star, $
         z_array:z_array,small_delta_c_array:small_delta_c_array,big_delta_c_array:big_delta_c_array}

closing:
END
