;+
; NAME:
;    DEFINE_COSMO
; ONE LINE HELP:
;    Create a structure containing the cosmological parameters
; PURPOSE:
;    Create a structure containing the cosmological parameters
; CALLING SEQUENCE:
;    cosmo = DEFINE_COSMO(h=h, omega_m=omega_m, omega_b=omega_b, lambda=lambda, silent=silent)
; INPUT:
;   
; OUTPUT:
;    cosmo : the structure containing the cosmological parameters
; KEYWORDS:
;    h : hubble parameter at z=0 i.e. H0/100
;    omega_m : omega matter
;    omega_b : omega baryon
;    lambda : cosmological constant (actually omega_lambda)
;    silent : to avoid comments
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, September 27 2001
;    keyword structure added, JBM, October 30 2002
;    gaz fraction gf added, JBM, October 3 2003
;    gamma expression improved, JBM, March 5 2004
;    gaz fraction, sigma_8, gamma removed from cosmo structure. Now in cluster structure, JBM, February 2006
;-

FUNCTION DEFINE_COSMO, t_cmb=t_cmb, h=h, omega_m=omega_m, omega_b=omega_b, omega_k=omega_k, omega_nu=omega_nu, omega_de=omega_de, $
                       sigma_8=sigma_8, n_scal=n_scal, n_s_running=n_s_running, n_tens=n_tens, n_t_running=n_t_running, r_tens_scal=r_tens_scal, tau_reion=tau_reion, fraction_he=fraction_he, $
                       n_massless_nu=n_massless_nu, w_dark_energy=w_dark_energy, k_pivot=k_pivot, scalar_amplitude=scalar_amplitude, $
                       silent=silent
; omega_de is computed from the other parameters

  cosmo = {cosmological_parameters, $
           T_CMB:2.725, $                ; CMB temperature
           h:0.704, $                    ; Hubble parameter
           Omega_m:0.272, $              ; Omega matter
           Omega_b:0.0456, $             ; Omega baryon
           Omega_k:0., $                 ; Omega curvature
           Omega_nu:0., $                ; Omega neutrino
           Omega_de:FLOAT('NaN'), $      ; Omega dark energy or vacuum (computed below)
;          lambda:FLOAT('NaN'), $        ; Duplicates Omega_de (computed below)
           Sigma_8:0.809, $              ; sigma_8 (for LSS power normalisation, use in SZ)
           n_s: 0.963, $                 ; scalar spectral index
           n_s_running: 0., $            ; running of the scalar spectral index
           n_t: 0., $                    ; tensor spectral index
           n_t_running: 0., $            ; running of the tensor spectral index
           r:0.00, $                     ; tensor to scalar ratio (at k_pivot)
           tau_reion:0.087, $            ; reionisation optical depth
;          z_reion:11., $                ; reionisation redshift
           he_fraction:0.25, $           ; helium fraction by mass
           n_massless_nu:3.04, $         ; number of effective massless neutrinos species
           n_massive_nu:0., $            ; number of massive neutrino species
           w_dark_energy: -1., $         ; dark energy equation of state
           k_pivot: 0.002, $             ; pivot scale for power spectrum normalisation
           scalar_amplitude: 2.441e-9 $  ; scalar amplitude
          }

  ;; CMB temperature
  ;;-----------------
  IF DEFINED(t_cmb) NE 0 THEN cosmo.t_cmb = t_cmb

  ;; Sigma8
  ;;-----------------
  IF DEFINED(sigma_8) NE 0 THEN cosmo.sigma_8 = sigma_8

  ;; Hubble parameter h
  ;;--------------------
  IF DEFINED(h) NE 0 THEN cosmo.h = h

  ;; Density parameters
  ;;--------------------
  IF DEFINED(omega_m) NE 0 THEN cosmo.omega_m = omega_m
  IF DEFINED(omega_b) NE 0 THEN cosmo.omega_b = omega_b
  IF DEFINED(omega_nu) NE 0 THEN cosmo.omega_nu = omega_nu

  ;; Check consistency between density parameters
  ;;----------------------------------------------
  IF DEFINED(omega_k) THEN BEGIN
     cosmo.omega_k = omega_k
     IF DEFINED(omega_de) THEN BEGIN
        cosmo.omega_de = omega_de
        ok = (cosmo.omega_de) EQ (1 - cosmo.omega_m - cosmo.omega_k - cosmo.omega_nu)
        IF NOT ok THEN MESSAGE, 'Inconsistant Cosmological Parameters'
     ENDIF ELSE BEGIN
        cosmo.omega_de = 1 - cosmo.omega_m - cosmo.omega_k - cosmo.omega_nu
     ENDELSE
  ENDIF ELSE BEGIN
     IF DEFINED(omega_de) THEN BEGIN
        cosmo.omega_de = omega_de
        cosmo.omega_k = 1 - cosmo.omega_m - cosmo.omega_de - cosmo.omega_nu
     ENDIF ELSE BEGIN
        cosmo.omega_de = 1 - cosmo.omega_m - cosmo.omega_k - cosmo.omega_nu
     ENDELSE
  ENDELSE

  ;; Dark Energy or cosmological constant (?)
  ;;------------------------------------------
  IF DEFINED(w_dark_energy) NE 0 THEN cosmo.w_dark_energy = w_dark_energy
;  cosmo.lambda = cosmo.omega_de

  ;; Initial power spectrum
  ;;------------------------
  IF DEFINED(n_scal) NE 0 THEN cosmo.n_s = n_scal
  IF DEFINED(n_s_running) NE 0 THEN cosmo.n_s_running = n_s_running
  IF DEFINED(n_tens) NE 0 THEN cosmo.n_t = n_tens
  IF DEFINED(n_t_running) NE 0 THEN cosmo.n_t_running = n_t_running
  IF DEFINED(scalar_amplitude) NE 0 THEN cosmo.scalar_amplitude = scalar_amplitude
  IF DEFINED(r_tens_scal) NE 0 THEN cosmo.r = r_tens_scal
  IF DEFINED(k_pivot) NE 0 THEN cosmo.k_pivot = k_pivot

  ;; Reionisation
  ;;--------------
  IF DEFINED(tau_reion) NE 0 THEN cosmo.tau_reion = tau_reion

  ;; Helium fraction
  ;;-----------------
  IF DEFINED(fraction_he) NE 0 THEN cosmo.he_fraction = fraction_he

  ;; Neutrino types
  ;;----------------
  IF DEFINED(n_massless_nu) NE 0 THEN cosmo.n_massless_nu = n_massless_nu
  IF DEFINED(n_massive_nu) NE 0 THEN cosmo.n_massive_nu = n_massive_nu


  IF NOT KEYWORD_SET(silent) THEN PRTSTRUCT, cosmo, name='COSMO'
  
  RETURN, cosmo

;; IF ((DEFINED(h) EQ 0) OR (DEFINED(omega_m) EQ 0) OR (DEFINED(omega_b) EQ 0) OR (DEFINED(lambda) EQ 0)) THEN BEGIN
;;   PRINT,'CALL IS : cosmo=DEFINE_COSMO()'
;;   PRINT, 'KEYWORDS : h, omega_m, omega_b, lambda, silent'
;;   PRINT, 'You have to set the keywords h, omega_m, omega_b, lambda'
;;   GOTO, closing
;; ENDIF

;; IF NOT KEYWORD_SET(silent) THEN BEGIN
;;   PRINT, 'h = '+NUMBER2STRING(h)
;;   PRINT, 'omega_m = '+NUMBER2STRING(omega_m)
;;   PRINT, 'omega_b = '+NUMBER2STRING(omega_b)
;;   PRINT, 'lambda = omega_de = '+NUMBER2STRING(lambda)
;; ENDIF

;; RETURN, {h:h,omega_m:omega_m,omega_b:omega_b,lambda:lambda,omega_de:lambda}

closing:
END
