;PRO COMOVING2OBSERVABLE, x, y, z, lon, lat, redshift, radian=radian,
;                         cosmo=cosmo, reverse=reverse
;+
; NAME:
;     COMOVING2OBSERVABLE
; ONE LINE HELP:
;     Convert cartesian comoving coordinates to observable coordinates
; PURPOSE:
;     Convert cartesian comoving coordinates to observable coordinates
; CALLING SEQUENCE:
;     COMOVING2OBSERVABLE, x, y, z, lon, lat, redshift, radian=radian,
;                          cosmo=cosmo, reverse=reverse
; INPUT:
;     x, y, z : cartesian comoving coordinates (i.e. x,y,z)
; OUTPUT:
;     lon, lat, redshift : observable coordinates
; KEYWORDS:
;     cosmo : the structure containing the cosmological parameters
;     reverse : set this keyword to compute comoving coordinates
;               from observable coordinates
;     radian : to compute in radians (degrees is the default)
; EXAMPLES:
;     
; RESTRICTIONS:
;     Report any bugs to Jacques Delabrouille
;     j.delabrouille@cdf.in2p3.fr
; PROCEDURES CALLED:
;     CARTESIAN2SPHERICAL, COM_DIST
; REVISION HISTORY
;     Written, Jean-Baptiste Melin 21 May 2001
;     radian keyword added, JD & JBM, 29 May 2001
;     now the inputs can be arrays, JBM & JD, 29 May 2001
;     outputs same format as input when arrays, handles long arrays, and
;                   increase value of z_cut, JD, 30 may 2001
;     Keyword cosmo introduced, JBM, Sept 27 2001
;     bug in comoving distance computation fixed, JBM, June 30 2009
;-

;------------------------------------------------------------------------------
FUNCTION COMOVING_DIST2Z, comoving_dist, cosmo=cosmo, $
                          z_cut=z_cut, accuracy=accuracy
;------------------------------------------------------------------------------

IF KEYWORD_SET(accuracy) EQ 0 THEN accuracy = 0.001
IF KEYWORD_SET(z_cut) EQ 0 THEN z_cut = 10.

sor = SORT(comoving_dist)
ordered_dist = comoving_dist(sor)
;ordered_dist = [ordered_dist, $
;  (1+(z_cut+1))*ANG_DIST((z_cut+1), cosmo=cosmo)] ;-> BUG if Omega_tot NE 1
ordered_dist = [ordered_dist, COM_DIST(z_cut, cosmo=cosmo)]

z = 0.
co_dist = 0.
index = 0L
indexmax = N_ELEMENTS(comoving_dist)-1 

z_out = FLTARR(indexmax+1)

WHILE ((index LE indexmax) AND (z LE z_cut)) DO BEGIN
  z = z + accuracy
  co_dist = COM_DIST(z, cosmo=cosmo)
  WHILE (co_dist GT ordered_dist[index]) DO BEGIN
     z_out[index] = z - accuracy
     index = index + 1
  ENDWHILE
ENDWHILE

IF (index LE indexmax) THEN BEGIN
  PRINT, '  WARNING! Cutting redshift z_cut too small for input'
ENDIF ELSE BEGIN
  ;z_return = FLTARR(indexmax+1)
  z_return = 0.*comoving_dist
  z_return[sor] = z_out
  RETURN, FLOAT(z_return)
ENDELSE

END
;------------------------------------------------------------------------------

PRO COMOVING2OBSERVABLE, x, y, z, lon, lat, redshift, radian=radian, $
                           cosmo=cosmo, reverse=reverse

IF NOT KEYWORD_SET(reverse) THEN BEGIN
  CARTESIAN2SPHERICAL, x, y, z, r, theta, lon, radian=radian
  redshift = COMOVING_DIST2Z(r, cosmo=cosmo, z_cut=6.)
  IF KEYWORD_SET(radian) THEN lat = !pi/2. - theta ELSE lat = 90. - theta
ENDIF ELSE BEGIN
  IF KEYWORD_SET(radian) THEN theta = !pi/2. - lat ELSE theta = 90. - lat
  r = COM_DIST(redshift, cosmo=cosmo)
  CARTESIAN2SPHERICAL, r, theta, lon, x, y, z, radian=radian, /reverse
ENDELSE

END
