;FUNCTION COSMO_TK, k, cosmo=cosmo, cluster=cluster, bbks=bbks
;+
; NAME:
;    COSMO_TK
; ONE LINE HELP:
;    Compute the transfer function T(k)
; PURPOSE:
;    Given a wavenumber k (Mpc^(-1.)), this routine returns the corresponding
;    value of the transfer function T(k).
;    The general form of the transfer function is taken from Eisenstein & Hu (1998) or from BBKS (1986) if bbks is set.
; CALLING SEQUENCE:
;    tk = COSMO_TK(k, cosmo=cosmo, cluster=cluster, bbks=bbks)
; 
; INPUT:
;    k : the wavenumber(s) (can be an array) in Mpc^(-1.)
; OUTPUT:
;    tk : the corresponding transfer function T(k)
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters.
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;    bbks : set this keyword if you want to use the BBKS (1986) function instead of the one from Eisenstein & Hu (1998)
; EXAMPLES:
;
; RESTRICTIONS:
;    Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, July 28 2009
;-

;------------------------------------------------------------------------
FUNCTION T_0_TILDE_EH, k, alpha, beta, keq=keq
;---------------------------------------------

q = k/13.41/keq

big_c = 14.2/alpha + 386./(1.+69.9*q^1.08)

RETURN, ALOG(EXP(1)+1.8*beta*q)/(ALOG(EXP(1)+1.8*beta*q)+big_c*q^2.)

END
;------------------------------------------------------------------------

;------------------------------------------------------------------------
FUNCTION G_Y_EH, y
;-----------------

RETURN, y*(-6.*SQRT(1.+y)+(2.+3.*y)*ALOG((SQRT(1+y)+1.)/(SQRT(1.+y)-1.)))

END
;------------------------------------------------------------------------

FUNCTION COSMO_TK, k, cosmo=cosmo, cluster=cluster, bbks=bbks

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
	PRINT,'  CALL IS : tk = COSMO_TK(k)'
	PRINT,'  Keywords : cosmo=cosmo, cluster=cluster, bbks=bbks'
     GOTO, closing
ENDIF

IF KEYWORD_SET(bbks) THEN BEGIN

  h = cosmo.h
  gamma = cluster.gamma
  q = k/gamma/h

  tk  =  alog(1.+2.34*q)/(2.34*q)*(1.+3.89*q+(16.1*q)^2+(5.46*q)^3 $
	 +(6.71*q)^4)^(-0.25)

ENDIF ELSE BEGIN

  omega_0 = cosmo.omega_m
  omega_b = cosmo.omega_b
  omega_c = omega_0-omega_b
  h = cosmo.h

  theta = CMB_TEMPERATURE()/2.7

  zeq = 2.50d4*omega_0*h^2.*theta^(-4.)
  keq = 7.46d-2*omega_0*h^2.*theta^(-2.) ; Mpc^-1

  bd_1 = 0.313*(omega_0*h^2.)^(-0.419)*(1.+0.607*(omega_0*h^2.)^0.674)
  bd_2 = 0.238*(omega_0*h^2.)^0.223
  zd = 1291.*((omega_0*h^2.)^0.251)/(1.+0.659*(omega_0*h^2.)^0.828)*(1.+bd_1*(omega_b*h^2.)^bd_2)

  req = 31.5*(omega_b*h^2.)*theta^(-4.)*(zeq/1d3)^(-1.)
  rd = 31.5*(omega_b*h^2.)*theta^(-4.)*(zd/1d3)^(-1.)

  s = 2/(3.*keq)*SQRT(6./req)*ALOG((SQRT(1+rd)+SQRT(rd+req))/(1.+SQRT(req)))

  ksilk = 1.6*((omega_b*h^2.)^0.52)*((omega_0*h^2.)^0.73)*(1.+(10.4*omega_0*h^2.)^(-0.95))

  a_1 = (46.9*omega_0*h^2.)^0.670*(1.+(32.1*omega_0*h^2.)^(-0.532))
  a_2 = (12.0*omega_0*h^2.)^0.424*(1.+(45.0*omega_0*h^2.)^(-0.582))
  alpha_c = (a_1^(-omega_b/FLOAT(omega_0)))*(a_2^((-omega_b/FLOAT(omega_0))^3.))

  b_1 = 0.944*(1.+(458.*omega_0*h^2.)^(-0.708))^(-1.)
  b_2 = (0.395*omega_0*h^2.)^(-0.0266)
  beta_c = 1./(1.+b_1*(((omega_c/FLOAT(omega_0))^b_2)-1.))

  alpha_b = 2.07*keq*s*(1+rd)^(-3/4.)*G_Y_EH((1.+zeq)/(1.+zd))
  beta_b = 0.5+omega_b/FLOAT(omega_0)+(3.-2.*omega_b/FLOAT(omega_0))*SQRT(1.+(17.2*omega_0*h^2.)^2.)
  beta_node = 8.41*(omega_0*h^2.)^0.435

  fk = 1./(1.+(k*s/5.4)^4.)
  tk0 = T_0_TILDE_EH(k, 1., beta_c, keq=keq)
  tk1 = T_0_TILDE_EH(k, alpha_c, beta_c, keq=keq)
  tk_c = fk*tk0+(1.-fk)*tk1

  tk2 = T_0_TILDE_EH(k, 1., 1., keq=keq)
  sk_tilde = s/(1.+(beta_node/(k*s))^3.)^(1/3.)
  klim = 500d0
  tk_b = (tk2/(1.+(k*s/5.2)^2.)+alpha_b/(1.+(beta_b/(k*s))^3.)*EXP(-(((k/ksilk)^1.4)<klim)))*SIN(k*sk_tilde)/(k*sk_tilde)

  tk = omega_b/FLOAT(omega_0)*tk_b + omega_c/FLOAT(omega_0)*tk_c

ENDELSE

RETURN, tk

closing:
END
