;PRO PK_SIGMA, mass, sigma, alpha, cosmo=cosmo, cluster=cluster, slope=slope
;+
; NAME:
;    PK_SIGMA
; ONE LINE HELP:
;    Compute integrated P(k).
; PURPOSE:
; Given the mass scale mass (in 10^15 solar mass units), the bias, h and omega_m,
; this routine returns
;      * the density fluctuation amplitude sigma i.e., SQRT(1./(2*!pi^2)*int(k^2*P(k)*W(k)^2)),
;      * the local slope alpha, i.e. -dln(sigma)/dln(mass)
;        (if keyword slope is set)
;  
; CALLING SEQUENCE:
;    PK_SIGMA, mass, sigma, alpha, cosmo=cosmo, cluster=cluster, slope=slope
; 
; INPUT:
;    mass : mass scale, in units of 10^15 solar masses
; OUTPUT:
;    sigma  : fluctuation amplitude for scale M
;    alpha  : -dln(sigma)/dln(mass)
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;    slope : set this keyword to compute the slope
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    COSMO_PK
; REVISION HISTORY
;    Written, Jim Bartlett, December 1996
;    minor modifications and name changed from CDMsig to cdm_sigma, April 2001
;    sig_vec added and modifications in header, JBM, May 2001
;    CDM_SIGMA becomes PK_SIGMA, now the procedure uses COSMO_PK,
;    sig_vec now computed in PK_SIGMA_VELOCITY, JBM, September 7 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    cluster keyword added, JBM, Feb 6 2006
;-

;------------------------------------------------------------------------------
function PK_SIGMA_I_SIGMA, x
;---------------------------
common PK_SIGMA_COSMO, pk_sigma_cosmo, pk_sigma_cluster
common PK_SIGMA_SCALE, pk_sigma_r

k = x/pk_sigma_r
Pk = COSMO_PK(k, cosmo=pk_sigma_cosmo, cluster=pk_sigma_cluster, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))  

return, k^2*Pk*Wk^2
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
function PK_SIGMA_I_SLOPE, x
;---------------------------

common PK_SIGMA_COSMO, pk_sigma_cosmo, pk_sigma_cluster
common PK_SIGMA_SCALE, pk_sigma_r

k = x/pk_sigma_r
Pk = COSMO_PK(k, cosmo=pk_sigma_cosmo, cluster=pk_sigma_cluster, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))  

return, k*Pk*Wk*sin(x)
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO PK_SIGMA, mass, sigma, alpha, cosmo=cosmo, cluster=cluster, slope=slope

common PK_SIGMA_COSMO, pk_sigma_cosmo, pk_sigma_cluster
common PK_SIGMA_SCALE, pk_sigma_r


pk_sigma_cosmo=cosmo
pk_sigma_cluster=cluster

rho = 2.76e-4*pk_sigma_cosmo.omega_m*(pk_sigma_cosmo.h)^2
; in units of 10^15 solar masses/Mpc^3

x1 = 0.0                       ; roots of Wth(x)
x2 = 4.496
x3 = 7.723
x4 = 10.902
x5 = 14.066
x6 = 17.221
x7 = 20.371

; compute the spectrum normalization 
norm = COSMO_PK(1., cosmo=pk_sigma_cosmo, cluster=pk_sigma_cluster) / COSMO_PK(1., cosmo=pk_sigma_cosmo, cluster=pk_sigma_cluster, /nonorm)

; Calculation of sig

pk_sigma_r  = (3.*mass/(4.*!pi*rho))^(1./3)
sig1 = qromo('PK_SIGMA_I_SIGMA',x1,x2)
sig2 = qromo('PK_SIGMA_I_SIGMA',x2,x3)
sig3 = qromo('PK_SIGMA_I_SIGMA',x3,x4)
sig4 = qromo('PK_SIGMA_I_SIGMA',x4,x5)
sig5 = qromo('PK_SIGMA_I_SIGMA',x5,x6)
if (sig5/sig1 gt 0.1) then stop, 'sig not converging'
sigma = SQRT(norm/(2.*!pi^2)*(1./pk_sigma_r)*(sig1+sig2+sig3+sig4+sig5))


; Calculation of the slope

IF KEYWORD_SET(slope) THEN BEGIN
slope1 = qromo('PK_SIGMA_I_SLOPE',x1,x2)
slope2 = qromo('PK_SIGMA_I_SLOPE',x2,x3)
slope3 = qromo('PK_SIGMA_I_SLOPE',x3,x4)
slope4 = qromo('PK_SIGMA_I_SLOPE',x4,x5)
slope5 = qromo('PK_SIGMA_I_SLOPE',x5,x6)
slope6 = qromo('PK_SIGMA_I_SLOPE',x6,x7)
if (slope6/slope1 gt 0.1) then stop, 'slope not converging'
alpha = norm/(2.*!pi^2)*1./(sigma*pk_sigma_r)^2*(slope1+slope2+slope3+slope4 $
        +slope5+slope6)
alpha = 1. - alpha
ENDIF

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
