;FUNCTION PK_SIGMA_J, j, scale, cosmo=cosmo, cluster=cluster
;
;+
; NAME:
;    PK_SIGMA_VELOCITY
; ONE LINE HELP:
;    Compute integrated P(k).
; PURPOSE:
;    Given the scale "scale" (in Mpc), the redshifts z_array, 
;    the cosmology, the function returns
;    SQRT(1./(2*!pi^2)*int(P(k)*W(k)^2*k^(2*j+2)))
; 
; CALLING SEQUENCE:
;    sig_j = PK_SIGMA_J(j, scale, cosmo=cosmo, cluster=cluster)
; 
; INPUT:
;    scale : scale in Mpc
; OUTPUT:
;    sig_j  : sigma_j defined by SQRT(1./(2*!pi^2)*int(P(k)*W(k)^2*k^(2*j+2)))
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    COSMO_PK
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, March 18 2001
;    cluster keyword added, JBM, Feb 6 2006
;-

;------------------------------------------------------------------------------
function PK_SIGMA_J_I_VEC, x
;----------------------------------
common PK_SIGMA_J_COSMO, pk_sigma_j_cosmo, pk_sigma_j_cluster
common PK_SIGMA_J_SCALE, pk_sigma_j_r, pk_sigma_j_j

k = x/pk_sigma_j_r
Pk = COSMO_PK(k, cosmo=pk_sigma_j_cosmo, cluster=pk_sigma_j_cluster, /silent, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))

i = 2.*pk_sigma_j_j+2.

return, Pk*Wk^2*k^i
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FUNCTION PK_SIGMA_J, j, scale, cosmo=cosmo, cluster=cluster

common PK_SIGMA_J_COSMO, pk_sigma_j_cosmo, pk_sigma_j_cluster
common PK_SIGMA_J_SCALE, pk_sigma_j_r, pk_sigma_j_j

pk_sigma_j_cosmo = cosmo
pk_sigma_j_cluster = cluster
pk_sigma_j_j = j

x1 = 0.0                       ; roots of Wth(x)
x2 = 4.496
x3 = 7.723
x4 = 10.902
x5 = 14.066
x6 = 17.221
x7 = 20.371

; compute the spectrum normalization 
norm = COSMO_PK(1., cosmo = pk_sigma_j_cosmo, cluster=pk_sigma_j_cluster, $
                /silent) / COSMO_PK(1., cosmo = pk_sigma_j_cosmo, cluster=pk_sigma_j_cluster, /silent, /nonorm)
pk_sigma_j_r  = scale

vec1 = qromo('PK_SIGMA_J_I_VEC',x1,x2)
vec2 = qromo('PK_SIGMA_J_I_VEC',x2,x3)
vec3 = qromo('PK_SIGMA_J_I_VEC',x3,x4)
vec4 = qromo('PK_SIGMA_J_I_VEC',x4,x5)
vec5 = qromo('PK_SIGMA_J_I_VEC',x5,x6)
vec6 = qromo('PK_SIGMA_J_I_VEC',x6,x7)
if (vec6/vec1 gt 0.1) then stop, 'vec not converging'
vec = norm/(2.*!pi^2)*(1./pk_sigma_j_r)*(vec1+vec2+vec3+vec4 $
        +vec5+vec6)

RETURN, SQRT(vec)

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
