; -----------------------------------------------------------------------------
;
; Copyright (C) 2002-2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	TCCY_MF_DENSITY
;
; PURPOSE:
;	Compute the density dN/(dlntheta_c*dlnY) given by a mass
;	function in the (theta_c,Y) plane for dOmega = 1 square degree.
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	TCCY_MF_DENSITY, Theta_c, Cy, Tccy_mf_dens, COSMO=, CLUSTER=, MASS_FUNCTION=
;
; INPUTS:
;	Theta_c:	Core radius theta core array (in arcmin)
;	Cy:		SZ integrated Compton parameter Y array (in arcmin^2)
;
; KEYWORD PARAMETERS:
;	COSMO:		The structure containing the cosmological parameters
;			(created with DEFINE_COSMO). If not defined, then a
;			default cosmology is used, with h=0.7,
;			omega_m=0.3, omega_b=0.04, omega_de=0.7 
;	CLUSTER:	The structure containing the structure formation parameters
;			(created with DEFINE_CLUSTER).
;	MASS_FUNCTION:	String of characters specifying the mass function to be used.
;
; OUTPUTS:
;	Tcc_mf_dens:	The density (dN/dlntheta_c*dlnY)) given by the mass_function
;			as a function of theta_c and cy for dOmega = 1 square degree.
;
; RESTRICTIONS:
;	The result is obtained with M between 0.1 and 10 (x 10e15 solar masses).
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	19/03/2002:	Jean-Baptiste Melin, First version.
; 	09/10/2003:	JBM, Theta_c changed in ln(theta_c).
; 	dd/02/2006:	JBM, CLUSTER keyword added.
; 	11/01/2012:	Jacques Delabrouille, Try to have a single
; 			structure in the PSM to define cosmological
; 			parameters. Step 1, harmonize the PSM
; 			model.cosmo with the use of DEFINE_COSMO. 
; 	20/09/2012:	JD, update of doc and license header.
;-
PRO TCCY_MF_DENSITY, theta_c, cy, tccy_mf_dens, $
                     cosmo=cosmo, cluster=cluster, mass_function=mass_function

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        PRINT, 'CALL IS : TCCY_MF_DENSITY, theta_c, cy, tccy_mf_dens'
        PRINT, 'Keywords : cosmo=cosmo, cluster=cluster, mass_function=mass_function'
     GOTO, closing
ENDIF

; default mass_function
IF NOT KEYWORD_SET(mass_function) THEN BEGIN
  PRINT, 'Using the Press-Schechter function...'
  mass_function = 'press_schechter'
ENDIF

; default cosmo
IF NOT KEYWORD_SET(cosmo) THEN BEGIN
  PRINT, 'Using Omega_m=0.3 & Lambda=0.7'
  cosmo = DEFINE_COSMO(h=0.7, omega_m=0.3, omega_b=0.04, omega_de=0.7)
ENDIF

; default cluster
IF NOT KEYWORD_SET(cluster) THEN BEGIN
  cluster = DEFINE_CLUSTER(cosmo=cosmo)
ENDIF

IF NOT DEFINED(theta_c) THEN BEGIN
  min_theta_c = 1d-2
  max_theta_c = 1d1
  step_log10_theta_c = 100.
  log10_theta_c = (FINDGEN(CEIL((ALOG10(max_theta_c)-ALOG10(min_theta_c))*step_log10_theta_c)+1)/step_log10_theta_c)+ $
                   ALOG10(min_theta_c)
  theta_c = EXP(ALOG(10)*log10_theta_c)
ENDIF

IF NOT DEFINED(cy) THEN BEGIN
  min_cy = 1d-5
  max_cy = 1d0
  step_log10_cy = 100.
  log10_cy = (FINDGEN(CEIL((ALOG10(max_cy)-ALOG10(min_cy))*step_log10_cy)+1)/step_log10_cy)+ALOG10(min_cy)
  cy = EXP(ALOG(10)*log10_cy)
ENDIF

nt = N_ELEMENTS(theta_c)
ny = N_ELEMENTS(cy)

fy = REPLICATE(1.,nt)
ftc = REPLICATE(1.,ny)
tc_array = theta_c#TRANSPOSE(ftc)
tc_array = REFORM(tc_array, nt*ny)
cy_array = fy#TRANSPOSE(cy)
cy_array = REFORM(cy_array, nt*ny)

FAST_TCCY2ZM, tc_array, cy_array, z_ar, m_ar, cosmo=cosmo, cluster=cluster

; mass limits
M_inf = 0.1
M_sup = 10.
; z limit
z_sup = 5.
z_inf = 0.01

in_do = INTO_DOMAIN_TC_CY(tc_array, cy_array, z_inf, z_sup, M_inf, M_sup, cosmo=cosmo, cluster=cluster)

; step z
step_log10_z = 100.

; step m
step_log10 = 200. ;100.
delta_lnM = ALOG(10)/step_log10

; building redshift array
;z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf
log10_z = (FINDGEN(CEIL((ALOG10(z_sup)-ALOG10(z_inf))*step_log10_z)+1)/step_log10_z)+ALOG10(z_inf)
z_list = EXP(ALOG(10)*log10_z)

; building mass array in 10^15 M_sun units
log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
mass_list = EXP(ALOG(10)*log10_m)

MF_DENSITY, z_list, mass_list, mf_plot, $
cosmo=cosmo, cluster=cluster, mass_function=mass_function

nz = N_ELEMENTS(z_list)
nm = N_ELEMENTS(mass_list)
fm = REPLICATE(1.,nz)
fz = REPLICATE(1.,nm)
z_array = z_list#TRANSPOSE(fz)
z_array = REFORM(z_array, nz*nm)
M_array = fm#TRANSPOSE(mass_list)
M_array = REFORM(M_array, nz*nm)

FAST_TCCY2ZM, tc_ar, cy_ar, z_array, M_array, cosmo=cosmo, cluster=cluster, /reverse

E = HUBBLE_AT_Z(z_list, cosmo=cosmo)/100./cosmo.h
D = ANG_DIST(z_list, cosmo=cosmo)
bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_list)

fact1b = DERIV(z_list, E^(2./3)/D^2.*bdc^(1./3))/(E^(2./3)/D^2.*bdc^(1./3))
fact1bb = fact1b#TRANSPOSE(fz)
cy_ar = REFORM(cy_ar, nz, nm)
fact1 = fact1bb
fact2 = 5./3
;tc_ar = REFORM(tc_ar, nz, nm) correspond a l'ancienne version ou on
;avait theta_c a la place de ln(theta_c) ; cela changeait le jacobien.
fact3b = DERIV(z_list, E^(-2./3)/D*bdc^(-1./3))/(E^(-2./3)/D*bdc^(-1./3))
fact3bb = fact3b#TRANSPOSE(fz)
fact3 = fact3bb ;tc_ar*fact3bb
fact4 = 1/3. ;tc_ar/3.
Jaco = fact4*fact1 - fact2*fact3
Jaco = 1./Jaco

;first_res = mf_plot*jaco

;z_ar = z_ar>z_inf
;x = (ALOG(z_ar)-ALOG(z_inf))/(ALOG(10)/step_log10_z) 
;m_ar = m_ar>0.1
;y = (ALOG(m_ar)-ALOG(M_inf))/delta_lnM
x = INTERPOL(FINDGEN(nz),z_list,z_ar)
y = INTERPOL(FINDGEN(nm),mass_list,m_ar)

res = INTERPOLATE(mf_plot, x, y)
res = REFORM(res, nt, ny)
in_do = REFORM(in_do, nt, ny)
res = res*in_do
jac1 = INTERPOLATE(Jaco, x, y)
jac1 = REFORM(jac1, nt, ny)

tccy_mf_dens = jac1*res

;print, total(tccy_mf_dens)*step_theta_c*step_cy
;surface, tccy_mf_dens, theta_c, cy, background=255, color=0, charsize=2.
;plot, theta_c, tccy_mf_dens[*,1], color=0, background=255
;oplot, theta_c, tccy_mf_dens[*,2], color=0
;oplot, theta_c, tccy_mf_dens[*,3], color=0

closing:
END
