PRO READ_PEP_FIRB_FILE, file, cor, poi

  ON_ERROR, 2

  ;; Test input data file
  ;;----------------------
  ok = TEST_PSM_DATA(file,/get)
  IF NOT ok THEN MESSAGE, 'Input data file not available'

  ;; Correlated part, converted to MJy^2/sr
  ;;----------------------------------------
  READCOL, file, l, cor, comment='#'
  cor = cor/1e12

  ;; Poisson part, converted to MJy^2/sr
  ;;-------------------------------------
  GET_LUN, unit
  OPENR, unit, file
  line = ''
  READF, unit, line
  line = STRSPLIT(STRTRIM(STRCOMPRESS(PSM_STR_REPLACE(line, '#', ' ')),2),/extract)
  poi = FLOAT(line[1])/1e12
  CLOSE, unit
  FREE_LUN, unit

END

FUNCTION PEP_FIRB_CL, correl=correl, poisson=poisson, lmax=lmax, c857x160mu=c857x160mu, c160mux100mu=c160mux100mu, plot=plot, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, "PEP_FIRB_CL: function to create a cl structure corresponding to FIRB spectrum from Planck early paper"
     PSM_INFO_MESSAGE, "SYNTAX: result = PEP_FIRB_CL(correl=, poisson=, lmax=, c857x160mu=, c160mux100mu=, /plot)"
     PSM_INFO_MESSAGE, "'correl' is an output keyword for recovering the correlated part"
     PSM_INFO_MESSAGE, "'poisson' is an output keyword for recovering the poisson part"
     PSM_INFO_MESSAGE, "Set 'lmax' to the maximum harmonic mode (default=3000)"
     PSM_INFO_MESSAGE, "Set 'c857x160mu' (correlation between 857 GHz and 160 micron) to a number between 0 and 1 (default 0.9)"
     PSM_INFO_MESSAGE, "Set 'c160mux100mu (correlation between 160 and 100 micron) to a number between 0 and 1 (default 0.9)'"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(c857x160mu) THEN c857x160mu=0.9
  IF NOT DEFINED(c160mux100mu) THEN c160mux100mu=0.9

  IF c857x160mu GT 1 THEN MESSAGE, 'The correlation between 857GHz and 160 micron cannot be larger than 1'
  IF c160mux100mu GT 1 THEN MESSAGE, 'The correlation between 160 micron and 100 micron cannot be larger than 1'

  IF NOT KEYWORD_SET(lmax) THEN lmax=3000
  uselmax = lmax<3000

  DEFSYSV, '!phyconst', exists=exists
  IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS

  datadir = GETPSMDIR('psmdata')+'ancillary/models/firb/PEP-CIB-spectra-v2/' 
  fieldnames=['100GHz','143GHz','217GHz','353GHz','545GHz','857GHz','160micron','100micron']
  freqs = [100e9,143e9,217e9,353e9,545e9,857e9,!phyconst.c/160e-6,!phyconst.c/100e-6]
  
  nf = N_ELEMENTS(fieldnames)
  correl = CL_STRUCTURE(fieldnames=fieldnames, units=REPLICATE('MJy/sr',nf), lmin=0, lmax=lmax, /double)
  poisson = CL_STRUCTURE(fieldnames=fieldnames, units=REPLICATE('MJy/sr',nf), lmin=0, lmax=lmax, /double)
  
  IF lmax GT uselmax THEN BEGIN
     lextra = (FINDGEN(lmax-uselmax)+uselmax+1)
  ENDIF
  
  ;; From 100 GHz to 857 GHz
  ;;-------------------------
  FOR i=0, 5 DO FOR j=i,5 DO BEGIN
     file = datadir+'cl_simu_px-cibxcib_ij-pep-'+fieldnames[j]+'x'+fieldnames[i]+'.txt'
     READ_PEP_FIRB_FILE, file, cor, poi
     IF lmax GT uselmax THEN BEGIN
        extra = EXP(ALOG(cor[uselmax-1000]) + (ALOG(cor[uselmax])-ALOG(cor[uselmax-1000])) * (ALOG(lextra) - ALOG(uselmax-1000)) / (ALOG(uselmax) - ALOG(uselmax-1000)))
        cor = [cor,extra]
     ENDIF
     cor[1]=cor[2]
     cor[0]=cor[2]
     si = NUMBER2STRING(i+1)
     sj = NUMBER2STRING(j+1)
     ok = EXECUTE('correl.c_'+si+'_'+sj+' = cor[0:lmax]')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'
     ok = EXECUTE('poisson.c_'+si+'_'+sj+'[*] = poi')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'
  ENDFOR
  
  ;; 100 micron auto-spectrum
  ;;--------------------------
  file = datadir+'cl_simu_px-cibxcib_ij-pep-100micronx100micron.txt'
  READ_PEP_FIRB_FILE, file, cor, poi
  IF lmax GT uselmax THEN BEGIN
     extra = EXP(ALOG(cor[uselmax-1000]) + (ALOG(cor[uselmax])-ALOG(cor[uselmax-1000])) * (ALOG(lextra) - ALOG(uselmax-1000)) / (ALOG(uselmax) - ALOG(uselmax-1000)))
     cor = [cor,extra]
  ENDIF
  cor[1]=cor[2]
  cor[0]=cor[2]
  correl.c_8_8 = cor[0:lmax]
  poisson.c_8_8[*] = poi
  
  ;; FIRB emission law
  ;;-------------------
  firb = FIRB_NU_INU(freqs,/fixsen1998)/freqs
  cfirb = firb*0
  cfirb[0] = SQRT(TOTAL(correl.c_1_1*(2*correl.l+1)))
  cfirb[1] = SQRT(TOTAL(correl.c_2_2*(2*correl.l+1)))
  cfirb[2] = SQRT(TOTAL(correl.c_3_3*(2*correl.l+1)))
  cfirb[3] = SQRT(TOTAL(correl.c_4_4*(2*correl.l+1)))
  cfirb[4] = SQRT(TOTAL(correl.c_5_5*(2*correl.l+1)))
  cfirb[5] = SQRT(TOTAL(correl.c_6_6*(2*correl.l+1)))
  cfirb[7] = SQRT(TOTAL(correl.c_8_8*(2*correl.l+1)))
  
  pfirb = firb*0
  pfirb[0] = SQRT(TOTAL(poisson.c_1_1*(2*poisson.l+1)))
  pfirb[1] = SQRT(TOTAL(poisson.c_2_2*(2*poisson.l+1)))
  pfirb[2] = SQRT(TOTAL(poisson.c_3_3*(2*poisson.l+1)))
  pfirb[3] = SQRT(TOTAL(poisson.c_4_4*(2*poisson.l+1)))
  pfirb[4] = SQRT(TOTAL(poisson.c_5_5*(2*poisson.l+1)))
  pfirb[5] = SQRT(TOTAL(poisson.c_6_6*(2*poisson.l+1)))
  pfirb[7] = SQRT(TOTAL(poisson.c_8_8*(2*poisson.l+1)))
  
  ff = (FINDGEN(300)+1)*1e10
  
  ;;======================================================================================================================
  
  ;; Fit of the SED of the correlated part
  ;;---------------------------------------
  p = [0,1,2,3,4,5,7]
  nuref = freqs[5]
  FIT_GREYBODY, DOUBLE(freqs[p]), DOUBLE(cfirb[p]), amp1, specind1, temp1, nuref=nuref, dflux=DOUBLE(cfirb[p]*[.3,.3,.2,.1,.1,.1,.1])
  
  ;; Use the fit to get the correlated spectrum at 160 micron
  ;;----------------------------------------------------------
  cf160mu = amp1*(freqs[6]/nuref)^specind1*PLANCK_BNU(freqs[6],temp=temp1)/PLANCK_BNU(nuref,temp=temp1)
  cf100mu = amp1*(freqs[7]/nuref)^specind1*PLANCK_BNU(freqs[7],temp=temp1)/PLANCK_BNU(nuref,temp=temp1)
  cf857 = amp1*(freqs[5]/nuref)^specind1*PLANCK_BNU(freqs[5],temp=temp1)/PLANCK_BNU(nuref,temp=temp1)
  correl.c_7_7 = SQRT(correl.c_6_6*correl.c_8_8) * (cf160mu/cf100mu) * (cf160mu/cf857)
  cfirb[6] = SQRT(TOTAL(correl.c_7_7*(2*correl.l+1)))
  
  ;; Limit cross-correlation so that it does not exceed 1
  ;; (correlated part)
  ;;------------------------------------------------------
  ;; FOR i=1,6 DO FOR j=i,6 DO BEGIN
  ;;    si = NUMBER2STRING(i)
  ;;    sj = NUMBER2STRING(j)
  ;;    ok = EXECUTE('correl.c_'+si+'_'+sj+' = correl.c_'+si+'_'+sj+' < SQRT(correl.c_'+si+'_'+si+'*correl.c_'+sj+'_'+sj+')')
  ;;    IF NOT ok THEN MESSAGE, 'Command could not be executed'
  ;; ENDFOR
  
  ;; Cross-spectra for 160 micron (correlated part)
  ;;------------------------------------------------
  ;; correl.c_6_7 = c857x160mu * SQRT(correl.c_6_6*correl.c_7_7)
  ;; correl.c_5_7 = c857x160mu * correl.c_5_6 * SQRT(correl.c_7_7/correl.c_6_6)
  ;; correl.c_4_7 = c857x160mu * correl.c_4_6 * SQRT(correl.c_7_7/correl.c_6_6)
  ;; correl.c_3_7 = c857x160mu * correl.c_3_6 * SQRT(correl.c_7_7/correl.c_6_6)
  ;; correl.c_2_7 = c857x160mu * correl.c_2_6 * SQRT(correl.c_7_7/correl.c_6_6)
  ;; correl.c_1_7 = c857x160mu * correl.c_1_6 * SQRT(correl.c_7_7/correl.c_6_6)
  
  ;; Cross-spectra for 100 micron (correlated part)
  ;;------------------------------------------------
  ;; correl.c_7_8 = c160mux100mu * SQRT(correl.c_7_7*correl.c_8_8)
  ;; correl.c_6_8 = c160mux100mu * correl.c_6_7 * SQRT(correl.c_8_8/correl.c_7_7)
  ;; correl.c_5_8 = c160mux100mu * correl.c_5_7 * SQRT(correl.c_8_8/correl.c_7_7)
  ;; correl.c_4_8 = c160mux100mu * correl.c_4_7 * SQRT(correl.c_8_8/correl.c_7_7)
  ;; correl.c_3_8 = c160mux100mu * correl.c_3_7 * SQRT(correl.c_8_8/correl.c_7_7)
  ;; correl.c_2_8 = c160mux100mu * correl.c_2_7 * SQRT(correl.c_8_8/correl.c_7_7)
  ;; correl.c_1_8 = c160mux100mu * correl.c_1_7 * SQRT(correl.c_8_8/correl.c_7_7)
  
  ;; Make sure correlation always decreases when frequencies are further apart?
  ;;----------------------------------------------------------------------------
  
  
  
  ;;======================================================================================================================
  
  ;; Fit of the SED of the poisson part
  ;;------------------------------------
  p = [2,3,4,5,7]
  nuref = freqs[5]
  FIT_GREYBODY, DOUBLE(freqs[p]), DOUBLE(pfirb[p]), amp2, specind2, temp2, nuref=nuref, dflux=DOUBLE(pfirb[p]*[.2,.1,.1,.1,.1])
  
  ;; Use the fit to get the poisson spectrum at 160 micron
  ;;----------------------------------------------------------
  cf160mu = amp2*(freqs[6]/nuref)^specind2*PLANCK_BNU(freqs[6],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  cf100mu = amp2*(freqs[7]/nuref)^specind2*PLANCK_BNU(freqs[7],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  cf857 = amp2*(freqs[5]/nuref)^specind2*PLANCK_BNU(freqs[5],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  poisson.c_7_7 = SQRT(poisson.c_6_6*poisson.c_8_8) * (cf160mu/cf100mu) * (cf160mu/cf857)
  pfirb[6] = SQRT(TOTAL(poisson.c_7_7*(2*poisson.l+1)))
  
  ;; Limit cross-correlation so that it does not exceed 1
  ;; (poisson part)
  ;;------------------------------------------------------
  ;; FOR i=1,6 DO FOR j=i,6 DO BEGIN
  ;;    si = NUMBER2STRING(i)
  ;;    sj = NUMBER2STRING(j)
  ;;    ok = EXECUTE('poisson.c_'+si+'_'+sj+' = poisson.c_'+si+'_'+sj+' < SQRT(poisson.c_'+si+'_'+si+'*poisson.c_'+sj+'_'+sj+')')
  ;;    IF NOT ok THEN MESSAGE, 'Command could not be executed'
  ;; ENDFOR
  
  ;; Auto and cross-spectra for 100 and 143 GHz
  ;;--------------------------------------------
  ;; cf100 = amp2*(freqs[0]/nuref)^specind2*PLANCK_BNU(freqs[0],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  ;; cf143 = amp2*(freqs[1]/nuref)^specind2*PLANCK_BNU(freqs[1],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  ;; cf217 = amp2*(freqs[2]/nuref)^specind2*PLANCK_BNU(freqs[2],temp=temp2)/PLANCK_BNU(nuref,temp=temp2)
  ;; poisson.c_2_2 = poisson.c_3_3 * (cf143/cf217)^2
  ;; poisson.c_1_1 = poisson.c_3_3 * (cf100/cf217)^2
  
  ;; poisson.c_1_2 = 0.99 * SQRT(poisson.c_1_1*poisson.c_2_2)
  ;; poisson.c_1_3 = 0.99^2 * SQRT(poisson.c_1_1*poisson.c_3_3)
  ;; poisson.c_1_4 = 0.99^2 * poisson.c_3_4 * SQRT(poisson.c_1_1/poisson.c_3_3)
  ;; poisson.c_1_5 = 0.99^2 * poisson.c_3_5 * SQRT(poisson.c_1_1/poisson.c_3_3)
  ;; poisson.c_1_6 = 0.99^2 * poisson.c_3_6 * SQRT(poisson.c_1_1/poisson.c_3_3)
  ;; poisson.c_1_7 = 0.99^2 * poisson.c_3_7 * SQRT(poisson.c_1_1/poisson.c_3_3)
  ;; poisson.c_1_8 = 0.99^2 * poisson.c_3_8 * SQRT(poisson.c_1_1/poisson.c_3_3)
  
  ;; poisson.c_2_3 = 0.99 * SQRT(poisson.c_2_2*poisson.c_3_3)
  ;; poisson.c_2_4 = 0.99 * poisson.c_3_4 * SQRT(poisson.c_2_2/poisson.c_3_3)
  ;; poisson.c_2_5 = 0.99 * poisson.c_3_5 * SQRT(poisson.c_2_2/poisson.c_3_3)
  ;; poisson.c_2_6 = 0.99 * poisson.c_3_6 * SQRT(poisson.c_2_2/poisson.c_3_3)
  ;; poisson.c_2_7 = 0.99 * poisson.c_3_7 * SQRT(poisson.c_2_2/poisson.c_3_3)
  ;; poisson.c_2_8 = 0.99 * poisson.c_3_8 * SQRT(poisson.c_2_2/poisson.c_3_3)
  
  ;; Overwrite covariance matrix for Planck channels
  ;; using Guilaine and Matthieu's note 
  ;; http://wiki.planck.fr/index.php/Proc/ShotNoiseIRgal?action=download&upname=Poisson_IRgal.pdf
  ;;----------------------------------------------------------------------------------------------
  jy2megajy = 1d-12
  poisson.c_1_1[*] = jy2megajy * 0.35
  poisson.c_1_2[*] = jy2megajy * 0.64
  poisson.c_1_3[*] = jy2megajy * 1.7
  poisson.c_1_4[*] = jy2megajy * 5.3
  poisson.c_1_5[*] = jy2megajy * 15*.8
  poisson.c_1_6[*] = jy2megajy * 30*.8
  
  poisson.c_2_2[*] = jy2megajy * 1.4
  poisson.c_2_3[*] = jy2megajy * 4.0
  poisson.c_2_4[*] = jy2megajy * 12
  poisson.c_2_5[*] = jy2megajy * 31*.9
  poisson.c_2_6[*] = jy2megajy * 56*.9
  
  poisson.c_3_3[*] = jy2megajy * 12
  poisson.c_3_4[*] = jy2megajy * 38
  poisson.c_3_5[*] = jy2megajy * 95
  poisson.c_3_6[*] = jy2megajy * 160
  
  poisson.c_4_4[*] = jy2megajy * 138
  poisson.c_4_5[*] = jy2megajy * 361
  poisson.c_4_6[*] = jy2megajy * 629
  
  poisson.c_5_5[*] = jy2megajy * 1150
  poisson.c_5_6[*] = jy2megajy * 2230
  
  poisson.c_6_6[*] = jy2megajy * 5923
  
  pfirb[0] = SQRT(TOTAL(poisson.c_1_1*(2*poisson.l+1)))
  pfirb[1] = SQRT(TOTAL(poisson.c_2_2*(2*poisson.l+1)))
  
  ;; Cross-spectra for 160 micron (poisson part)
  ;;---------------------------------------------
  poisson.c_6_7 = c857x160mu * poisson.c_6_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  poisson.c_5_7 = c857x160mu * poisson.c_5_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  poisson.c_4_7 = c857x160mu * poisson.c_4_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  poisson.c_3_7 = c857x160mu * poisson.c_3_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  poisson.c_2_7 = c857x160mu * poisson.c_2_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  poisson.c_1_7 = c857x160mu * poisson.c_1_6 * SQRT(poisson.c_7_7/poisson.c_6_6)
  
  ;; Cross-spectra for 100 micron (poisson part)
  ;;---------------------------------------------
  poisson.c_7_8 = c160mux100mu * poisson.c_7_7 * SQRT(poisson.c_8_8*poisson.c_7_7)
  poisson.c_6_8 = c160mux100mu * poisson.c_6_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  poisson.c_5_8 = c160mux100mu * poisson.c_5_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  poisson.c_4_8 = c160mux100mu * poisson.c_4_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  poisson.c_3_8 = c160mux100mu * poisson.c_3_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  poisson.c_2_8 = c160mux100mu * poisson.c_2_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  poisson.c_1_8 = c160mux100mu * poisson.c_1_7 * SQRT(poisson.c_8_8/poisson.c_7_7)
  
  ;;======================================================================================================================
  
  ;; Correct the poisson covariance matrix so that it is equal to
  ;; the square of its square root
  ;;---------------------------------------------------------------
  cor = CL2COR(0,poisson)
  cor2 = cor
  a = MATRIX_SQRT(DOUBLE(cor),status)
  cor2 = a##a
  cor2 = (cor2+TRANSPOSE(cor2))/2
  a2 = MATRIX_SQRT(DOUBLE(cor2),status)
  IF status NE 1B THEN STOP
  
  ;; Overwrite cross spectra of correlated part to match
  ;; cross-channel correlation of the Poisson part
  ;;-----------------------------------------------------
  
  FOR i=1,8 DO FOR j=i, 8 DO BEGIN
     si = NUMBER2STRING(i)
     sj = NUMBER2STRING(j)
     ok = EXECUTE('correl.c_'+si+'_'+sj+' = cor2['+si+'-1,'+sj+'-1] * SQRT(correl.c_'+si+'_'+si+'*correl.c_'+sj+'_'+sj+')')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'        
  ENDFOR
  
  ;; Same thing for poisson part
  ;;-----------------------------
  FOR i=1,8 DO FOR j=i, 8 DO BEGIN
     si = NUMBER2STRING(i)
     sj = NUMBER2STRING(j)
     ok = EXECUTE('poisson.c_'+si+'_'+sj+' = cor2['+si+'-1,'+sj+'-1] * SQRT(poisson.c_'+si+'_'+si+'*poisson.c_'+sj+'_'+sj+')')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'        
  ENDFOR
  
  ;;======================================================================================================================
  
  ;; Make plots to check what is going on
  ;;--------------------------------------
  IF KEYWORD_SET(plot) NE 0 THEN BEGIN
     
     ;; Plot everything rescaled
     ;;--------------------------
     WINDOW, /free
     plot, freqs, firb, /xlog, /ylog, xrange=[10,5000]*1e9, yrange=[1e-23,3e-20], /ystyle, /nodata, title='CIB SED Normalised to 857 GHz', charsize=1.5
     oplot, freqs, cfirb*firb[5]/cfirb[5],psym=4, color=get_color('green')
     oplot, freqs, pfirb*firb[5]/pfirb[5],psym=2, color=get_color('pink')
     fix = FIRB_NU_INU(ff,/fixsen1998)/ff
     lag = FIRB_NU_INU(ff,/lagache1999)/ff
     oplot, ff, fix, linestyle=2, color=GET_COLOR('red') 
     oplot, ff, lag, linestyle=2, color=GET_COLOR('blue') 
     oplot, ff, amp1*(ff/nuref)^specind1*PLANCK_BNU(ff,temp=temp1)/PLANCK_BNU(nuref,temp=temp1)*firb[5]/cfirb[5], color=get_color('green')
     oplot, ff, amp2*(ff/nuref)^specind2*PLANCK_BNU(ff,temp=temp2)/PLANCK_BNU(nuref,temp=temp2)*firb[5]/pfirb[5], color=get_color('pink')
     AL_LEGEND, ['Total CIB Fixsen1998','TOTAL CIB Lagache1999','Correlated anisotropies','Poisson anisotropies'], $
                colors=GET_COLOR(['red','blue','green','pink']),linestyle=[2,2,0,0], charsize=1.5
     
     ;; Plot power of correlated and poisson parts
     ;;--------------------------------------------
     WINDOW, /free
     plot, [freqs,freqs], [cfirb,pfirb], /nodata, /xlog, /ylog, xrange=[10,5000]*1e9, yrange=[1e-4,1], /ystyle, title='CIB SED', charsize=1.5
     oplot, freqs, cfirb,psym=4, color=get_color('green')
     oplot, freqs, pfirb,psym=2, color=get_color('pink')
     oplot, ff, amp1*(ff/nuref)^specind1*PLANCK_BNU(ff,temp=temp1)/PLANCK_BNU(nuref,temp=temp1), color=get_color('green')
     oplot, ff, amp2*(ff/nuref)^specind2*PLANCK_BNU(ff,temp=temp2)/PLANCK_BNU(nuref,temp=temp2), color=get_color('pink')
     AL_LEGEND, ['Correlated anisotropies','Poisson anisotropies'], colors=GET_COLOR(['green','pink']),linestyle=[0,0], charsize=1.5
     
     ;; Print poisson correlation
     ;;---------------------------
     print, cor
     
  ENDIF
  
  cl = correl
  
  cl.c_1_1 = cl.c_1_1 + poisson.c_1_1
  cl.c_1_2 = cl.c_1_2 + poisson.c_1_2
  cl.c_1_3 = cl.c_1_3 + poisson.c_1_3
  cl.c_1_4 = cl.c_1_4 + poisson.c_1_4
  cl.c_1_5 = cl.c_1_5 + poisson.c_1_5
  cl.c_1_6 = cl.c_1_6 + poisson.c_1_6
  cl.c_1_7 = cl.c_1_7 + poisson.c_1_7
  cl.c_1_8 = cl.c_1_8 + poisson.c_1_8
  
  cl.c_2_2 = cl.c_2_2 + poisson.c_2_2
  cl.c_2_3 = cl.c_2_3 + poisson.c_2_3
  cl.c_2_4 = cl.c_2_4 + poisson.c_2_4
  cl.c_2_5 = cl.c_2_5 + poisson.c_2_5
  cl.c_2_6 = cl.c_2_6 + poisson.c_2_6
  cl.c_2_7 = cl.c_2_7 + poisson.c_2_7
  cl.c_2_8 = cl.c_2_8 + poisson.c_2_8
  
  cl.c_3_3 = cl.c_3_3 + poisson.c_3_3
  cl.c_3_4 = cl.c_3_4 + poisson.c_3_4
  cl.c_3_5 = cl.c_3_5 + poisson.c_3_5
  cl.c_3_6 = cl.c_3_6 + poisson.c_3_6
  cl.c_3_7 = cl.c_3_7 + poisson.c_3_7
  cl.c_3_8 = cl.c_3_8 + poisson.c_3_8
  
  cl.c_4_4 = cl.c_4_4 + poisson.c_4_4
  cl.c_4_5 = cl.c_4_5 + poisson.c_4_5
  cl.c_4_6 = cl.c_4_6 + poisson.c_4_6
  cl.c_4_7 = cl.c_4_7 + poisson.c_4_7
  cl.c_4_8 = cl.c_4_8 + poisson.c_4_8
  
  cl.c_5_5 = cl.c_5_5 + poisson.c_5_5
  cl.c_5_6 = cl.c_5_6 + poisson.c_5_6
  cl.c_5_7 = cl.c_5_7 + poisson.c_5_7
  cl.c_5_8 = cl.c_5_8 + poisson.c_5_8
  
  cl.c_6_6 = cl.c_6_6 + poisson.c_6_6
  cl.c_6_7 = cl.c_6_7 + poisson.c_6_7
  cl.c_6_8 = cl.c_6_8 + poisson.c_6_8
  
  cl.c_7_7 = cl.c_7_7 + poisson.c_7_7
  cl.c_7_8 = cl.c_7_8 + poisson.c_7_8
  
  cl.c_8_8 = cl.c_8_8 + poisson.c_8_8
  
  c0 = FIRB_NU_INU(freqs,/megajysr,/bestfit2012)/freqs
  FOR i=1,8 DO FOR j=i, 8 DO BEGIN
     ok = EXECUTE('cl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[0] = c0[i-1]*c0[j-1]')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'
  ENDFOR
  FOR i=1,8 DO FOR j=i, 8 DO BEGIN
     ok = EXECUTE('correl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[0] = c0[i-1]*c0[j-1] - poisson.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[0]')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'
  ENDFOR


  RETURN, cl
  
END

