; Copyright 2007 Observatoire de Paris

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Computes the total intensity and/or polarized emission produce by unresolved IR PS at
; given frequency (MJy/sr)(Proto-spheroidal galaxies + Strong lensed proto-spheroidal galaxies)
; 
;
; @categories ps
; 
; @param nside {in}{required}{type=int} HEALPix resolution for output map
; @param nu {in}{required}{type=int} frequency at which calculation is to be done (GHz)
; @param Slim {in}{required}{type=double} flux limit for strong sources in Jy
; @keyword all_sources {in}{optional}{type=boolean} If set, ALL sources appear in the output map
; @keyword outputdir {in}{optional}{type=string} where to save the catalog
; @keyword nopol {in}{optional}{type=boolean} if set don't take into account
; the polarisation.
;
; @history <p> edited to match the format of the working group 2
;    CMB and foreground template repository, prepared by Joaquin
;    Gonzalez-Nuevo, Gianfranco De Zotti and Carlo Baccigalupi, February 7, 2006</p>
; @history <p> added ecliptic coordinates, February 20 2006</p>
; @history <p> default Slim added,March 7 2006 </p>
; @history <p> Level S style fits catalogues are now created.
;                     At the moment each catalogue contains information
;                    about a single frequency only. The number of
;                    sources depends on the frequency chosen (which
;                    must be a Planck frequency), March 31 2006, Samuel Leach. </p>
; @history <p> Modifications to I/0. Routine now
;    accepts "outputdir" variable;  automatically finds its own
;    path for reading in template files, May 22 2006, Sam Leach</p>
; @history <p> bug fix to lensing_cat.dat. New catalogue added.
;                  Changes the lensed source counts (and only slightly overall
;                  maps/catalogues), May 31 2006, SML and JGN</p>
; @history <p> /nopol keyword added,June 15 2006 Sam Leach </p>
; @history <p> adapted the old version 'iqu_infraredsources' splitting it
;              into 'psm_ps_irb' under the requisits of the new PSM,  March 12 2007, JGN </p>
; @history <p> added external polarization properties,April 3 2007, JGN</p>
; @history <p> general cleaning, removed limit of 2048 for nside, bug fixed in ud_grading map in Jy, April 5 2007, JD</p>
; @history <p> now pointing computed in double precision, April 5 2007, JD</p>
; @history <p> doc PSM format, november 6 2007, Maude Martin</p>
;-

PRO PSM_PS_IRB, nside, freq_list, Slim, all_sources=all_sources, outputdir=outputdir,$
                     new_LS_cat=new_LS_cat,$
                     new_sph_sim=new_sph_sim


  IF (N_PARAMS() lt 3) THEN BEGIN
     PRINT, ' Wrong number of arguments in psm_ps_irb'
     PRINT, ' Syntax : '
     PRINT, ' psm_ps_irb, nside, freq_list (in GHz), Slim (in Jy)'
  ENDIF
  
  COMMON psm, param
  COMMON sky, model

;; Define parameters for convenience
;;-----------------------------------
  gp = param.global
  psp = model.ps
  firbp = model.firb
  names = param.names
  runp = param.run
  pixp = model.mapping.pix

  IF  NOT keyword_set(outputdir) THEN outputdir = GETPSMDIR('firb')
    
  PRINT,' Generating FarIr sources emission map in MJy/sr'
  PRINT,' in HEALPix format at Nside = '+NUMBER2STRING(nside)+'.'
  PRINT,''
  

;;----------------------------------------------------------------------------------
;; To generate or read the map of correlated distribution of spheroidal galaxies...
;;----------------------------------------------------------------------------------

  ;; default map
  ;lenscat = FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/Tmap353GHz.fits'
  lenscat = firbp.firb_files.Tmap353
  IF KEYWORD_SET(new_sph_sim) THEN BEGIN
     PRINT,' ... Generating a new input spheroid simulation at 353GHz...'
     ;firbdir = FIX_SEPARATOR(param.run.output_directory) + $
     ;          FIX_SEPARATOR(param.names.component_basename) + $
     ;          FIX_SEPARATOR(param.names.ps_basename) + $
     ;          FIX_SEPARATOR(param.names.firb_basename)
     ;lenscat = FIX_SEPARATOR(firbdir)+'lensed_Tmap353GHz.fits'

     Smap0 = CORRSKY_SPH(2048, -5., 0., Slim=0.1)
     lenscat_exists = FILE_TEST(lenscat)
     IF lenscat_exists THEN SPAWN, 'rm '+ lenscat
     units = 'K/KCMB'
     skypix = SKY_PIX()
     WRITE_FITS_MAP, lenscat, Smap0, /ring, coordsys=skypix.coordsys, units=units

     ;; Erase the file
     FILE_DELETE, lenscat, /quiet
  ENDIF ELSE BEGIN
     ;; reading the simulation of spheroids
     ;; map in DT/T (=K/KCMB), nside = 2048
     lenscat_exists = FILE_TEST(lenscat)
     IF lenscat_exists EQ 0 THEN GET_PSM_DATA, datafile = lenscat
     ;; MESSAGE,'ERROR : MISSING Tmap353GHz.fits in PSM data directory'
     ;READ_FITS_MAP, FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/Tmap353GHz.fits', Smap0
     ;h = HEADFITS(FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/Tmap353GHz.fits',exten=1)
     READ_FITS_MAP, lenscat, Smap0
     h = HEADFITS(lenscat,exten=1)
     units_in_file = STRTRIM(SXPAR(h,'TUNIT1'),2)
     UNIT2PSMUNIT, units_in_file, units
  ENDELSE
  
  ;; Find nside and npix of the map
  npix0 = N_ELEMENTS(Smap0)
  nside0 = LONG(NPIX2NSIDE(npix0))

  ;; Conversion from DT/T to Jy/sr, and then to Jy.
  conv_fact = CONVERSION_FACTOR(units, 'Jy/sr', 353e9)
  pix_size0  = 4*!PI / npix0
  Smap0     = Smap0 * conv_fact * pix_size0        ; The final units is Jy (not intensive unit)

;SL  ;; Degrading Smap0 if necessary
;SL  IF (nside LT nside0) THEN BEGIN
;SL     print,'nside lower than the default simulation, ud_grading the map'
;SL     ud_grade, Smap0, Smap1, nside_out=nside, ORDER_IN='RING'
;SL     Smap0 = TEMPORARY(Smap1)
;SL  ENDIF
;SL  IF (nside GT nside0) THEN BEGIN
;SL     print,'nside larger than the default simulation, ud_grading the map'
;SL     ud_grade,Smap0, Smap1,nside_out=nside,ORDER_IN='RING'
;SL     Smap0 = TEMPORARY(Smap1)
;SL  ENDIF
  
;;----------------------------------------------------------------------------------
;; Reading the spheroids spectral index information
;;----------------------------------------------------------------------------------
  ;;;;;;;;; Contain 10 columns with the spectral index to transform
  ;;;;;;;;; fluxes at 353 GHz to all the Planck channel (S and from 30 to 857)

  ;file = FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/sph_extrapol_table_353GHz.dat'
  file = firbp.firb_files.sph_extrapol_table_353GHz
  ok = FILE_TEST(file) 
  IF NOT ok THEN GET_PSM_DATA, datafile=file
  ;; MESSAGE,'ERROR : MISSING sph_extrapol_table_353GHz.dat in PSM data directory'
  GET_LUN, unit
  OPENR, unit, file
  alpha = FLTARR(10,100)
  READF, unit, alpha
  CLOSE, unit
  FREE_LUN, unit

  ;; Find for each pixel the appropriate flux bin of the alpha table
  ;;-----------------------------------------------------------------
  w  = WHERE(Smap0 GT 0.)
  Sh = ROUND(MAX(alpha[0,*]/0.1)) - ROUND(ALOG10(Smap0[w])/0.1)
  
;;----------------------------------------------------------------------------------
;; Flux correction for Smap0, and change units
;;----------------------------------------------------------------------------------

  ;; Correct the impact of udgrade, which is not OK for units = Jy
  ;;---------------------------------------------------------------
  ;;;;; Jy not INTENSIVE UNIT 
  ;;;;; This has to be done after finding the proper bin in the alpha table
  ;;;;; Factor of 1.e-6 / (4.*!pi/(12.*nside0*nside0)) for conversion from Jy
  ;;;;; in initial map at nside0, to MJy/sr
  ;;;;; (and not 1.e-6 / (4.*!pi/(12.*nside*nside)) 
  factor_for_units = 1.e-6 / (4.*!pi/(12.*nside0*nside0))
  Smap0        = factor_for_units * TEMPORARY(Smap0)

  ;; Reading the lensed sources spectral index information
  ;;-------------------------------------------------------
  ;;;;;;;;; Contain 8 columns with the spectral index to transform
  ;;;;;;;;; fluxes at 857 GHz to all the Planck channels (from 30 to 545)

  ;file = FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/lensing_alpha.dat'
  file = firbp.firb_files.lensing_alpha
  ok = FILE_TEST(file) 
  IF NOT ok THEN GET_PSM_DATA, datafile=file
  ;; MESSAGE,'ERROR : MISSING lensing_alpha.dat in PSM data directory'
  GET_LUN, unit
  OPENR, unit, file
  alpha_len = FLTARR(8,100)
  READF, unit, alpha_len
  CLOSE, unit
  FREE_LUN, unit

;;----------------------------------------------------------------------------------
;; To generate the catalogue of strongly lensed spheroidal galaxies
;;----------------------------------------------------------------------------------

  ;; To generate a new lensed sources catalogue...
  IF KEYWORD_SET(new_LS_cat) THEN BEGIN
     PRINT,' ... Generating a new input lensed source catalogue at 857GHz ...'
     GENERATE_INPUT_LENSING_CATALOGUE, 0.01, -1.
  ENDIF

  ;; Reading the lensed source catalog
  ;; Glon Glat S857GHz(mJy)
  ;file = FIX_SEPARATOR(!PSMROOT) + 'Data/ancillary/models/firb/JGN-2005/intermediate_lensing_cat.fits'
  file = firbp.firb_files.intermediate_lensing_cat
  ok = FILE_TEST(file) 
  IF NOT ok THEN GET_PSM_DATA, datafile=file
  ;a    = FINDFILE(file,count=ncat) 
  ;IF ncat EQ 0 THEN MESSAGE,'ERROR : MISSING intermediate_lensing_cat.fits in PSM data directory'
  cat  = MRDFITS(file, 1)
   ;; Erase the file
  IF KEYWORD_SET(new_LS_cat) THEN FILE_DELETE, file, /quiet

;;----------------------------------------------------------------------------------
;; Checking frequency in Planck band + frequency extrapolation for Smap0 and cat
;;----------------------------------------------------------------------------------
  planck_frequency = float([30,44,70,100,143,217,353,545,857])
  nfreq = N_ELEMENTS(freq_list)
  IF nfreq GT 10 THEN STOP      ; Function COMPONENT_STRUCT does not support more than 9 emission laws

  ;; Find the appropriate flux bin in alpha_len table to convert the source flux 
  Sh_len = 0 - ROUND(ALOG10(cat.F_mjy)/0.05)

  ;; Conversion coef.   
;SL  mJy_2_MJy_sr = 1.e-9 / (4.*!pi/(12.*nside*nside))
  mJy_2_MJy_sr = 1.e-9 / (4.*!pi/(12.*nside0*nside0)) ;SL

  ;; Initialize file list
  firbfiles = STRARR(nfreq)

  ;; FIRB ID
  PSM_ID_KEYGEN, firb_id

  FOR f=0, nfreq-1 DO BEGIN
     
     nu = freq_list[f]

     ;; Find the appropriate freq. bin in the alpha table to extrapolate the map flux
     mn = MIN(ABS(planck_frequency-nu),nu_id)
     firb = Smap0
     firb[w]  = Smap0[w] * (nu/353.)^(-alpha[1+nu_id,Sh])
     
     ;; Convert the source flux 
     IF nu_id LT 7 THEN cat_flux = cat.F_mjy * mJy_2_MJy_sr * (nu/857.)^(-alpha_len[nu_id,Sh_len])

     ;; Conversion to pixel numbers
;SL     ANG2PIX_RING, nside, (90D0-cat.glat_deg)/180D0*!DPI, cat.glon_deg/180D0*!DPI, ipring
     ANG2PIX_RING, nside0, (90D0-cat.glat_deg)/180D0*!DPI, cat.glon_deg/180D0*!DPI, ipring ;SL
     ;; Add these sources in the map
     nsourcetotal = n_elements(cat.glat_deg)  
     FOR i=0L, nsourcetotal-1 DO BEGIN
        firb[ipring[i]] = firb[ipring[i]] + cat_flux[i]
     ENDFOR

     ;; Smooth the map to sky resolution
     CHANGE_MAP_RESOLUTION, firb, smoothfirb, beam_in=0, beam_out=SKY_RESOLUTION(), nside_out=SKY_NSIDE(), lmax=SKY_LMAX(), $
                           ordering_in='RING', ordering_out=SKY_ORDERING(), double=double, /nohdr, /overwrite
     ;; Write the map
     IF NOT KEYWORD_SET(outputdir) THEN outputdir = GETPSMDIR('firb')
     fitsfile = outputdir + param.names.firb_basename + '_map_'+NUMBER2STRING(nu)+'GHz.fits'
     ;firbfiles[f] = PSM_STR_REPLACE(outputdir,GETPSMDIR('output'),'') + param.names.firb_basename + '_map_'+NUMBER2STRING(nu)+'GHz.fits'
     ;firbfiles[f] = param.names.firb_basename + '_map_'+NUMBER2STRING(nu)+'GHz.fits'
     firbfiles[f] = fitsfile
     basehdr = PSM_BASE_HDR(datatype='comp',dataform='map',/setval)
     maphdr = PSM_MAP_HDR(pixtype='HEALPIX',lmax=SKY_LMAX())
     xhdr = MERGE_FITSHDRS(basehdr,maphdr)
     bndhdr = PSM_BAND_HDR(planck_frequency[f]*1e9)
     xhdr = MERGE_FITSHDRS(xhdr,bndhdr)
     beamhdr = PSM_BEAM_HDR(SKY_RESOLUTION())
     xhdr = MERGE_FITSHDRS(xhdr,beamhdr)
     ;pixhdr = PSM_PIX_HDR(SKY_PIX())         ; done by WRITE_HPXMAP
     ;xhdr = MERGE_FITSHDRS(xhdr,pixhdr)      ; done by WRITE_HPXMAP
     
     skypix=SKY_PIX()
     WRITE_HPXMAP, fitsfile, tstokes=smoothfirb, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
     ;mollview, smoothfirb * conversion_factor('MJy/sr','mK_RJ',nu*1e9), title = 'FIRB map at '+ number2string(nu) + ' GHz (mK_RJ)'

  ENDFOR

  ;; Emission laws
  READ_MAP, firbfiles[0], ampmin
  nurefs = planck_frequency*1e9
  
  FOR i=1, nfreq-1 DO BEGIN
     em = EMLAW_PARAMSTRUCT('POWERLAW',/include_nurange,/include_ampl)
     *em.numax.value = nurefs[i]
     *em.numin.value = nurefs[i-1]
     *em.nuref.value = nurefs[PREVIOUS_EVEN(i-1)+1]
     *em.ampl.value = firbfiles[PREVIOUS_EVEN(i-1)+1]
     em.ampl.unit = 'MJy/sr'
     READ_MAP, firbfiles[i], ampmax
     specind = ALOG10(ampmax/ampmin) / ALOG10(*em.numax.value/*em.numin.value)

     basehdr = PSM_BASE_HDR(datatype='comp',dataform='map',/setval)
     maphdr = PSM_MAP_HDR(pixtype='HEALPIX',lmax=SKY_LMAX())
     xhdr = MERGE_FITSHDRS(basehdr,maphdr)
     beamhdr = PSM_BEAM_HDR(0)  ; It is assumed that only the amplitude maps are smoothed, not the spectral index
     xhdr = MERGE_FITSHDRS(xhdr,beamhdr)

     fitsfile = outputdir + param.names.firb_basename + '_specind_e'+NUMBER2STRING(i)+'.fits'
     WRITE_HPXMAP, fitsfile, tstokes=specind, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
     
     *em.specind.value = fitsfile
     ok = EXECUTE('e'+NUMBER2STRING(i)+'=em')
     ampmin = TEMPORARY(ampmax)
  ENDFOR

  ;; Fix frequency limits for first and last emission law
  *e1.numin.value = 0.
  *e8.numax.value = FLOAT('INF')

  ;; Create the component structure
  IF nfreq NE 9 THEN STOP
  comp = COMPONENT_STRUCT(e1,e2,e3,e4,e5,e6,e7,e8, name=param.names.firb_basename, class='diffuse')

  ;; Write the component structure
  IF FIRB_MODEL() NE 'no_firb' THEN BEGIN
     SAVE_COMP, comp
     PRTSTRUCT, comp, name='FIRB', file=GETPSMDIR('firb')+'firb.txt'
     DESTROY_DATA, comp
  ENDIF

END


;; ;; if (w[0] ge 0 AND w_sph[0] ge 0) then begin
;; ;;     catout=[[catout_LS],[cat_sph]]
;; ;; endif else begin
;; ;;     if (w[0] ge 0) then begin
;; ;;         catout=catout_LS
;; ;;     endif
;; ;;     if (w_sph[0] ge 0) then begin
;; ;;         catout=cat_sph
;; ;;     endif
;; ;; endelse

;; ;#################Catalog output#########################
;; ;Writing the catalog of stronger sources into fits format
;; ;
;; ;; nlensed= n_elements(cat[0,*])
;; ;; nsph   = 12l*nside^2
;; ;; ;;First assigning an id to each sources
;; ;; id_ls  = lindgen(nlensed)+1
;; ;; id_sph = lindgen(nsph) + 1

;; ;; ;;Selecting strongest sources (> Slim at this frequency)
;; ;; if nstrongls gt 0 then begin
;; ;;     namels = namearray('ls_',id_ls[w])
;; ;;     names  = namels
;; ;; endif
;; ;; if nstrongsph gt 0 then begin
;; ;;     namesph = namearray('sph_',id_sph[w_sph])
;; ;;     names   = namesph
;; ;; endif
;; ;; id_ls  = 0
;; ;; id_sph = 0
;; ;; if (nstrongls gt 0) and (nstrongsph gt 0) then names = [namels,namesph]

;; ;; if nstrongls+nstrongsph gt 0 then begin
;; ;;     if keyword_set(nopol) then begin
;; ;;         catstr = {name:names,$
;; ;;                   glon_deg:REFORM(catout[0,*]), glat_deg:REFORM(catout[1,*]), $
;; ;;                   flux_mJy:REFORM(catout[2,*])}
;; ;;     endif else begin
;; ;;         catstr = {name:names,$
;; ;;                   glon_deg:REFORM(catout[0,*]), glat_deg:REFORM(catout[1,*]), $
;; ;;                   flux_mJy:REFORM(catout[2,*]),$
;; ;;                   pol_perc:REFORM(catout[3,*]),polang_rad:REFORM(catout[4,*])}
;; ;;     endelse
;; ;;     filename = outputdir+'irb_cat_'+strtrim(round(nu),2)+'GHz_lowerfluxlimit_'+strtrim(round(Slim*1000.),2)+'mJy_nside_'+strtrim(nside,2)+'.fits'
;; ;;     mwrfits, catstr, filename, /create
;; ;;     catstr = 0
;; ;; endif
;; ;; catout = 0
