FUNCTION psm_free_free,pix_num,freq

;+
; NAME:
;    psm_free_free
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes free-free emission at given position(s) and frequency
; CALLING SEQUENCE:
;    vec=psm_free_free(pix_num,freq)
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              initialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = Free-Free contribution at given pixels and frequency (W/m2/sr/Hz)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    None
; OUTPUT KEYWORDS:
;    None
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_free_free(findgen(12L*Nside^2),100.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
;    Undefined pixels in the template are assumed not to contribute
; PROCEDURES CALLED:
;    make_ff_map.pro
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 15/01/2004 Uses the routine
;    make_ff_map developped by C. Dickinson and A. Banday
;    Modified Mar 15 07 by MAMD to use (*!sky_model).free_free.modelnumber
;    Modified Jan 18 08 by MAMD : output in MJy/sr
;    Modified Feb 26 08 by MAMD : add model 5
;    Modified Dec 01 09 by JD: now use conversion_factor to convert
;    units
;    Jan 08 2012 by MAMD : replace constant spectral index (-2.17) for
;    models 3 and 6 with the computation of r2ff
;-


  IF n_params() EQ 0 THEN BEGIN
     print,'vec=psm_free_free(pix_num,freq)'
     goto,exit
  ENDIF

  modnum = (*!sky_model).free_free.modelnumber
  freq_scaling = float( r2ff(freq, T_E=(*!sky_model).free_free.T_e)/r2ff(23., T_E=(*!sky_model).free_free.T_e) )
  case modnum of
     6: begin                   ; MAMD2008 Free-Free
        free_free_map = (*(*!sky_model).free_free.mamd2008)(pix_num)*freq_scaling
        ;transform into MJy/sr 
        free_free_map = free_free_map * CONVERSION_FACTOR('mK_RJ','MJy/sr',freq*1e9)
     end
     5: begin                   ; model from miville-deschenes et al. (2008)
        ; Free-Free from Halpha
        make_ff_map,(*(*!sky_model).free_free.halpha)(pix_num), $
                    (*(*!sky_model).free_free.Ebv)(pix_num), $
                    free_free_Halpha, $
                    FREQ_GHZ= freq, $
                    F_D=(*!sky_model).free_free.f_d, $
                    LIMIT=(*!sky_model).free_free.limit, $
                    T_e=(*!sky_model).free_free.T_e
        free_free_Halpha=free_free_Halpha/1.e3 ; transform in K
        ; Free-Free from WMAP MEM decomposition 
        free_free_wmap = (*(*!sky_model).free_free.wmap_mem)(pix_num)
        free_free_wmap = free_free_wmap*freq_scaling
        ; E(B-V) selection
        ebmv = (*(*!sky_model).free_free.Ebv)
        free_free_map = free_free_wmap<free_free_Halpha
        ind = where(ebmv gt 2.)
        PSM_DEBUG_MESSAGE, 'Check the quality of the output free-free map for model 5 (some pixels seem to be wrong)', routine = 'PSM_FREE_FREE'
        free_free_map(ind) = free_free_wmap(ind)
        ; transform to MJy/sr
        free_free_map = free_free_map * CONVERSION_FACTOR('mK_RJ','MJy/sr',freq*1e9)
     end
     4: begin                   ; BM model
        print, 'BM model not defined yet'
        free_free_map = -1
     end
     3: begin                   ; WMAP MEM model
        free_free_map = (*(*!sky_model).free_free.wmap_mem)(pix_num)*freq_scaling
        ;transform into MJy/sr 
        free_free_map = free_free_map * CONVERSION_FACTOR('mK_CMB','MJy/sr',freq*1e9)
     end
     else: begin                ; default model
        make_ff_map,(*(*!sky_model).free_free.halpha)(pix_num), $
                    (*(*!sky_model).free_free.Ebv)(pix_num), $
                    free_free_map, $
                    FREQ_GHZ= freq, $
                    F_D=(*!sky_model).free_free.f_d, $
                    LIMIT=(*!sky_model).free_free.limit, $
                    T_e=(*!sky_model).free_free.T_e
        ;transform into MJy/sr - I assume that the above map was in microK RJ 
        free_free_map = free_free_map * CONVERSION_FACTOR('uK_RJ','MJy/sr',freq*1e9)
     end
  endcase


  RETURN,free_free_map
exit:

END
