; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jonathan Aumont
;   Jacques Delabrouille
;   Francois-Xavier Desert
;   Lauranne Fauvet
;   Juan Macias-Perez
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	LOS_BFIELD_BSS_TURB
;
; PURPOSE:
;	calculate the Stokes parameters associated to the polarized
;	diffuse Galactic emissions
;	...
;	...
;
; CATEGORY:
;	Galaxy
;
; CALLING SEQUENCE:
;	LOS_BFIELD_BSS_TURB, kpc, l, b, los, r0, B0, Beta, p, ki0, ki_height_scale, B_turb, A, Bnlt
;
; INPUTS:
;	kpc:	...
;
;	l:	...
;
;	b:	...
;
;	los:	...
;
; OPTIONAL INPUTS:
;	none
;	
; KEYWORD PARAMETERS:
;	none
;
; OUTPUTS:
;	...:	...
;
;	...:	...
;
;	...:	...
;
;
; OPTIONAL OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;	...
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	...
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	15/03/2012:	Lauranne Fauvet and Jacques Delabrouille,
; 	                First version, adapted from original code
; 	                described in Fauvet et al. 2011.
;-
;--------PROGRAMME LOS_BFIELD_BSS_TURB.PRO---------------------
;
;Same as LOS_BFIELD with a BSS Magnetic Field Model.
;
;---used routines:
;
;
;---inputs:
;
;kpc=distance units.
;l=longitude (radians).
;b=latitude (radians).
;los=array(3*npts) containing the carthesian coordinates of each point of the line of sight.
;r0,B0,Beta,p,ki0,ki_height_scale=Bfield model parameters.
;
;---outputs:
;
;Bnlt: array(3*npts) containing the Bn,Bt,Bs components of B field at each point of the line of sight.
;Possible to uncomment if you need: Bxyz: array(3*npts) containing the Bx,By,Bz components of B field at each point of the line of sight.
;
;---coordinates:
;
;Main axis are X,Y,Z.
;Solar system is at the origine, Z axis is along the galactic north direction and Galacitc center is on X axis at x=8.0 kpc.
;
;We also use l(longitude) and b(latitude), the equator is in the Galactic plane, l starts from X axis and origine is Sun.
;
;Then we use plane of polar base. en is along los, et is also perpendicular to z axis and the third axis is es
;



PRO LOS_BFIELD_BSS_TURB,kpc,l,b,los,r0,B0,Beta,p,ki0,ki_height_scale,B_turb, A,Bnlt

;------------- Input parameters and definitions ------------------------
;Galaxy
dsun=8.*kpc
;Galaxy limits
exterior_limit=20.*kpc
interior_limit=3.*kpc

delta_L = 25.0d0/512.0d0

;------------ normalisation --------------------------------
; d'aprés Han et al, 2006
;R_B = 8.5d0*kpc
;R = sqrt((los(0,*)-dsun)*(los(0,*)-dsun)+los(1,*)*los(1,*)+los(2,*)*los(2,*))*kpc
;B_reg = B0*2.1d-6*exp(-(R-dsun)/R_B)

; Modele constant
B_reg = 1.0d0

;---------------------- Calculation ------------------------
phi=Atan(los(1,*),(los(0,*)-dsun))
ki=ki0*tanh(los(2,*)/ki_height_scale)

GalCenter=where(sqrt(((los(0,*)-dsun)*(los(0,*)-dsun))+(los(1,*)*los(1,*))) lt interior_limit)
GalExterior=where(sqrt(((los(0,*)-dsun)*(los(0,*)-dsun))+(los(1,*)*los(1,*))) gt exterior_limit)


;Bnlt
M=cos(ki)*cos(phi+Beta*Alog(sqrt(((los(0,*)-dsun)*(los(0,*)-dsun))+(los(1,*)*los(1,*)))/r0))

Bnlt(0,*)= B_reg*(M*cos(b)*sin(p+phi-l)+sin(ki)*sin(b)) 
Bnlt(1,*)= -B_reg*M*cos(p+phi-l)
Bnlt(2,*)= B_reg*(M*sin(b)*sin(p+phi-l)-sin(ki)*cos(b))



Bnlt(0,*) = Bnlt(0,*) + A*B_turb(abs(los(0,*)/25.*511.),abs(los(1,*)/25.*511.),abs(los(2,*)/25.*511.))
Bnlt(1,*) = Bnlt(1,*) + A*B_turb(abs(los(1,*)/25.*511.),abs(los(0,*)/25.*511.),abs(los(2,*)/25.*511.))
Bnlt(2,*) = Bnlt(2,*) + A*B_turb(abs(los(2,*)/25.*511.),abs(los(1,*)/25.*511.),abs(los(0,*)/25.*511.))

;Bturbls = B_turb(abs(los(0,*)/25.*511.),abs(los(1,*)/25.*511.),abs(los(2,*)/25.*511.))

;file='/archeops/fauvet/plot_B_turb.ps'
;!x.margin=[10,5]
;set_plot, 'ps'
;device, filename=file, /color, xsize=25, ysize=15
;plot,  Bnlt(0,*), title='B field turbulent' 
;oplot, Bturbls, col=100
;device, /close
;set_plot, 'x'

;power_spec, Bnlt(0,*),10000./25., pw, nu
;power_spec, Bturbls,10000./25., pw_turb, nu

;file='/archeops/fauvet/plot_pw_turb.ps'
;!x.margin=[10,5]
;device, filename=file, /color, xsize=25, ysize=15
;plot, nu, pw, /xlog, /ylog, title='power spectrum B turbulent'
;oplot, nu, pw_turb, col=100
;device, /close
;set_plot, 'x'


if (size(Galcenter,/N_DIMENSIONS) eq 1) then Bnlt(*,Galcenter)=0.0
if (size(GalExterior,/N_DIMENSIONS) eq 1) then Bnlt(*,Galexterior)=0.0


end

