FUNCTION psm_spinning_dust,pix_num,freq

;+
; NAME:
;    psm_spinning_dust
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes spinning dust grain emission at given position(s) and frequency
; CALLING SEQUENCE:
;    vec=psm_spinning_dust(pix_num,freq)
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              intialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = spinning dust grain contribution at given pixels and frequency (in MJy/sr)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    None
; OUTPUT KEYWORDS:
;    None
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_spinning_dust(findgen(12L*Nside^2),100.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
;    Undefined pixels in the template are assumed not to contribute
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Marc-Antoine Miville-Deschenes : March 15 2007
;    Changed call to unit conversion, Jacques Delabrouille, december 02 2009
;-

IF n_params() EQ 0 THEN BEGIN
  print,'vec=psm_spinning_dust(pix_num,freq)'
  goto, exit
ENDIF

modnum =  (*!sky_model).spinning_dust.modelnumber 
case modnum of 
   1: begin      ; Juvela model
      vec = spin_dust(pix_num, freq)
   end
   2: begin      ; DL98 model scaled by E(B-V) converted to NH 
;      vecold = dl98_spin_dust(pix_num, freq)
      dust_nh=(*(*!sky_model).spinning_dust.sfd_ebv)(pix_num)
      dust_nh = dust_nh * 8.0e21 ; convert to n_h units (atoms/cm^2)
      ind=where(dust_nh EQ !indef,count)
      IF count NE 0 THEN dust_nh(ind)=0.
      model_dust = dl98_spectrum(freq, composition=(*!sky_model).spinning_dust.composition)
      vec = dust_nh * model_dust
      vec=vec*1.e-6 ;change units to MJy/sr
   end
   3: begin      ; Extrapolate MAMD2008 Model 2 with DL98 model
;      vec = mamd2008_spin_dust(pix_num, freq)
      spin23=(*(*!sky_model).spinning_dust.SPIN23GHZ)(pix_num)
      ;convert_mk_mjy, 3.e14/23./1.e9, 1., sig_Mjy,/CMB
      sig_Mjy = CONVERSION_FACTOR('mK_CMB','MJy/sr',2.3e10)
      spin23 = spin23 * sig_MJy
      model_dust = dl98_spectrum(freq, nuref=23., composition=(*!sky_model).spinning_dust.composition)
      vec = spin23 * model_dust
   end
   4: begin      ; Extrapolate MAMD2008 Model 4 with DL98 model
;      vec = mamd2008_spin_dust(pix_num, freq)
      spin23=(*(*!sky_model).spinning_dust.SPIN23GHZ)(pix_num)
      ;convert_mk_mjy, 3.e14/23./1.e9, 1., sig_Mjy,/CMB
      sig_Mjy = CONVERSION_FACTOR('mK_CMB','MJy/sr',2.3e10)
      spin23 = spin23 * sig_MJy
      model_dust = dl98_spectrum(freq, nuref=23., composition=(*!sky_model).spinning_dust.composition)
      vec = spin23 * model_dust
   end
   5: begin      ; Extrapolate MAMD2008 Model 4 with DL98 model using user-defined parameters for composition
;      vec = mamd2008_spin_dust(pix_num, freq)
      spin23=(*(*!sky_model).spinning_dust.SPIN23GHZ)(pix_num)
      ;convert_mk_mjy, 3.e14/23./1.e9, 1., sig_Mjy,/CMB
      sig_Mjy = CONVERSION_FACTOR('mK_CMB','MJy/sr',2.3e10)
      spin23 = spin23 * sig_MJy
      model_dust = dl98_spectrum(freq, nuref=23., composition=(*!sky_model).spinning_dust.composition)
      vec = spin23 * model_dust
   end
endcase

RETURN,vec

exit:

END
