; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COMPONENT_STRUCT
;
; PURPOSE:
;	Create a structre describing a component in the PSM
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	result = COMPONENT_STRUCT(e1 , e2, e3, e4, e5, e6, e7, e8, e9)
;
; INPUTS:
;	None
;
; OPTIONAL INPUTS:
;	E1...E9:	Names of emission laws used to describe the
;        	        emission process. Can be replaced by keyword
;        	        EMLAW_LIST (see below)
;	
; KEYWORD PARAMETERS:
;	EMLAW_LIST:	String array listing the emission laws used to
;	                model the emission
;	                (e.g. ['powerlaw','greybody'] for sources with
;	                radio and thermal emission, or
;	                ['greybody','greybody'] for thermal dust emission with
;	                two populations of grains, etc...) 
;
; OUTPUTS:
;	A structure describing a component emission
;
; SIDE EFFECTS:
;	Note: creates a structure that contains pointers. Beware
;	memory leaks.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	18/11/2009:	JD, Added polarisation info in component structure.
; 	18/12/2009:	JD, obsolete COMP_PARAM keyword desactivated.
; 	12/02/2010:	JD, systematically include frequency range and
; 	                amplitude for emission laws. Also add
; 	                component class (diffuse, pointsource,
; 	                compactsource). 
; 	15/02/2010:	JD, modified case when component is of class pointsource
; 	22/04/2010:	JD, modified handling of polarisation
; 	26/04/2010:	Guillaume Castex, implemtation of elliptical sources
;-
FUNCTION COMPONENT_STRUCT, e1, e2, e3, e4, e5, e6, e7, e8, e9, emlaw_list=emlaw_list, name=name, id=id, class=class, $
                           polarised=polarised, elliptic=elliptic, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'COMPONENT_STRUCT: function to create a structure for a PSM component'
     PSM_INFO_MESSAGE, 'SYNTAX: result = COMPONENT_STRUCT(e1, e2, e3, e4, ..., e9, emlaw_list=, name=, id=, $'
     PSM_INFO_MESSAGE, '                                  class=, /polarised, polar_model=)'
     PSM_INFO_MESSAGE, "e1, ... e9 are up to 9 emission laws, which can be either emission law structures, or emission law names"
     PSM_INFO_MESSAGE, "optionnally, emission laws can be passed using the emlaw_list keyword (instead of e1, e2, ...)"
     PSM_INFO_MESSAGE, "emlaw_list: array of string (emission law names for all emission processes of the sources)"
     PSM_INFO_MESSAGE, "'name' is the component name (default empty string)"
     PSM_INFO_MESSAGE, "set 'id' to the id key of the component (default: generated by the code)"
     PSM_INFO_MESSAGE, "class can be 'diffuse' (the default), 'pointsource', 'or 'compactsource'"
     PSM_INFO_MESSAGE, "set /polarised for a polarised component"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT KEYWORD_SET(class) THEN class = 'diffuse'
  creation_string = "comp = {name:'',type:'comp',id:''"

  CASE STRLOWCASE(class) OF
     'diffuse': BEGIN
        creation_string = creation_string + ",class:'diffuse'"
        creation_string = creation_string + ",polarised:0B"
     END
     'pointsource': BEGIN
        creation_string = creation_string + ",class:'pointsource'"
        creation_string = creation_string + ",nsources:LONG64(0L)"
        creation_string = creation_string + ",objectname:PTR_NEW(/allocate_heap)"
        lon = {value:PTR_NEW(/allocate_heap),unit:'rad',info:'galactic longitude'}
        creation_string = creation_string + ",lon:lon"
        lat = {value:PTR_NEW(/allocate_heap),unit:'rad',info:'galactic latitude'}
        creation_string = creation_string + ",lat:lat"
        IF KEYWORD_SET(polarised) NE 0 THEN BEGIN
           creation_string = creation_string + ",polarised:1B"
           creation_string = creation_string + ",polar_model:'GLOBAL'"
           creation_string = creation_string + ",pol_fraction:PTR_NEW(/allocate_heap)"
           creation_string = creation_string + ",pol_angle:PTR_NEW(/allocate_heap)"
        ENDIF ELSE BEGIN
           creation_string = creation_string + ",polarised:0B"
        ENDELSE
        IF KEYWORD_SET(elliptic) NE 0 THEN BEGIN
           creation_string = creation_string + ",elliptic:1B"
           creation_string = creation_string + ",el_fwhm_maj:PTR_NEW(/allocate_heap)"
           creation_string = creation_string + ",el_fwhm_min:PTR_NEW(/allocate_heap)"
           creation_string = creation_string + ",el_theta:PTR_NEW(/allocate_heap)"
        ENDIF ELSE BEGIN
           creation_string = creation_string + ",elliptic:0B"
        ENDELSE
     END
     'compactsource': BEGIN
        creation_string = creation_string + ",class:'compactsource'"        
        creation_string = creation_string + ",polarised:0B"
     END
  ENDCASE

  creation_string = creation_string + ",nlaw:nlaw"


  IF N_PARAMS() GT 0 THEN BEGIN

     IF KEYWORD_SET(emlaw_list) NE 0 THEN MESSAGE, "Cannot call COMPONENT_STRUCT with both 'e1, e2,...' inputs and 'emlaw_list' keyword"

     nlaw=0
     IF DEFINED(e1) THEN BEGIN
        IF ISSTRUCT(e1) THEN creation_string = creation_string + ',e1:e1' ELSE creation_string = creation_string + ',e1:EMLAW_PARAMSTRUCT(e1,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e2) THEN BEGIN
        IF ISSTRUCT(e2) THEN creation_string = creation_string + ',e2:e2' ELSE creation_string = creation_string + ',e2:EMLAW_PARAMSTRUCT(e2,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e3) THEN BEGIN
        IF ISSTRUCT(e3) THEN creation_string = creation_string + ',e3:e3' ELSE creation_string = creation_string + ',e3:EMLAW_PARAMSTRUCT(e3,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e4) THEN BEGIN
        IF ISSTRUCT(e4) THEN creation_string = creation_string + ',e4:e4' ELSE creation_string = creation_string + ',e4:EMLAW_PARAMSTRUCT(e4,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e5) THEN BEGIN
        IF ISSTRUCT(e5) THEN creation_string = creation_string + ',e5:e5' ELSE creation_string = creation_string + ',e5:EMLAW_PARAMSTRUCT(e5,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e6) THEN BEGIN
        IF ISSTRUCT(e6) THEN creation_string = creation_string + ',e6:e6' ELSE creation_string = creation_string + ',e6:EMLAW_PARAMSTRUCT(e6,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e7) THEN BEGIN
        IF ISSTRUCT(e7) THEN creation_string = creation_string + ',e7:e7' ELSE creation_string = creation_string + ',e7:EMLAW_PARAMSTRUCT(e7,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e8) THEN BEGIN
        IF ISSTRUCT(e8) THEN creation_string = creation_string + ',e8:e8' ELSE creation_string = creation_string + ',e8:EMLAW_PARAMSTRUCT(e8,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF
     IF DEFINED(e9) THEN BEGIN
        IF ISSTRUCT(e9) THEN creation_string = creation_string + ',e9:e9' ELSE creation_string = creation_string + ',e9:EMLAW_PARAMSTRUCT(e9,/include_nurange,/include_ampl)'
        nlaw=nlaw+1
     ENDIF

     
  ENDIF ELSE BEGIN
     nlaw = N_ELEMENTS(emlaw_list)
     FOR i=1, nlaw DO BEGIN
        IF ISSTRUCT(emlaw_list(i-1)) THEN creation_string = creation_string + ',e'+ number2string(i) +': emlaw_list('+number2string(i-1)+')' ELSE creation_string = creation_string + ',e' + NUMBER2STRING(i) + ":EMLAW_PARAMSTRUCT('" + emlaw_list(i-1) + "',/include_nurange,/include_ampl)"
     ENDFOR
  ENDELSE

  creation_string = creation_string + '}'
  ok = EXECUTE(creation_string)
  IF NOT ok THEN MESSAGE, 'An error occured in the creation of the component structure'

  IF DEFINED(name) THEN comp.name = name
  IF DEFINED(id) THEN comp.id = id
  IF KEYWORD_SET(polarised) NE 0 THEN comp.polarised = 1B

  RETURN, comp

END
