; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	RESTORE_COMP
;
; PURPOSE:
;	This procedure is an I/O tool for restoring a PSM component
;	structure previously saved to file, hiding the technical
;	detail of the format of the component representation, as well
;	as the format and location of the file saved and the name of
;	the saved structure. The component structure must have been
;	saved to file with with SAVE_COMP.
;
; CATEGORY:
;	SKY_MODEL
;
; CALLING SEQUENCE:
;	RESTORE_COMP, Comp
;
; INPUTS:
;	None
;
; COMMON BLOCKS:
;	COMPNAME:	Name of the component to be restored. The list
;	                of valid PSM components is obtained with the
;	                PSM_ALLCOMP function.
;
; OUTPUTS:
;	Comp:	A sky emission component structure
;
; SIDE EFFECTS:
;	Creates a file in the current PSM output directory
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 	18/09/2009:	Jacques Delabrouille, First version.
; 	02/12/2009:	JD, Added sophistication to handle directory
; 	                in file names, which must be handled at the
; 	                PSM param COMMON level and not here.
; 	18/12/2009:	JD, check that compname is a valid PSM
; 	                component, + details about where it is stored.
; 	01/04/2010:	JD, Cosmetics, HELP keyword, + details
; 	06/01/2012:	JD, doc and PSM license header.
;-
PRO RESTORE_COMP, comp, compname=compname, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'RESTORE_COMP: procedure to restore a PSM component'
     PSM_INFO_MESSAGE, 'SYNTAX: RESTORE_COMP, comp, compname='
     PSM_INFO_MESSAGE, "set psm_output_dir to the PSM output directory if needed (default read from PSM param)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  COMMON psm, param

  PSM_DEBUG_MESSAGE, 'Revise restoring the parameters if needed', routine='RESTORE_COMP', infolimit=2, stoplimit=3

  ;; IF NOT DEFINED(param) THEN BEGIN
  ;;    IF KEYWORD_SET(psm_output_dir) NE 0 THEN RESTORE_PSM_PARAM, psm_output_dir=psm_output_dir $
  ;;    ELSE BEGIN
  ;;       MESSAGE, "Path to component not found -- set the 'psm_output_dir' keyword or define PSM parameters", /info
  ;;       RETURN
  ;;    ENDELSE
  ;; ENDIF ELSE BEGIN
  ;;    currentparam = TEMPORARY(param)
  ;;    IF KEYWORD_SET(psm_output_dir) NE 0 THEN RESTORE_PSM_PARAM, psm_output_dir=psm_output_dir
  ;;    IF PSM_CAREFULNESS() GE 1 THEN BEGIN
  ;;       ;; Check whether the restored parameters match the saved ones
  ;;       IF DEFINED(param) THEN ok = STRUCT_EQUAL(currentparam, param) ELSE STOP
  ;;       IF NOT ok THEN STOP
  ;;    ENDIF
  ;; ENDELSE

  ;; Check that compname is defined
  ;;--------------------------------
  IF NOT DEFINED(compname) THEN MESSAGE, 'Name of component to be restored is not defined'

  ;; Check that compname is the name of a valid PSM component
  ;;----------------------------------------------------------
  allcomp = PSM_ALLCOMP()
  wh = WHERE(allcomp EQ compname, nwh)
  IF nwh NE 1 THEN MESSAGE, "No such PSM component, '+compname'"

  ;; Find file in which the component is saved
  ;;-------------------------------------------
  filename = GETPSMDIR(compname)+compname+'.sav'
  exists = FILE_TEST(filename)

  ;; If no such file, then make a void component
  ;;---------------------------------------------
  IF NOT exists THEN BEGIN
     comp = COMPONENT_STRUCT(name=compname)
     PSM_INFO_MESSAGE, 'Component does not exist'
     PSM_DEBUG_MESSAGE, 'Check what is going on', routine='RESTORE_COMP', infolimit=1, stoplimit=2

  ;; Otherwise, restore the component
  ;;----------------------------------
  ENDIF ELSE BEGIN
     RESTORE, filename=filename
     IF comp.name NE compname THEN MESSAGE, 'Unexpected component restored!' 
     
     ;; Add component directory in file names written 
     
     FOR i=1, comp.nlaw DO BEGIN
        ok = EXECUTE('em = comp.e'+NUMBER2STRING(i))
        IF NOT ok THEN MESSAGE, 'Cannot execute command'
        params = EMLAW_PARAMLIST(em.law)
        params = ['AMPL',params]
        taglist = TAG_NAMES(em)
        FOR j=0, N_ELEMENTS(params)-1 DO BEGIN
           wh = WHERE(taglist EQ params[j],nwh)
           IF nwh EQ 1 THEN BEGIN
              ok = EXECUTE('value = em.'+params[j]+'.value')
              IF NOT ok THEN MESSAGE, 'Cannot execute command'
              s = SIZE(*value, /type)
              IF s EQ 7 THEN *value = GETPSMDIR(comp.name) +*value
              IF NOT ISARRAY(*value) THEN BEGIN
                 IF ISFITSFILE(*value) THEN BEGIN
                    ok = EXECUTE('*(comp.e'+NUMBER2STRING(i)+'.'+params[j]+'.value) = *value')
                    IF NOT ok THEN MESSAGE, 'Cannot execute command'
                 ENDIF
              ENDIF ELSE BEGIN
                 FOR k=0L, N_ELEMENTS(*value)-1 DO BEGIN
                    IF ISFITSFILE((*value)[k]) THEN BEGIN
                    ok = EXECUTE('(*(comp.e'+NUMBER2STRING(i)+'.'+params[j]+'.value))[k] = (*value)[k]')
                    IF NOT ok THEN MESSAGE, 'Cannot execute command'
                    ENDIF
                 ENDFOR
              ENDELSE
           ENDIF
        ENDFOR
     ENDFOR
  ENDELSE     

  PSM_DEBUG_MESSAGE, 'Make sure that param is also restored upon an error in the execution of restore_comp', routine='RESTORE_COMP', infolimit=2, stoplimit=3
  IF DEFINED(currentparam) THEN param=currentparam

END 
