; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
;+
; Function to create a catalogue of sources (with data in a set of bands)
; 
; @categories ps
; 
; @param nsources {in}{required}{type=long64} Number of sources in the catalogue 
; @param band_list {in}{required}{type=array of structures or of pointers to structures} bands in which the flux is given 
; @keyword polarised {in}{optional}{type=boolean} Set this keyword to make a catalogue of polarised sources (default = not polarised)
;
; @history <p>30/03/2009: First version, Jacques Delabrouille</p>
; @history <p>17/04/2009: polar added in the header, Jacques Delabrouille</p>
; @history <p>18/04/2009: full set of fields and units now in the header, Jacques Delabrouille</p>
; @history <p>18/02/2010: Fully rewritten, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS (2009)
;-
FUNCTION CREATE_PS_DATA_CAT, nsources, band_list, name=name, id=id, polarised=polarised, help=help;, allocate_memory=allocate_memory

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CREATE_PS_DATA_CAT: function to create a point source data catalog'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CREATE_PS_DATA_CAT(nsources, band_list, name=, id=, /polarised)'
     PSM_INFO_MESSAGE, "nsources is the number of sources in the catalogue"
     PSM_INFO_MESSAGE, "band_list: array of structures or of pointers to structures (bands in which the flux is given)"
     PSM_INFO_MESSAGE, "'name' is the catalogue name (default empty string)"
     PSM_INFO_MESSAGE, "set 'id' to the id key of the catalogue (default: generated by the code)"
     PSM_INFO_MESSAGE, "set /polarised for a polarised catalogue"
     ;PSM_INFO_MESSAGE, "set /allocate_memory to create empty arrays for all observations"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  nobs = N_ELEMENTS(band_list)

  IF NOT KEYWORD_SET(name) THEN name=''
  IF NOT KEYWORD_SET(id) THEN PSM_ID_KEYGEN, id
  IF NOT KEYWORD_SET(polarised) THEN polarised=0 ELSE polarised=1

  creation_string = "cat = {name:name,type:'obs',id:id,polarised:polarised,class:'pointsource',nsources:LONG64(nsources)"
  creation_string = creation_string + ",objectname:PTR_NEW(STRARR(nsources),/allocate_heap)"
  lon = {value:PTR_NEW(DBLARR(nsources),/allocate_heap),unit:'rad',info:'galactic longitude'}
  creation_string = creation_string + ",lon:lon"
  lat = {value:PTR_NEW(DBLARR(nsources),/allocate_heap),unit:'rad',info:'galactic latitude'}
  creation_string = creation_string + ",lat:lat"

  creation_string = creation_string + ",nobs:nobs"

  FOR i=1, nobs DO BEGIN
     s = SIZE(band_list[i-1],/type)
     IF s EQ 8 THEN BEGIN
        band = band_list[i-1]
     ENDIF ELSE BEGIN
        IF s EQ 10 THEN band = *(band_list[i-1]) ELSE MESSAGE, 'Case not implemented'
     ENDELSE
     obsstring = "obs"+NUMBER2STRING(i)+" = {band:PTR_NEW(band,/allocate_heap)"
     obsstring = obsstring + ",iflux:PTR_NEW(/allocate_heap)"
     obsstring = obsstring + ",ierr:PTR_NEW(/allocate_heap)"
     IF KEYWORD_SET(polarised) NE 0 THEN BEGIN
        obsstring = obsstring + ",qflux:PTR_NEW(/allocate_heap)"
        obsstring = obsstring + ",qerr:PTR_NEW(/allocate_heap)"
        obsstring = obsstring + ",uflux:PTR_NEW(/allocate_heap)"
        obsstring = obsstring + ",uerr:PTR_NEW(/allocate_heap)"
     ENDIF
     obsstring = obsstring + ",unit:''"
     obsstring = obsstring+"}"
     ok = EXECUTE(TEMPORARY(obsstring))
     IF NOT ok THEN STOP
     creation_string = creation_string + ",obs"+NUMBER2STRING(i)+":obs"+NUMBER2STRING(i)
  ENDFOR

  creation_string = creation_string+"}"
  ok = EXECUTE(creation_string)
  IF NOT ok THEN STOP

  RETURN, cat

END


  
  

