; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2006
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  BETA_PROFILE
;
; PURPOSE:
;  compute the y and tau values at distances r_array*core radii
;  from the cluster centers
;
; CALLING SEQUENCE:
;  result = BETA_PROFILE(z_array, M_array, r_array,
;           cosmo=cosmo,
;           r_cut=r_cut, beta=beta)
;
; INPUT:
;  z_array : the array containing cluster redshifts z
;  M_array : the array containing cluster masses M (Mpc)
;  r_array : the points (in rc) where you want to compute tau and y
;
; OUTPUT:
;  The catalog of clusters with
;      result(i).tau_array[j] : tau computed for the cluster i
;                               at distance r_array[j]*rc[i] from the center
;      result(i).y_array[j] : y computed for the cluster i
;                             at distance r_array[j]*rc[i] from the center
;      result(i).rad_array[j] : r_array[j]*rc[i] (in Mpc)
;
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  r_cut : the radius (in number of rc) where you stop your beta model
;            default : r_cut = 10.
;  beta : the factor beta in the beta model (e.g. 0.66)
;            default : beta = 2./3
; EXAMPLES:
;
; RESTRICTIONS:
;  Report any bugs to Jacques Delabrouille
;  delabrouille@apc.univ-paris7.fr
;
; REVISION HISTORY
;  Written, Jean-Baptiste Melin April & May 2001
;  Te in keV, JBM, May 30 2001
;  ne0 in cm^(-3), May 31 2001
;  general case for r_cut <> 10 in beta_function implemented, June 6 2001
;  lambda removed, Sept 24 2001
;  Keyword cosmo introduced, JBM, Sept 27 2001
;  cluster structure added, JBM, February 3 2006
;-

;------------------------------------------------------------------------------
FUNCTION BETA_FUNCTION, theta
;----------------------------
 COMMON BETA_FUNCTION_PARAMETERS, beta_function_theta_min, beta_function_beta, $
                                  beta_function_r_cut

 RETURN, (1+(beta_function_r_cut*sin(beta_function_theta_min)/sin(theta))^2)^ $
         (-3*beta_function_beta/2.)/(sin(theta))^2

END
;------------------------------------------------------------------------------

FUNCTION BETA_PROFILE, z_array, M_array, r_array, $
                         cosmo=cosmo, cluster=cluster, $
                         r_cut=r_cut, beta=beta

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'CALL IS : result = BETA_PROFILE(z_array, M_array, r_array)'
 PRINT,'Keywords : cosmo=cosmo, cluster=cluster, r_cut=r_cut, beta=beta'
 GOTO, closing
ENDIF

Mpc2m = 3.0856775807d+22 ; in m
c_light = 2.9979246d+08 ; in m/s
electron_mass = 0.51099906*1d3 ; in keV
sigma_thomson = 0.665245854d-28 ; in m^2

; format the result
step_nb = N_ELEMENTS(r_array)
cluster_nb = N_ELEMENTS(z_array)
amas = {y_array:DBLARR(step_nb),tau_array:DBLARR(step_nb), $
        rad_array:DBLARR(step_nb)}
result = REPLICATE(amas,cluster_nb)

IF NOT KEYWORD_SET(r_cut) THEN r_cut=10.
IF NOT KEYWORD_SET(beta) THEN beta=2./3

; compute the temperatures, the core radii and the central densities
; of the clusters
catalog = CLUSTER_MODEL(z_array, M_array, cosmo=cosmo, cluster=cluster, $
          /Te, /ne0, /rc, r_cut=r_cut, beta=beta)

; give the value of beta for each subfunction called
COMMON BETA_FUNCTION_PARAMETERS, beta_function_theta_min, beta_function_beta, $
                                 beta_function_r_cut
beta_function_beta = beta
beta_function_r_cut = r_cut
COMMON CLUSTER_MODEL_PARAMETER, cluster_model_beta
cluster_model_beta = beta

FOR i=0, step_nb-1 DO BEGIN
  delta = r_array[i]
  beta_function_theta_min = ASIN(delta/r_cut)
  IF delta EQ 0. THEN integrale = QROMB('subfunc_model',0.,r_cut) $
  ELSE integrale = QROMB('beta_function', beta_function_theta_min, !pi/2.)* $
                   delta
 result.tau_array[i]=2*catalog.ne0*integrale*catalog.rc*sigma_thomson*Mpc2m*1d6
 result.y_array[i]=result.tau_array[i]*catalog.Te/electron_mass
 result.rad_array[i]=delta*catalog.rc
ENDFOR

RETURN, result

closing:
END
