; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CLUSTER_LUMX2TEMP
;
; PURPOSE:
;	Compute the cluster temperature given its X-ray luminosity in
;	an Einstein-de Sitter model, using Markevitch 1998 Lx-T relation. 
;
; CATEGORY:
;	Sky model (clusters)
;
; CALLING SEQUENCE:
;	result  = CLUSTER_LUMX2TEMP(Lx)
;
; INPUTS:
;	Lx:	The x-ray luminosity (0.1-2.4keV) in 10^44 erg/s for an Einstein-de Sitter model
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	The output is the  temperature in keV deduced from the Markevitch Lx-T relation.
;
; SIDE EFFECTS:
;	None
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille and Jean-Baptiste Melin
;       delabrouille@apc.univ-paris7.fr
;       jean-baptiste.melin@cea.fr
;
; MODIFICATION HISTORY:
; 	08/03/2007:	Jean-Baptiste Melin, First version, inculded in
; 	                the ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG routine.
; 	08/01/2012:	Jacques Delabrouille, now standalone routine, doc and PSM license header.
;-
FUNCTION CLUSTER_LUMX2TEMP, lx

; From Markevitch 1998, ApJ 504:27-34

  h =0.5
  A6 = 1.71
  alpha = 2.02
  
  tx = 6.*(lx*h^2/A6)^(1./alpha) ; keV
  
  PSM_DEBUG_MESSAGE, 'We should check the validity of this relation for non Einstein - de Sitter universes -- if useful', routine='LUMX2TEMP', infolimit=2, stoplimit=3
  
  RETURN, tx
  
END
