; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marcella Massardi
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;******************************************************************************************
; NAME:
;    cluster_rs_contamination
; PURPOSE:
;   Computes radio sources contamination of Sunyaev-Zel'dovich (SZ) effect
;	in galaxy clusters at a given frequency
; CALLING SEQUENCE:
;    cluster_rs_contamination, file_in, file_out, nside, nu
; INPUT PARAMETERS:
;	 file_in = name of the galaxy clusters catalogue that contains
; 			the following columns in the following order (any format is allowed):
;			- ID of the cluster (no blank space is allowed)
;			- redshift
;			- virial radius [Mpc]
;			- SZ flux [Jy]
;			- "l" galactic longitude [degrees]
;			- "b" galactic latitude  [degrees]
;   nside = HEALPix resolution for output map
;   nu  = frequency at which calculation is to be done [GHz]
; OPTIONAL INPUT PARAMETERS
; KEYWORDS
;   param_cp: cosmological parameters
; OUTPUT:
;	Total intensity maps of the fluxes of faint radiosources in galaxies clusters. That is a
;	component of the contamination of SZ signal from clusters. It is extimated
;	according to the luminosity function by Magliocchetti et al. 2002 for steep
; 	sources in the field and, following Massardi et al.2003, corrected to reproduce
;	the case in clusters. For each resolved cluster (angular dimension < 5') the
;	distribution of the total contaminating flux is evaluated on the basis of the
;	correlation function of radiosources in clusters (Massardi et al. 2003).
;	A catalogue of the pixel positions and fluxes for all the clusters is also produced.
;
; EXAMPLES:
;   I maps with nside= 2048 and catalogue of mapped pixels of the cluster catalogue
;	"cat_in" at 70 GHz
;
;     	        cluster_rs_contamination,'cat_in',  2048, 70.
;
;	This procedure will produce the .fits map
;		'I_szrscontamination_70GHz_MJy_sr_2048.fits'
;	and the catalogue
;		'I_szrscontamination_70GHz_MJy_sr_2048_pixelcat.fits'
;
; REVISION HISTORY:
;    may 5, 2006, edited to match the format of the WG2
;    july 17, 2007, now uses the cosmological parameters of the PSM run, passed in keyword using param_cp, Jacques Delabrouille
;    july 17, 2007, replaced !pi by !dpi in all pointing computation, Jacques Delabrouille
;    july 17, 2007, uses GET_LUN and FREE_LUN for i/o, JD
;    july 18, 2007, modified for inclusion into PSM. Gets cluster catalog as input, componentdir in keyword, Jacques Delabrouille
;    july 19, 2007, first modifications for optimisation of computing speed
;    NOTE: This code still requires serious revision
;******************************************************************************************

;----------------------------------------
; Set of functions to estimate the angular dimension with the cosmological parameters set
; in the main program

function tointegrate_dm, var
 	common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
	tointegrate_dm = 1./(omega_m*(1.+var)^3+omega_r*(1+var)^2+omega_lambda)
	return,  tointegrate_dm
end

function dm
	; comoving distance  (Hogg 2000)
	common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
	H0cm 		= h*10./pc
	qtrap, 'tointegrate_dm', 0.,z, integr ,max_iter=40, eps=1.e-7
	if (omega_r eq 0) then begin
		dm = c/H0cm* integr
	endif
	if (omega_r gt 0) then begin
		dm = c/H0cm* omega_r^(-0.5)* sinh(omega_r^(0.5)*integr	/(c/H0cm))
	endif
	if (omega_r lt 0) then begin
		dm = c/H0cm* ABS(omega_r)^(-0.5)* sinh( ABS(omega_r)^(0.5)*integr	/(c/H0cm))
	endif
	return, dm
end
;----------------------------------------
;----------------------------------------
; set of functions used to integrate the luminosity function for steep sources by Magliocchetti et al. 2002
function flum_magliocchetti, logL

	common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
	L=10^logL
	n0=10^(-6.91) * (h/0.5)^3 *1.D0					;correction for H0
	a=0.69
	b=2.17
	logLstar=(32.12)+ALOG10((0.5/h)^2)*1.D0+ALOG10(4*!pi)		;correction for H0
	flum_magliocchetti=n0* L/(10^((logL-logLstar)*a)+10^((logL-logLstar)*b))
	return, flum_magliocchetti
end

function int_flum_magliocchetti, logL_min, logL_max

	common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
	integr=0.d0
	qtrap, 'flum_magliocchetti', logL_min, logL_max, integr ,max_iter=25 , eps=1.e-7
	int_flum_magliocchetti = integr
	return, int_flum_magliocchetti

end
;----------------------------------------
;----------------------------------------
; set of functions used to integrate the correlation function for cluster sources by Massardi et al. 2003

function fcorr_r, var
	; parameters of the correlation function are r0=0.7 Mpc, b=1.65
	fcorr_r=(1+var^2/0.7^2)^(-1.5*1.65+0.5)
	return, fcorr_r
end

function int_fcorr, rv
	common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
	qtrap, 'fcorr_r', 0.0, rv, integr ,max_iter=25 , eps=1.e-7
	int_fcorr = 2*!pi * integr
	return, int_fcorr
end

function fcorr_xy_pix, dist
	fcorr_xy_pix=(1+(dist)^2/0.7^2)^(-1.5*1.65+0.5)
	return, fcorr_xy_pix
end
;----------------------------------------
;
;**********************************************************************************************
;!!!!!!!!!!MAIN PROGRAM!!!!!!!!!!
;
pro cluster_rs_contamination, catalog, nside, nu, param_cp = param_cp, componentdir = componentdir

	if (n_params() lt 2) then begin
	    PRINT, 'Wrong number of arguments in cluster_rs_contamination'
	    print, 'Syntax : '
	    print, 'cluster_rs_contamination, catalog, nside, nu[GHz]'
	    return
	endif

	if (nside gt 2048) then begin
	    print,'Sorry, the nside for the reference sky is 2048 and cannot be increased, exiting.'
	    return
	endif

	print,'Generating cluster contaminating radio sources total intensity map in MJy/sr at',nu,' GHz in HEALPix format at Nside',nside,'.'
	print,'Generating cluster contaminating radio sources total intensity catalogue of 1.5arcmin-sided-pixel positions and fluxes in Jy at',nu,' GHz.'


	; definition of main cosmology and physical constants passed through the programs

        common par_all, c, pc, omega_m, omega_r, omega_lambda, h, z, xmin, xmax, ymin, ymax
                                ;here we change things to use the defined cosmology
                                ;stored in the param variable of the PSM run
	;omega_m		= 0.24
	;omega_lambda           = 0.76
	;omega_r		= 0.0
	;h			= 0.73
	;c 			= 29979245800.	 	;cm/s
        omega_m		        = param_cp.omega_m
	omega_lambda            = param_cp.lambda
	omega_r		        = 1. - omega_m - omega_lambda
	h		        = param_cp.h
        c 		        = 100 * !const.c                ;cm/s
        pc  		        = 3.08567802E18                 ;cm

	; definition of parameters fixed for contamination analysis

	S_max_real	= 2.5d-26	; flux limit of the catalogues of real radiosources in the RefSky to integrate only the
						; "faint" part of the luminosity function for real clusters
  	logL_min	= 28.5d0	; limit to sources luminosity to fix the lower integration extreme forthe luminosity function
	alpha		= 0.75
        alpha_steep = 0.87d0            ; spectral index for steep radiosources in clusters (Massardi et al. 2003)
   	within_corr = 3000.		; ratio between luminsity function of field and cluster radiosources (Massardi et al. 2003)

	;line=' '
	;riga=strarr(7,1)

	;close, 1
        ;close, 2
                                ; here it is better not to close units
                                ; instead we open free units
        ;GET_LUN, unit1
        GET_LUN, unit2

	;openr, unit1, file_in		;input file
        ncl = N_ELEMENTS(catalog)
        struct_inputs = REPLICATE({id:'',z:0.,rv:0.,lgal:0.,bgal:0.},ncl)

	file_out= fix_separator(componentdir)+'I_rsszcontamination_'+strtrim(nu, 2)+'GHz_MJy_sr_'+strtrim(nside,2)+'_pixelcat.txt'
	openw, unit2, file_out		;output file

	Map=fltarr(12L*nside*nside)
	Map[*]=0.d0

	print, 'Opening the input catalogue'

	;count=0
	;ncl=0.d0

        ;things moved out of the loop
	S_max = S_max_real
        oneminusalpha = 1.-alpha
        ;----------------------------

	;While (eof(unit1) ne 1) do begin

        FOR clus = 0L, ncl-1 DO BEGIN
           print, 'cluster number ',clus
		id	= catalog(clus).name		   	   ;cluster identification
                z	= catalog(clus).z                          ;redshift
                ;rv	= catalog(clus).rv                         ;virial radius in Mpc
                rv	= catalog(clus).rc*10                      ;virial radius in Mpc
                lgal    = catalog(clus).lon                        ;galactic longitude
                bgal    = catalog(clus).lat                        ;galactic latitude
                theta   = catalog(clus).tc*10                      ;virial radius in arcminutes

                ;dmm= dm()                                            ; transverse comoving distance in cm
                ;othertheta = rv* 1.D6 *pc *(1+z)/dmm * 180.*60./!pi  ; angular radius in arcmin
                ;PRINT, 'angles', theta, othertheta                   ; to check agreement

                oneplusz = 1+z
                ;dmm = rv* 1.D6 *pc *(1+z)/theta * 180.*60./!pi 
                dmm = rv * 3.4377467e+09 * pc * oneplusz / theta      ;optimize
                                ;fantastic! non need to recalculate dmm, we have it already in the cluster catalog !! (JD)

		;S_max = S_max_real

                ;d_lum=dmm*(1+z)                                 ;luminosity distance in cm
                d_lum=dmm*oneplusz                               ;luminosity distance in cm


                ;logL_max = alog10(S_max * 4. * !pi * d_lum^2 /(1+z)^(1.-alpha))
                logL_max = alog10(S_max * 12.566371 * d_lum^2 / oneplusz^oneminusalpha)
		; for each cluster the luminosity function by Magliocchetti et al. 2002 is integrated
                ;STOP
		flum= int_flum_magliocchetti(logL_min, logL_max)


		; luminosity/volume is converted into flux by multiplying for the cluster volume and corrected
		; for the ratio between the luminosity function of field sources (Magliocchetti et al. 2002) and
		; the luminosity function of cluster sources (=3000 see Massardi et al. 2003)

		;cont_steep = flum*(4./3.*!pi * rv^3)*within_corr/(4 * !pi * d_lum^2)*(1+z)^(1.-alpha)*1.E23
		cont_steep = flum*(4.1887903 * rv^3)*within_corr/(12.566371 * d_lum^2)*oneplusz^oneminusalpha*1.E23
		cont_steep_nu=cont_steep*(nu/1.4)^(alpha_steep)
                ;print, cont_steep

		If(cont_steep gt 0.d0) then begin

			; case of not resolved clusters :ID, positions and fluxes are catalogued
			If (theta lt 2.5) then begin

				;ang2pix_ring, nside, (90.-bgal)/180.*!PI, lgal/180.*!PI, ipring_gal
                                
                                ipring_gal = cat(clus).pixel
				for i=0L, n_elements(ipring_gal)-1 do begin
					j=ipring_gal[i]
					Map[j]=Map[j]+cont_steep_nu
					printf, unit2, 	id,ipring_gal[i], cont_steep_nu
				endfor

			endif

			; case of resolved clusters : the cluster is pixelized and the cluster ID the pixels
			; positions and fluxes are catalogued
			If (theta gt 2.5) then begin

				integr_tot=int_fcorr(rv)
				;bb=0.d0
				;ll=0.d0
				;bb=(90d0 - bgal)/180.*!DPI
				;ll=lgal/180d0*!DPI
				;ang2pix_ring, nside, bb, ll, ipring
                                ipring = cat(clus).pixel

				pix2vec_ring, nside, ipring, vector

				th=theta/60d0*!DPI/180d0
				query_disc , nside, vector, th, listpix, nlist


				for i=1, n_elements(listpix)-1 do begin
					pix2ang_ring, nside, listpix[i], bbpix, llpix
					;dist=0.d0


					;dist=ACOS(SIN(!dpi/2.-bbpix)*SIN(!dpi/2.-bb)+COS(!dpi/2.-bbpix)*COS(!dpi/2.-bb)*COS(llpix-ll))
					dist=ACOS(COS(bbpix)*COS(bb)+SIN(bbpix)*SIN(bb)*COS(llpix-ll))

					If(dist(0) LT th(0))Then begin
						dist=dist*dmm/(1+z)/(1.E6*pc)
						fcorr_pix=fcorr_xy_pix(dist)
						cont_steep_pix_nu=cont_steep_nu *fcorr_pix
						;integr_pix=int_fcorr_xy()
						;cont_steep_pix_nu=cont_steep_nu *integr_pix/integr_tot

						;print, cont_steep_pix_nu
						j=listpix[i]
						Map[j]=Map[j]+cont_steep_pix_nu

						printf, unit2, id,listpix[i], cont_steep_pix_nu
					Endif

				endfor

			Endif
		Endif
		;count=count+1



	ENDFOR
 	;close, unit1 & free_lun, unit1
	close, unit2 & free_lun, unit2

	print, 'No problem in reading and analyzing the input catalogue'

	Map=Map *1.e-6/(4.*acos(-1.)/(12.*nside*nside)) ; Jy ---> MJy/sr

	map_name=fix_separator(componentdir)+'I_szrscontamination_'+strtrim(nu,2)+'GHz_MJy_sr_nside'+strtrim(nside,2)+'.fits'

	write_fits_map,map_name, Map,/ring,coord='G',units='MJy/sr'
	print, max(Map), min(Map)

	print,'The final fits map has been produced'

; to produce the ps file of the map and visualisation
;-----------------------------------------------------

	map_out= fix_separator(componentdir)+'I_szrscontamination_'+strtrim(nu,2)+'GHz_MJy_sr_nside'+strtrim(nside,2)+'.eps'
;	map_out_gnome_norot= 'I_szrscontamination_'+strtrim(nu,2)+'GHz_MJy_sr_nside'+strtrim(nside,2)+'_gnome_norot.ps'
;	map_out_gnome_45= 'I_szrscontamination_'+strtrim(nu,2)+'GHz_MJy_sr_nside'+strtrim(nside,2)+'_gnome_45.ps'
;	map_out_gnome_pole= 'I_szrscontamination_'+strtrim(nu,2)+'GHz_MJy_sr_nside'+strtrim(nside,2)+'_gnome_pole.ps'

;	mollview, map_name, ps=map_out, /log
;	gnomview, map_name , rot=[0,0] , ps=map_out_gnome_norot ;, /log
;	gnomview, map_name , rot=[0,45] , ps=map_out_gnome_45 ;, /log
;	gnomview, map_name , rot=[0,90] , ps=map_out_gnome_pole ;, /log

; end of program
;----------------
;	print, 'the end'
	print, 'radiosource contamination in galaxy clusters at frequency '+STRTRIM(STRING(nu),1)+' GHz added'

end

