; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Reads the specific parameters used for the generation of the CMB in the PSM
;
; @categories pipe
;   
; @param table {in}{required}{type=strarr(*,2)} vector of couple (keyword, value)
; @param sz_seed {in}{required}{type=long} seed value
; @param szp {out}{required}{type=struct} cmb code specific parameters
; @keyword carefulness {in}{optional}{type=integer} level of carefulness in handling missing or repeated keywords in the table. 
;
; @uses fill_struct
; 
; @history <p>08/04/2009: First version, Jacques Delabrouille, Jean-Baptiste Melin </p>
; @history <p>28/04/2009: Removed unnecessary intermediate 'model' structure 
;                         + added output file names in structure, Jacques Delabrouille </p>
; @history <p>19/05/2009: Some minor bugs fixed, Jean-Baptiste Melin </p>
; @history <p>22/05/2009: Keyword for carefulness added, improved handling of missing or repeated keywords, Jacques Delabrouille</p>
; @history <p>18/09/2009: Now is a function + restore keyword added, Jacques Delabrouille</p>
; @history <p>04/02/2011: Adapted to be compatible with last updates of the SZ models, JBM</p>
;-
FUNCTION READ_SZ_PARAM, table, carefulness=carefulness, restore=restore

  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'sz_parameters.sav'

  IF NOT KEYWORD_SET(restore) THEN BEGIN

     which_sz = {sz_model: 'no_sz'}  
     FILL_STRUCT, table, which_sz, tags=['SZ_MODEL'], carefulness=carefulness
     
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; SZ pipeline parameters
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     CASE which_sz.sz_model OF  ; {'nbody+hydro','dmb','hydro+dmb'}
        'nbody+hydro': BEGIN
           szp = { $
                 model:which_sz.sz_model, $           ; {'nbody+hydro'}
                 sz_include_thermal:'yes', $          ;
                 sz_include_kinetic:'yes', $          ;
                 sz_include_polarised:'no', $         ; not read for this model
                 sz_relativistic:0, $                 ;
                 sz_firs_contamination:'no', $        ; contamination by far infrared sources; now, not read
                 sz_radio_contamination:'no', $       ; contamination by radio sources; now, not read
                 save_sz_map:'yes', $
                 save_sz_alm:'yes', $
                 save_sz_catalogue:'yes' $
                 }
           tags = 'SZ_'+['INCLUDE_THERMAL','INCLUDE_KINETIC']
           FILL_STRUCT, table, szp, tags = tags, carefulness=carefulness
           
        END
        'dmb': BEGIN
           szp = { $
                 model:which_sz.sz_model, $           ; {'dmb'}
                 sz_include_thermal:'yes', $          ;
                 sz_include_kinetic:'yes', $          ;
                 sz_include_polarised:'no', $         ;
                 mass_function:'tinker', $            ; {'jenkins','sheth_tormen','press_schechter','evrard','tinker'}
                 sz_constrained:'yes', $              ; {'yes','no'}
                 sz_relativistic:0, $                 ;
                 sz_input_cat:STRARR(10), $           ;
                 cluster_profile:'xmm', $             ; {'xmm', 'chandra' or 'beta'}
                 nstd_profile:'yes', $                ; {'yes', 'no'}
                 norm_profile:'yes', $                ; normalisation taken on observation (yes) or from the model (no)
                 profile_bounds:'5r500', $            ; bounds of the (xmm or chandra) cluster profile is 5r500 (ie 5 times r500) or rvir (top-hat)
                 cluster_m_inf:0.1, $                 ; limit mass in 10^15 solar masses, default 0.1 = 10^14 solar masses.
                 cluster_t_star:1.48, $               ; normalisation of the mass-temperature relation for clusters
                 sz_firs_contamination:'no', $        ; contamination by far infrared sources; now, not read
                 sz_radio_contamination:'no', $       ; contamination by radio sources; now, not read
                 save_sz_map:'yes', $
                 save_sz_alm:'yes', $
                 save_sz_catalogue:'yes' $
                 }
           tags = 'SZ_'+['INCLUDE_THERMAL','INCLUDE_KINETIC','INCLUDE_POLARISED']
           tags = [tags, 'MASS_FUNCTION', 'SZ_CONSTRAINED', 'SZ_RELATIVISTIC', 'SZ_INPUT_CAT', 'CLUSTER_PROFILE', 'NSTD_PROFILE', 'NORM_PROFILE', 'PROFILE_BOUNDS', 'CLUSTER_M_INF']
           tags = [tags, 'CLUSTER_T_STAR']
           FILL_STRUCT, table, szp, tags = tags, carefulness=carefulness
           IF szp.cluster_profile EQ 'beta' AND szp.profile_bounds EQ '5r500' THEN BEGIN
             PSM_INFO_MESSAGE, "WARNING: CLUSTER_PROFILE='beta' and PROFILE_BOUNDS='5r500', PROFILE_BOUNDS is set to 'rvir'"
             szp.profile_bounds='rvir'
           ENDIF
           IF STRLEN(szp.sz_input_cat[0]) EQ 0 THEN BEGIN
             szp.sz_input_cat[0:1]=['rosat','sdss']
	   ENDIF
        END
        'hydro+dmb': BEGIN
           szp = { $
                 model:which_sz.sz_model, $           ; {'hydro+dmb'}
                 sz_include_thermal:'yes', $          ;
                 sz_include_kinetic:'yes', $          ;
                 sz_include_polarised:'no', $         ;
                 mass_function:'tinker', $            ; {'jenkins','sheth_tormen','press_schechter','evrard','tinker'}
                 sz_relativistic:0, $                 ;
                 cluster_profile:'xmm', $             ; {'xmm', 'chandra' or 'beta'}
                 nstd_profile:'yes', $                ; {'yes', 'no'}
                 norm_profile:'yes', $                ; normalisation taken on observation (yes) or from the model (no)
                 profile_bounds:'5r500', $            ; bounds of the (xmm or chandra) cluster profile is 5r500 (ie 5 times r500) or rvir (top-hat)
                 cluster_m_inf:0.1, $                 ; limit mass in 10^15 solar masses, default  0.1 = 10^14 solar masses.
                 cluster_t_star:1.48, $               ; normalisation of the mass-temperature relation for clusters
                 sz_firs_contamination:'no', $        ; contamination by far infrared sources; now, not read
                 sz_radio_contamination:'no', $       ; contamination by radio sources; now, not read
                 save_sz_map:'yes', $                 ;
                 save_sz_alm:'yes', $                 ;
                 save_sz_catalogue:'yes' $            ;
                 }
           tags = 'SZ_'+['INCLUDE_THERMAL','INCLUDE_KINETIC']
           tags = [tags, 'MASS_FUNCTION', 'SZ_RELATIVISTIC', 'CLUSTER_PROFILE', 'NSTD_PROFILE', 'NORM_PROFILE', 'PROFILE_BOUNDS', 'CLUSTER_M_INF']
           tags = [tags, 'CLUSTER_T_STAR']
           FILL_STRUCT, table, szp, tags = tags, carefulness=carefulness
           IF szp.cluster_profile EQ 'beta' AND szp.profile_bounds EQ '5r500' THEN BEGIN
             PSM_INFO_MESSAGE, "WARNING: CLUSTER_PROFILE='beta' and PROFILE_BOUNDS='5r500', PROFILE_BOUNDS is set to 'rvir'"
             szp.profile_bounds='rvir'
           ENDIF
        END 
        'prediction': BEGIN
           szp = { $
                 model:which_sz.sz_model, $                   ; {'prediction'}
                 sz_include_thermal:'yes', $                  ;
                 sz_include_kinetic:'no', $                   ; kinetic SZ not known for sample catalogue
                 sz_include_polarised:'no', $                 ;
                 sz_relativistic:0, $                         ;
                 sz_input_cat:STRARR(10), $                   ;
                 cluster_profile:'xmm', $                     ; cluster profile can be xmm, chandra or beta
                 nstd_profile:'yes', $                        ;
                 norm_profile:'yes',  $                       ; normalisation taken on observation (yes) or from the model (no)
                 profile_bounds:'5r500', $                    ; bounds of the (xmm or chandra) cluster profile is 5r500 (ie 5 times r500) or rvir (top-hat)
                 cluster_t_star:1.48, $                       ; normalisation of the mass-temperature relation for clusters
                 sz_firs_contamination:'no', $                ; contamination by far infrared sources; now, not read
                 sz_radio_contamination:'no', $               ; contamination by radio sources; now, not read
                 save_sz_map:'yes', $
                 save_sz_alm:'yes', $
                 save_sz_catalogue:'yes' $           ;
                 }
           tags = 'SZ_'+['INCLUDE_THERMAL']
           tags = [tags, 'SZ_INPUT_CAT', 'SZ_RELATIVISTIC', 'CLUSTER_PROFILE', 'NSTD_PROFILE', 'NORM_PROFILE', 'PROFILE_BOUNDS']
           tags = [tags, 'CLUSTER_T_STAR']
           FILL_STRUCT, table, szp, tags = tags, carefulness=carefulness
           IF szp.cluster_profile EQ 'beta' AND szp.profile_bounds EQ '5r500' THEN BEGIN
             PSM_INFO_MESSAGE, "WARNING: CLUSTER_PROFILE='beta' and PROFILE_BOUNDS='5r500', PROFILE_BOUNDS is set to 'rvir'"
             szp.profile_bounds='rvir'
           ENDIF
	   IF STRLEN(szp.sz_input_cat[0]) EQ 0 THEN BEGIN
             szp.sz_input_cat[0:1]=['rosat','sdss']
	   ENDIF
        END
        'no_sz': szp = {model:'no_sz'}
        ELSE: MESSAGE, 'SZ MODEL NOT ALLOWED'
     ENDCASE
  
     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, szp
     
  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE

  RETURN, szp
     
END
  

