; -----------------------------------------------------------------------------
;
; Copyright (C) 2001-2002
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	BETA_MODEL_MAP
; PURPOSE:
;	program to create a Compton parameter map or an optical depth map
;	(dimensions nb_pix*nb_pix) according to the "beta model"
;
; CALLING SEQUENCE:
;	result = BETA_MODEL_MAP(nb_pix, d_pix, y_or_tau_array, rad_array, hd_cluster)
; INPUT:
;	Nb_pix: 	The number of pixels along an axis.
;	D_pix:  	The pixel width (in Mpc).
;	y_or_tau_array:	An array of the Compton or tau values.
;	Rad_array:	An array of the corresponding radii in Mpc.
;
; OUTPUT:
;	hd_cluster:	The header of the cluster.
;	cluster_map:	The map of the cluster.
;
; KEYWORDS:
;	COSMO:	The structure containing the cosmological parameters.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY
;	April 2001:	Jean-Baptiste Melin, First version.
;	May 2001:	JBM & Jacques Delabrouille, Small speed-up.
;	May 2001:	JBM, Change in input format.	
;	May 2001:	JD, Header updated, name changed from "beta_map" to "beta_model_map".	
;	27/09/2001:	JBM, COSMO keyword introduced.
;	05/08/2002:	JBM, Change in comments, d_pix added.
;-
FUNCTION BETA_MODEL_MAP, nb_pix, d_pix, y_or_tau_array, rad_array, hd_cluster, $
                         cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : BETA_MODEL_MAP(nb_pix,d_pix,y_or_tau_array,rad_array,hd_cluster)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

; create the map of the cluster
map = FLTARR(nb_pix, nb_pix)

; compute the value of r_max radius
r_max = rad_array[N_ELEMENTS(rad_array)-1]

; create a distance-to-the-center map 
map = DIST_TO_CENTER(nb_pix)*d_pix
wh = WHERE(map GE r_max,nwh)
; transform the distance-to-the-center map in y parameter or tau map
map = INTERPOL(y_or_tau_array, rad_array, map)
; put every pixel with distance to center greater than r_max equal to 0
IF nwh GT 0 THEN map(wh) = 0.

; generate a header for the cluster map
MKHDR, hd_cluster, 4, [nb_pix,nb_pix]
SXADDPAR, hd_cluster, 'NAXIS1', nb_pix
SXADDPAR, hd_cluster, 'NAXIS2', nb_pix
SXADDPAR, hd_cluster, 'RAXIS1', nb_pix*d_pix*cosmo.h
SXADDPAR, hd_cluster, 'RAXIS2', nb_pix*d_pix*cosmo.h
SXADDPAR, hd_cluster, 'H0', 100.*cosmo.h
SXADDPAR, hd_cluster, 'OMEGA_M', cosmo.omega_m
SXADDPAR, hd_cluster, 'OMEGA_L', cosmo.omega_de

RETURN, map
closing:
END
