; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2002
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     PLACE_CLUSTER
; ONE LINE HELP:
;     program to place a cluster of given redshift on a map
; PURPOSE:
;     program to place a cluster (redshift z, orientation ang)
;     on a map at pixel x,y
; CALLING SEQUENCE:
;     PLACE_CLUSTER, cluster, map, x, y, z, ang, hd_cluster, hd_map
; INPUT:
;     cluster : the array of the cluster
;     map : the array of the map
;     x : x position of the center of the cluster on the map
;     y : y position of the center of the cluster on the map
;     z : the redshift
;     ang : angular orientation of the cluster (positive for anticlockwise)
;     hd_cluster : header of the cluster
;     hd_map : header of the map
;
; OUTPUT:
;     map : the array of the modified map
; KEYWORDS:
;      
; EXAMPLES:
;
; RESTRICTIONS:
;     Not fully tested yet. Report any bugs to Jacques Delabrouille
;     j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;     ANG2SIZE, SXPAR, PIXEL_NUMBER
; REVISION HISTORY
;     Written, Jean-Baptiste Melin April&May 2001
;     Modified to take account of small clusters, JBM, May 2001
;     Now in Mpc, JBM, June 1st 2001
;     Removed CONGRID that shifted the center of the clusters on the final map, JBM, Aug 6 2002
;-

;------------------------------------------------------------------------------
FUNCTION PIXEL_NUMBER, ratio_x, ratio_y, x, y, i, j
;--------------------------------------------------
; given the pixel number (i,j) of a map, this subroutine returns the pixel
; number in the final map (result[0], result[2]) and the surface of the pixel
; corresponding to each dimension result[1] and result[3]
;------------------------------------------------------------------------------

result= DBLARR(4)

;along x-axis
x1 = (x+0.5)+(i-0.5)*ratio_x ;x+i*ratio_x
k1 = FIX(x1)
k2 = FIX(x1+ratio_x)
result[0]=k1
IF k1 EQ k2 THEN result[1]=ratio_x ELSE result[1]= k2-x1

;along y-axis
y1 = (y+0.5)+(j-0.5)*ratio_y ;y+j*ratio_y
l1 = FIX(y1)
l2 = FIX(y1+ratio_y)
result[2]=l1
IF l1 EQ l2 THEN result[3]=ratio_y ELSE result[3]= l2-y1

RETURN, result
END
;------------------------------------------------------------------------------


PRO PLACE_CLUSTER, cluster, map, x, y, z, ang, hd_cluster, hd_map
; case of non square clusters still unresolved

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
print,'CALL IS : PLACE_CLUSTER(cluster, map, x, y, z, ang, hd_cluster, hd_map)'
GOTO, closing
ENDIF

Mpc2m = 3.0856775807e+22

; read the cosmological parameters from the header of the cluster
h = SXPAR(hd_cluster,'H0')/100.
omega_m = SXPAR(hd_cluster,'OMEGA_M')
lambda = SXPAR(hd_cluster,'OMEGA_L')
cosmo = {h:h, omega_m:omega_m, lambda:lambda}

; read the number of pixels along x and y axis for the cluster on the cluster
; map
npix_clustermap_x = SXPAR(hd_cluster,'NAXIS1')
npix_clustermap_y = SXPAR(hd_cluster,'NAXIS2')

; read the number of pixels along x and y axis for the map
npix_map_x = SXPAR(hd_map,'NAXIS1')
npix_map_y = SXPAR(hd_map,'NAXIS2')

; compute the angular dimensions of the map
ang_map_x = ABS(SXPAR(hd_map,'CDELT1')*npix_map_x)
ang_map_y = ABS(SXPAR(hd_map,'CDELT2')*npix_map_y)

; compute the angular dimensions of the cluster map on the map
ANG2SIZE, ang_cluster_x, SXPAR(hd_cluster,'RAXIS1')/h, z, /reverse, $
            cosmo=cosmo
ANG2SIZE, ang_cluster_y, SXPAR(hd_cluster,'RAXIS2')/h, z, /reverse, $
            cosmo=cosmo

; compute the dimension ratio between the two maps
ratio_x = (ang_cluster_x/npix_clustermap_x)/(ang_map_x/npix_map_x)
ratio_y = (ang_cluster_y/npix_clustermap_y)/(ang_map_y/npix_map_y)

; compute the real number of pixels for the cluster on the map (float)
npix_cluster_x = npix_map_x*ang_cluster_x/ang_map_x
npix_cluster_y = npix_map_y*ang_cluster_y/ang_map_y

; the coordinates of the left bottom corner of the cluster on the map
x_min = x-(npix_clustermap_x-1)/2.*ratio_x  ; (npix_cluster_x-1)/2.
y_min = y-(npix_clustermap_y-1)/2.*ratio_y  ; (npix_cluster_y-1)/2.

; rotate the cluster
IF ang NE 0 THEN new_cluster=ROT(cluster,-1.*ang,1.,/INTERP) ELSE new_cluster=cluster

; case when the cluster is smaller than 11*11 pixels on the final map
IF (npix_cluster_x LE 11.) OR (npix_cluster_y LE 11.) THEN BEGIN
  FOR i=0,npix_clustermap_x-1 DO BEGIN
    FOR j=0,npix_clustermap_y-1 DO BEGIN
      result = PIXEL_NUMBER(ratio_x, ratio_y, x_min, y_min, i, j)
      prod = result[1]*result[3]
      map[result[0],result[2]] = map[result[0],result[2]]+prod*new_cluster[i,j]
      IF NOT(prod EQ ratio_x*ratio_y) THEN BEGIN
        a = ((ratio_x-RESULT[1])*RESULT[3])*new_cluster[i,j]
        b = ((ratio_y-RESULT[3])*RESULT[1])*new_cluster[i,j]
        c = ((ratio_x-RESULT[1])*(ratio_y-RESULT[3]))*new_cluster[i,j]
        map[result[0]+1,result[2]]=map[result[0]+1,result[2]]+ a
        map[result[0],result[2]+1]=map[result[0],result[2]+1]+ b
        map[result[0]+1,result[2]+1]=map[result[0]+1,result[2]+1]+c
      ENDIF
    ENDFOR
  ENDFOR
ENDIF ELSE BEGIN ; case when the cluster is bigger than 11*11 pixels
npix_cluster_x = ROUND(npix_cluster_x)
npix_cluster_y = ROUND(npix_cluster_y)
add_to_map = new_cluster ;CONGRID(new_cluster, npix_cluster_x, npix_cluster_y,/INTERP)
result = PIXEL_NUMBER(ratio_x, ratio_y, x_min, y_min, 0, 0)
new_add_map = DBLARR(npix_cluster_x+1, npix_cluster_y+1)
new_add_map[0:npix_cluster_x-1,0:npix_cluster_y-1] = add_to_map*result[1]*result[3]
new_add_map[0:npix_cluster_x-1,1:npix_cluster_y] = new_add_map[0:npix_cluster_x-1,1:npix_cluster_y] + $
                                                   add_to_map*result[1]*(ratio_y-result[3])
new_add_map[1:npix_cluster_x,1:npix_cluster_y] = new_add_map[1:npix_cluster_x,1:npix_cluster_y] + $
                                                 add_to_map*(ratio_x-result[1])*(ratio_y-result[3])
new_add_map[1:npix_cluster_x,0:npix_cluster_y-1] = new_add_map[1:npix_cluster_x,0:npix_cluster_y-1] + $
                                                   add_to_map*(ratio_x-result[1])*result[3]
x_min = FIX(x_min)
y_min = FIX(y_min)
map[x_min:x_min+npix_cluster_x, y_min:y_min+npix_cluster_y] = $
map[x_min:x_min+npix_cluster_x, y_min:y_min+npix_cluster_y] + new_add_map ;add_to_map
ENDELSE

closing:
END
