; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MAKE_SORTED_BINS
;
; PURPOSE:
;	Permits to bin elements of an array in bins of increasing
;	values, with a specified  maximum number of elements per bin,
;	and a maximum factor between elements of the same bin. The
;	function returns an array of same size as the input, which
;	specifies in which bin is each of the elements. 
;
; CATEGORY:
;	Array handling
;
; CALLING SEQUENCE:
;	Result = MAKE_SORTED_BINS(Values, Numbermax, Valuefactormax)
;
; INPUTS:
;	Values: 	Input array of values
;	Numbermax:	Maximum number of elements per bin
;	Valuefactormax:	Maximum factor between elements of the same bin
;
; KEYWORD PARAMETERS:
;	NBINS:  	Number of bins used for binning the
;               	elements. Input keyword.
;
;	N_IN_BIN:	Array of size nbins, specifying the number of elements
;               	in each bin. Output keyword.
;
; OUTPUTS:
;	The function returns an array of same size as the input, which
;	specifies in which bin is each of the elements.
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	11/04/2009:	Jacques Delabrouille, First version.
;	05/01/2012:	JD, General clean-up + doc header update.
;-
FUNCTION MAKE_SORTED_BINS, values, numbermax, valuefactormax, nbins=nbins, n_in_bin=n_in_bin

  nv = N_ELEMENTS(values)
  bins = LONARR(nv)
  
  reorder = SORT(values)
  
  currentbin = 0L
  currentnumber = 1L
  currentvalue = values[reorder[0]]
  n_in_bin = [1]
  
  FOR i=1L, nv-1 DO BEGIN
     
     IF currentnumber LT numbermax THEN BEGIN
        IF values[reorder[i]]/currentvalue LE valuefactormax THEN BEGIN
           bins[reorder[i]] = currentbin
           currentnumber = currentnumber + 1L         
        ENDIF ELSE BEGIN
           n_in_bin[currentbin] = currentnumber
           n_in_bin = [n_in_bin,0]
           currentbin = currentbin + 1L
           bins[reorder[i]] = currentbin
           currentnumber = 1L
           currentvalue = values[reorder[i]]
        ENDELSE
     ENDIF ELSE BEGIN
        n_in_bin[currentbin] = currentnumber
        n_in_bin = [n_in_bin,0]
        currentbin = currentbin + 1L
        bins[reorder[i]] = currentbin
        currentnumber = 1L
        currentvalue = values[reorder[i]]
     ENDELSE
     
  ENDFOR
  
  nbins = currentbin + 1
  n_in_bin[currentbin] = currentnumber > n_in_bin[currentbin]
  RETURN, bins
  
END
