; -----------------------------------------------------------------------------
;
; Copyright (C) 2008-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_EMLAW
;
; PURPOSE:
;	Function to compute any emission law among the following: 
;	CMB, BLACKBODY, GREYBODY (with power law emissivity),
;	POWERLAW, CURVPOWERLAW, FREEFREE, SPINDUST, THERMALSZ, DIRAC.
;	The output is in brightness units (e.g. W/m2/sr/Hz), relative
;	to a reference value at a freference frequency (see
;	KEYWORDS). Spectral indices are assumed to 
;	properly describe the emission law in brightness units.  
;
; CATEGORY:
;	Emission law
;
; CALLING SEQUENCE:
;	result = PSM_EMLAW(Nu[, Paramstruct])
;
; INPUTS:
;	Nu:     	frequency at which the function should be computed (in
;	                Hz). May be an array. 
;
; OPTIONAL INPUTS:
;	Paramstruct:	A structure that provides all input parameter
;	                keywords (alternative way of passing to the
;	                function all input parameters, including
;	                required ones).
;	
; KEYWORD PARAMETERS:
;	Emission_name:	Name of the emission considered (Case
;	        	insensitive). Can be any among the following: 'CMB',
;	        	'BLACKBODY', 'GREYBODY', 'POWERLAW', 'CURVPOWERLAW',
;	        	'FREEFREE', 'SPINDUST', 'THERMALSZ', 'DIRAC', 
;	        	'RELATSZ1', 'RELATSZ2', 'RELATSZ3', 'RELATSZ4'
;	        	'SUBMMLAPI', 'SPIRALGAL', 'STARBURSTGAL'. 
;
;	NUREF:   	Reference frequency (Hz), at which the emission is
;	                normalised to 1 (or to AMPL, see next
;	                keyword). Required input keyword.
;
;	AMPL:   	Amplitude at reference frequency (default=1).
;
;	TEMP:    	Temperature. Used by 'BLACKBODY', 'GREYBODY',
;                	'FREEFREE' (in which case the temperature is
;                	in K) and by 'THERMALSZ' for relativistic
;                	corrections, (in which case the temperature is
;                	in keV). It is recommended to set this
;                	keyword, as default values are different for
;                	different emissions, and also may change. For
;                	SZ effect, a temperature of 0, or undefined,
;                	means that the non-relativistic limit is computed.
;
;	SPECIND:	Spectral index. Used by 'GREYBODY',
;	                'POWERLAW', 'CURVPOWERLAW'. Required input
;	                keyword. Warning, brightness units are assumed
;	                for the output, NOT brightness temperature.
;
;	CURVAMPL:	Value of the amplitude of the spectral index
;	                curvature. Used by 'CURVPOWERLAW'. Required input.  
;
;	CURVNUREF:	Value of the reference frequency (Hz) for the
;	                spectral index curvature. Used by 'CURVPOWERLAW'. Required input.
;
;	DLCOMP: 	Composition of the spinning dust according to
;	                Draine-Lazarian model. Optional (do not set it
;	                unless you know what you are doing).
;
;	REDSHIFT: 	Redshift at which the emisison law is computed.
;	                Used by 'SUBMMLAPI'
;
;	NUMAX:  	Maximum frequency of the emission. Default = Inf
;	                (No maximum frequency). Included, i.e. the
;	                emission does NOT vanish at Nu = NUMAX.
;
;	NUMIN:  	Minimum frequency of the emission. Default = 0
;	                (No minimum frequency). Excluded, i.e. the
;	                emission DOES vanish at Nu = NUMAX.
;
;	DOUBLE: 	Set this keyword to force double precision. 
;
;	ERROR:  	Output byte keyword giving the error status
;	                (0B, everything ok, 1B, an error occured). 
;
; OUTPUTS:
;	Value of the emission at the frequency Nu, relative to value
;	of emission at frequency nuref.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	12/08/2008:	Jacques Delabrouille, First version.
;	09/02/2009:	JD, Added RANGE keyword
;	19/04/2009:	JD, Use 'DEFINED' instead of KEYWORD_SET to allow 0 parameter values
;	22/04/2009:	JD, Free-Free emission law added
;	11/06/2009:	JD, Spinning dust emission law added
;	25/11/2009:	JD, Default temperature for Free-Free is now 7000K
;	18/12/2009:	JD, Now SZ emission is 'THERMALSZ'
;	15/02/2010:	JD, Check dimensions to avoid errors when both nu and emission law parameters are arrays
;	23/04/2010:	JD, Technical modifications for parameter input using 'Paramstruct'
;	27/05/2010:	JD, Added VALUEATNUREF keyword (replaces AMPL)
;	11/06/2010:	JD, Added DIRAC Emission Law
;	27/10/2010:	JD, Some additional checking of input parameters
;	09/04/2011:	JD, Some verification of dimensions of inputs
;	05/01/2012:	JD, General clean-up of doc header, + use CMB_TEMPERATURE() for CMB temperature
;	08/01/2012:	JD, Replace VALUEATNUREF keyword by AMPL...
;	                (to fix an inconsistency between calls made using paramstruct or using keywords...)
;	08/03/2012:	JD, Implement relativistic corrections to thermal SZ effect
;	09/03/2012:	JD & Guillaume Castex, Implement Lapi submillimeter SED
;	09/03/2012:	JD, Implement First order relativistic SZ
;		        correction as an independent emission law
;		        (RELATSZ1)
;       20/04/2012      GC, SPIRALGAL and STARBURSTGAL emission laws
;-

FUNCTION PSM_EMLAW, nu, paramstruct, emission_name=emission_name, $
                    nuref=nuref, ampl=ampl, $
                    temp=temp, specind=specind, curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, $
                    double=double, numin=numin, numax=numax, error=error, help=help
  
IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0) OR (N_PARAMS() GT 2)) THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_EMLAW: function to compute the value of an emission law at a frequency nu'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_EMLAW(nu, paramstruct, emission_name=, nuref=, ampl=, temp=, specind=, $'
     PSM_INFO_MESSAGE, '                           curvnuref=, curvampl=, dlcomp=, redshift=, double=, numin=, numax=, error=)'
     PSM_INFO_MESSAGE, "See header of the program for additional infromation"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


IF NOT DEFINED(numin) THEN numin=0
IF NOT DEFINED(numax) THEN numax=FLOAT('Inf')
error=0B

;; Test paramstruct is a structure, assign keyword values for all emission law parameters
;;----------------------------------------------------------------------------------------
  IF DEFINED(paramstruct) THEN BEGIN
     type = SIZE(paramstruct, /type)
     IF type NE 8 THEN MESSAGE, 'call to PSM_EMLAW has changed, please fix your codes'
     tagnames = TAG_NAMES(paramstruct)
     wh = WHERE(tagnames EQ 'NUMIN', nwh) & IF nwh GE 1 THEN include_nurange=1 ELSE include_nurange=0
     wh = WHERE(tagnames EQ 'AMPL', nwh) & IF nwh GE 1 THEN include_ampl=1 ELSE include_ampl=0
     tagnames = TAG_NAMES(paramstruct)
     paramlist = EMLAW_PARAMLIST(paramstruct.law, include_nurange=include_nurange, include_ampl=include_ampl)
     nparam = N_ELEMENTS(paramlist)
     FOR i=0, nparam-1 DO BEGIN        
        ok = EXECUTE('param = paramstruct.'+paramlist[i]+'.value')
        IF ISFITSFILE(*param) THEN BEGIN
           MESSAGE, 'CASE NOT IMPLEMENTED'
        ENDIF ELSE BEGIN 
           ok = EXECUTE(paramlist[i] +' = *param')
        ENDELSE
     ENDFOR
     IF NOT KEYWORD_SET(emission_name) THEN emission_name = paramstruct.law
  ENDIF ELSE BEGIN
     paramlist = EMLAW_PARAMLIST(emission_name)
     nparam = N_ELEMENTS(paramlist)
  ENDELSE
  
  FOR i=0, nparam-1 DO BEGIN
     ok = EXECUTE('test = ISSTRING('+paramlist(i)+')')
     IF test THEN MESSAGE, 'parameter '+ paramlist(i) +'cannot be a string'
  ENDFOR


;; Test nu is not a string
;;-------------------------
  IF DEFINED(nu) THEN BEGIN
     type = SIZE(nu, /type)
     IF type EQ 7 THEN MESSAGE, 'call to psm_emission_law has changed, please fix your codes'
  ENDIF

;; Check that dimensions are OK
;;------------------------------

  nnu = N_ELEMENTS(nu)
  IF nnu GT 1 THEN BEGIN
     checksize = PSM_EMLAW(nu[0], emission_name=emission_name, $
                    nuref=nuref, temp=temp, specind=specind, curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, redshift=redshift, $
                    double=double, numin=numin, numax=numax, error=error)
     IF error NE 0 THEN BEGIN
        GOTO, closing
     ENDIF
     IF N_ELEMENTS(checksize) NE 1 THEN BEGIN
        MESSAGE, 'Incompatible dimensions of inputs in call to function'
        GOTO, closing
     ENDIF
     IF ISARRAY(checksize) THEN BEGIN
        MESSAGE, 'Warning, scalar expected instead of array'
        GOTO, closing
     ENDIF
  ENDIF
  
  CASE STRUPCASE(emission_name) OF 
     'CMB': BEGIN   
        t_cmb = CMB_TEMPERATURE()
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PLANCK_DBNU_DT(nu, temp=t_cmb) / PLANCK_DBNU_DT(usenuref, temp=t_cmb)
     END
     'BLACKBODY': BEGIN
        IF NOT DEFINED(temp) THEN MESSAGE, 'temperature value needed for computing blackbody emission'
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PLANCK_BNU(nu, temp=temp) / PLANCK_BNU(usenuref, temp=temp)
     END
     'GREYBODY': BEGIN
        IF NOT DEFINED(nuref) THEN MESSAGE, 'reference frequency value needed for computing greybody emission'
        IF NOT DEFINED(specind) THEN MESSAGE, 'emissivity spectral index value needed for computing greybody emission'
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        ptrs = [ISPOINTER(specind),ISPOINTER(temp)]
        IF ptrs[0] THEN BEGIN
           IF ptrs[1] THEN res = (nu/usenuref)^(*specind) * (PLANCK_BNU(nu, temp=(*temp))/PLANCK_BNU(usenuref, temp=(*temp))) $
           ELSE res = (nu/usenuref)^(*specind) * (PLANCK_BNU(nu, temp=temp)/PLANCK_BNU(usenuref, temp=temp))
        ENDIF ELSE BEGIN
           IF ptrs[1] THEN res = (nu/usenuref)^(specind) * (PLANCK_BNU(nu, temp=(*temp))/PLANCK_BNU(usenuref, temp=(*temp))) $
           ELSE res = (nu/usenuref)^(specind) * (PLANCK_BNU(nu, temp=temp)/PLANCK_BNU(usenuref, temp=temp))
        ENDELSE
     END
     'POWERLAW': BEGIN
        IF NOT DEFINED(nuref) THEN MESSAGE, 'reference frequency value needed for computing power law emission'
        IF NOT DEFINED(specind) THEN MESSAGE, 'spectral index value needed for computing power law emission'
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(specind) THEN res = (nu/usenuref)^(*specind) ELSE res = (nu/usenuref)^specind
     END
     'CURVPOWERLAW': BEGIN
        IF NOT DEFINED(nuref) THEN MESSAGE, 'reference frequency value needed for computing curved power law emission'
        IF NOT DEFINED(specind) THEN MESSAGE, 'spectral index value needed for computing curved power law emission'
        IF NOT DEFINED(curvampl) THEN MESSAGE, 'amplitude of spectral index curvature term needed for computing curved power law emission'
        IF NOT DEFINED(curvnuref) THEN MESSAGE, 'reference frequency for spectral index curvature term needed for computing curved power law emission'
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(curvnuref) THEN usecurvnuref = *curvnuref ELSE usecurvnuref = curvnuref
        IF ISPOINTER(specind) THEN BEGIN
           IF ISPOINTER(curvampl) THEN res = (nu/usenuref)^((*specind) + (*curvampl)*ALOG10(nu/usecurvnuref)) $
           ELSE res = (nu/usenuref)^((*specind) + (curvampl)*ALOG10(nu/usecurvnuref))
        ENDIF ELSE BEGIN
           IF ISPOINTER(curvampl) THEN res = (nu/usenuref)^((specind) + (*curvampl)*ALOG10(nu/usecurvnuref)) $
           ELSE res = (nu/usenuref)^((specind) + (curvampl)*ALOG10(nu/usecurvnuref))
        ENDELSE
     END
     'FREEFREE': BEGIN
        IF NOT DEFINED(nuref) THEN MESSAGE, 'reference frequency value needed for computing spectral law of free-free emission'
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF NOT KEYWORD_SET(temp) THEN temp = 7000. ; Default is 7000 Kelvin
        IF KEYWORD_SET(double) NE 0 THEN nu2nuGHz = 1d-9 ELSE nu2nuGHz = 1e-9
        IF ISPOINTER(temp) THEN res = R2FF(nu*nu2nuGHz, T_E=(*temp))/R2FF(usenuref*nu2nuGHz, T_E=(*temp)) $
        ELSE res = R2FF(nu*nu2nuGHz, T_E=temp)/R2FF(usenuref*nu2nuGHz, T_E=temp)
        ;; convert to brightness units
        res = res*(nu/usenuref)^2
     END
     'SPINDUST': BEGIN
        IF NOT DEFINED(nuref) THEN BEGIN
           PSM_INFO_MESSAGE, 'reference frequency value needed for computing spinning dust law emission'
           error = 1B
           GOTO, closing
        ENDIF
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF KEYWORD_SET(double) NE 0 THEN nu2nuGHz = 1d-9 ELSE nu2nuGHz = 1e-9
        IF ISPOINTER(dlcomp) THEN res = DL98_SPECTRUM(nu*nu2nuGHz, nuref=usenuref*nu2nuGHz, composition=(*dlcomp)) $
        ELSE res = DL98_SPECTRUM(nu*nu2nuGHz, nuref=usenuref*nu2nuGHz, composition=dlcomp)
     END
     'THERMALSZ': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF NOT DEFINED(temp) THEN BEGIN
           ;; non relativistic thermal SZ
           res = CONVERSION_FACTOR('y_sz','W/m2/sr/Hz',nu) / CONVERSION_FACTOR('y_sz','W/m2/sr/Hz',usenuref)
        ENDIF ELSE BEGIN
           ;; relativistic thermal SZ
           ;; temp is assumed to be the electron temperature, in keV
           IF ISPOINTER(temp) THEN theta_e = *temp/510.99892 ELSE theta_e = temp/510.99892
           res = PSM_THERMALSZ(nu, order=4, theta_e=theta_e) / PSM_THERMALSZ(usenuref, order=4, theta_e=theta_e)
        ENDELSE
     END
     'RELATSZ1': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PSM_RELATSZ1(nu) / PSM_RELATSZ1(nuref)
     END
     'RELATSZ2': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PSM_RELATSZ2(nu) / PSM_RELATSZ2(nuref)
     END
     'RELATSZ3': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PSM_RELATSZ3(nu) / PSM_RELATSZ3(nuref)
     END
     'RELATSZ4': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        res = PSM_RELATSZ4(nu) / PSM_RELATSZ4(nuref)
     END
     'DIRAC': BEGIN
        IF NOT DEFINED(nuref) THEN MESSAGE, 'reference frequency value needed for computing dirac law emission'
        res = (nu EQ nuref)
     END
     'SUBMMLAPI': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(redshift) THEN useredshift = *redshift ELSE useredshift = redshift
        res = SUBMM_SED_LAPI(nu,redshift=useredshift) / SUBMM_SED_LAPI(nuref,redshift=useredshift)
     END
     'STARBURSTGAL': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(redshift) THEN useredshift = *redshift ELSE useredshift = redshift
        res = STARBURST_SED(nu,redshift=useredshift) / STARBURST_SED(nuref,redshift=useredshift)
     END
     'SPIRALGAL': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(redshift) THEN useredshift = *redshift ELSE useredshift = redshift
        res = SPIRAL_SED(nu,redshift=useredshift) / SPIRAL_SED(nuref,redshift=useredshift)
     END
     'SUBMMCAI': BEGIN
        IF ISPOINTER(nuref) THEN usenuref = *nuref ELSE usenuref = nuref
        IF ISPOINTER(redshift) THEN useredshift = *redshift ELSE useredshift = redshift
        res = SUBMM_SED_CAI(nu,redshift=useredshift) / SUBMM_SED_CAI(nuref,redshift=useredshift)
     END
     ELSE: MESSAGE, 'SORRY, ' + emission_name + ' emission not implemented'
  ENDCASE

  IF numin NE 0 THEN BEGIN
     keep = (nu GT numin)
     res = res*TEMPORARY(keep)
  ENDIF
  IF FINITE(numax) THEN BEGIN
     keep = (nu LE numax)
     res = res*TEMPORARY(keep)
  ENDIF
  
  IF nnu GT 1 THEN BEGIN
     IF N_ELEMENTS(res) NE nnu THEN MESSAGE, 'Problem with dimension of output'
  ENDIF

  IF DEFINED(ampl) THEN res = res*ampl

  IF ((KEYWORD_SET(double) NE 0) OR (PRECISION_ISDOUBLE())) THEN RETURN, DOUBLE(res) ELSE RETURN, FLOAT(res)

closing:
END
