; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ERASE_TMPFILES
;
; PURPOSE:
;	Procedure to erase temporary files generated by running PSM
;	pipelines or tools
;
; CATEGORY:
;	PSM tools
;
; CALLING SEQUENCE:
;	ERASE_TMPFILES
;
; INPUTS:
;	None
;
; OPTIONAL INPUTS:
;	None
;	
; KEYWORD PARAMETERS:
;	ALLPSM:	Set this keyword to erase all PSM temporary files
;		(from all PSM runs), instead of erasing only the
;		temporary files generated in the current run.
;
; OUTPUTS:
;	None
;
; OPTIONAL OUTPUTS:
;	None
;
; SIDE EFFECTS:
;	Files identified as PSM temporary files are erased from disk.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Get file list with function PSM_TMPFILES(), and erase them
;	with FILE_DELETE.
;
; MODIFICATION HISTORY:
; 	24/11/2009:	Jacques Delabrouille, First version.
;-
PRO ERASE_TMPFILES, allpsm=allpsm, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ERASE_TMPFILES: procedure to erase current PSM temporary files'
     PSM_INFO_MESSAGE, 'SYNTAX: ERASE_TMPFILES, /allpsm'
     PSM_INFO_MESSAGE, "If /allpsm is set,  the function erases all temporary files created by the PSM, in the"
     PSM_INFO_MESSAGE, "current or in other PSM runs, otherwise it erases only files created in the present session."
     PSM_INFO_MESSAGE, "/allpsm permits, for instance, to erase files remaining after a crash in a previous PSM run"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ;; Get all files
  files = PSM_TMPFILES(allpsm=allpsm)
  nf = N_ELEMENTS(files)

  IF nf EQ 1 THEN IF STRLEN(files[0]) EQ 0 THEN BEGIN
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, 'No temporary file to be erased'
     GOTO, closing
  ENDIF

  IF nf GE 1 THEN BEGIN
     IF PSM_VERBOSITY() GE 1 THEN BEGIN
        PSM_INFO_SUBTITLE, 'PSM temporary files to be erased'
        FOR i=0, nf-1 DO PSM_INFO_MESSAGE, files[i]
     ENDIF
  ENDIF

  ;; Erase them
  FOR i=0, nf-1 DO BEGIN
     IF FILE_TEST(files[i]) THEN BEGIN
        IF PSM_CAREFULNESS() GE 2 THEN BEGIN
           PSM_INFO_MESSAGE, 'File '+ files[i] + ' is about to be erased'
           PSM_INFO_MESSAGE, 'PSM run stopped. Type .c to erase the file and continue'
        ENDIF
        FILE_DELETE, files[i]
     ENDIF
  ENDFOR

closing:
END
