; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2011
;   Jacques Delabrouille
;   Guillaume Castex
;   Mathieu Remazeilles
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	APODIZE_MASK
;
; PURPOSE:
;	Procedure to apodize a 0-1 mask, for a smooth transition between the two regions.
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	APODIZE_MASK, Mask, Apmask
;
; INPUTS:
;	Mask:	Input mask (can be a file or a memory array)
;
; OPTIONAL INPUTS:
;	
; KEYWORD PARAMETERS:
;	LENGTH:	Input apodization length in radian (default is !pi),
;		unless /deg is set (default is 180)
;
;	DEG:	Sets /deg for an apodization length in degree 
;
;	NESTED:	If mask is a memory array, the ordering is assumed to
;		be 'RING' unless /nested is set
;
; OUTPUTS:
;	Apmask: Apodized mask
;
; OPTIONAL OUTPUTS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	This procedure uses the DISTANCE_TO_MASK procedure.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	25/01/2010:	Jacques Delabrouille, First version.
;	29/11/2011:     Guillaume Castex.
;	05/01/2012:     JD, PSM license header.
;	16/01/2012:     Mathieu Remazeilles, doc header.		
;-
PRO APODIZE_MASK, mask, apmask, length=length, deg=deg, nested=nested, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'APODIZE_MASK: procedure to apodize a 0-1 mask, for a smooth transition between the two regions'
     PSM_INFO_MESSAGE, 'SYNTAX: APODIZE_MASK, mask, apmask, length=length, /deg, /nested'
     PSM_INFO_MESSAGE, "mask is the input mask (can be a file or a memory array)"
     PSM_INFO_MESSAGE, "apmask is the output apodized mask"
     PSM_INFO_MESSAGE, "length is the input apodization length (in radian, unless /deg is set)"
     PSM_INFO_MESSAGE, "If mask is a memory array, the ordering is assumed to be 'RING' unless /nested is set"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF DEFINED(nested) EQ 0 THEN ordering='RING' ELSE ordering='NESTED'

  DISTANCE_TO_MASK, mask, dist, maxdist=length, deg=deg, ordering=ordering

  apmask = SIN((!pi/2.)*(TEMPORARY(dist)/length))
  apmask = TEMPORARY(apmask)^2

END
