; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2011
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FILL_BADPIX
;
; PURPOSE:
;	...
;
;-
PRO FILL_BADPIX, map, result, ordering=ordering, beam=beam, use_nest=use_nest, maxiter=maxiter, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'FILL_BADPIX: procedure to perform inpainting in a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: FILL_BADPIX, map, result, ordering=, beam=, /use_nest'
     PSM_INFO_MESSAGE, "'map' is the input map (array), containing bad values (-1.63750e+30) which will be filled."
     PSM_INFO_MESSAGE, "'result' is an optional output structure, where 'result.p' is the list of bad pixels"
     PSM_INFO_MESSAGE, "and 'result.map' the inpainted value in those pixel."
     PSM_INFO_MESSAGE, "Set 'ordering' to 'RING' [the default] or 'NESTED'."
     PSM_INFO_MESSAGE, "Set 'beam' to a FWHM (in arcmin) which defines the neighborhood used to fill the bad pixels."
     PSM_INFO_MESSAGE, "If 'beam' not set, the software uses a beam increasing with the distance inside the 'holes'."
     PSM_INFO_MESSAGE, "If 'use_nest' is set, 'beam' is disregarded, the code uses the nested structure of HEALPix"
     PSM_INFO_MESSAGE, "to fill the bad pixels."
     PSM_INFO_MESSAGE, "By default, FILL_BADPIX uses an increasing number of neighbouring pixels to fill regions"
     PSM_INFO_MESSAGE, "deeper in the hole. At each iteration (one row of pixels into the hole) it uses pixels at"
     PSM_INFO_MESSAGE, "up to 2 x pixel-diameter x iter. Set maxiter to any positive integer to limit the distance of"
     PSM_INFO_MESSAGE, "neighbouring pixels used."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'
  missing_pix = WHERE(map EQ !healpix.bad_value, nwh)

  IF ARG_PRESENT(result) THEN BEGIN
     zero = 0B*map[0]
     result = {p:missing_pix,map:REPLICATE(zero,nwh>1)}
  ENDIF
  IF nwh EQ 0 THEN RETURN

  nside = GET_NSIDE(map)
  
  IF (KEYWORD_SET(beam)) THEN BEGIN
     npix = NSIDE2NPIX(nside)
     omega=4*!pi/npix
     radius=2*acos(1-omega/(2*!pi))
     beamradiusradian = beam/60.*!dtor
     IF beamradiusradian LT radius THEN BEGIN
        PSM_INFO_MESSAGE, "Beam too small (beam is in arcmin)."
        GOTO, closing
     ENDIF
  ENDIF
  keep_going = (nwh GT 0)
  iter = 0

  WHILE keep_going DO BEGIN
     IF NOT KEYWORD_SET(beam) THEN BEGIN
        IF NOT KEYWORD_SET(use_nest) THEN BEGIN 
           IF NOT KEYWORD_SET(maxiter) THEN $
              ERODE_BADPIX, map, ordering=ordering, iter=iter+1 $
           ELSE ERODE_BADPIX, map, ordering=ordering, iter=((iter+1) < maxiter)
        ENDIF ELSE BEGIN
           UD_GRADE, map, mapp, order_in=ordering, order_out=ordering, nside_out = nside/2^iter, /pessimistic
           ERODE_BADPIX, mapp, ordering=ordering, use_nest=use_nest
           UD_GRADE, TEMPORARY(mapp), newmap, order_in=ordering, order_out=ordering, nside_out = nside
           map[missing_pix] = newmap[missing_pix]
        ENDELSE

     ENDIF ELSE BEGIN
        ERODE_BADPIX, map, ordering=ordering, beam=beam 
        ;STOP
     ENDELSE
     missing_pix = WHERE(map EQ !healpix.bad_value, nwh)
     keep_going = nwh GE 1
     iter = iter+1
     ;IF ((2^iter GT nside) AND (KEYWORD_SET(beam) EQ 0)) THEN BEGIN
     ;   STOP
     ;   map[missing_pix] = TBD
     ;   GOTO, closing
     ;ENDIF
  ENDWHILE

  IF ARG_PRESENT(result) THEN BEGIN
     result.map = map[result.p]
     ;map[result.p] = !healpix.bad_value
  ENDIF

closing:
END
