; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2007, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CARTESIAN2SPHERICAL
;
; PURPOSE:
;	Convert cartesian coordinates to spherical coordinates or
;	back. The angles are in degrees, unless the RADIAN keyword is set.
;
; CATEGORY:
;	Mathematics
;
; CALLING SEQUENCE:
;	CARTESIAN2SPHERICAL, Ci1, Ci2, Ci3, Co1, Co2, Co3
;
; INPUTS:
;	Ci1:	Cartesian coordinate x, or spherical coordinate r if
;	        /reverse is set.
;
;	Ci2:	Cartesian coordinate y, or spherical coordinate theta if
;	        /reverse is set.
;
;	Ci3:	Cartesian coordinate z, or spherical coordinate phi if
;	        /reverse is set.
;
; KEYWORD PARAMETERS:
;	REVERSE:	If set, convert spherical to cartesian instead.
;	RADIAN: 	Set this keyword for angles in radians.
;
; OUTPUTS:
;	Co1:	spherical coordinate r, or cartesian coordinate x if
;	        /reverse is set.
;
;	Co2:	spherical coordinate theta, or cartesian coordinate y if
;	        /reverse is set.
;
;	Co3:	spherical coordinate phi, or cartesian coordinate z if
;	        /reverse is set.
;
; RESTRICTIONS:
;	The calculation is done in double precision, regardless of the
;	type of the inputs.
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple mathematics.
;
; EXAMPLE:
;	Find the cartesian coordinates of the point with spherical
;	coordinates [r=5, theta=60., phi=120.]:
;
;	> CARTESIAN2SPHERICAL, 5, 60, 120, Co1, Co2, Co3
;
;	Check the result:
;	> PRINT, Co1, Co2, Co3
;
;	(IDL prints '-2.1650635       3.7500000       2.5000000')
;
; MODIFICATION HISTORY:
; 	19/05/2001:    	Jacques Delabrouille, First version.
;	July 2007:     	JD, Header information updated.
;	January 2012:	JD, doc header updated, psm license header added.
;-
PRO CARTESIAN2SPHERICAL, ci1,ci2,ci3,co1,co2,co3, reverse=reverse, radian=radian

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
	PRINT,'  CALL IS : CARTESIAN2SPHERICAL, ci1,ci2,ci3,co1,co2,co3'
	PRINT,'  Keywords : /reverse to convert spherical to cartesian'
     GOTO, closing
ENDIF

IF KEYWORD_SET(radian) EQ 0 THEN BEGIN 
;we compute in degrees
	dtor = !dpi/180d0
	IF KEYWORD_SET(reverse) EQ 0 THEN BEGIN
	;we go from cartesian (x,y,z) to spherical (r,theta,phi)
		co2 = ATAN(SQRT((ci1^2)+(ci2^2)),ci3)/dtor
		co3 = ATAN(ci2,ci1)/dtor
		co1 = SQRT((ci1^2)+(ci2^2)+(ci3^2))
	ENDIF ELSE BEGIN
	;we go from spherical (r,theta,phi) to cartesian (x,y,z) 
		co1 = ci1*SIN(ci2*dtor)*COS(ci3*dtor)
		co2 = ci1*SIN(ci2*dtor)*SIN(ci3*dtor)
		co3 = ci1*COS(ci2*dtor)
	ENDELSE
	
ENDIF ELSE BEGIN 
;we compute in radians
	IF KEYWORD_SET(reverse) EQ 0 THEN BEGIN
	;we go from cartesian (x,y,z) to spherical (r,theta,phi)
		co2 = ATAN(SQRT((ci1^2)+(ci2^2)),ci3)
		co3 = ATAN(ci2,ci1)
		co1 = SQRT((ci1^2)+(ci2^2)+(ci3^2))
	ENDIF ELSE BEGIN
	;we go from spherical (r,theta,phi) to cartesian (x,y,z) 
		co1 = ci1*SIN(ci2)*COS(ci3)
		co2 = ci1*SIN(ci2)*SIN(ci3)
		co3 = ci1*COS(ci2)
	ENDELSE
ENDELSE
closing:

END
