; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Soumen Basak
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_MAP2ALM
;
; PURPOSE:
;	...
;
;-
PRO PSM_MAP2ALM, map, alm, double=double, overwrite=overwrite, input_beam=input_beam, output_beam=output_beam, iter=iter, $
                 weighted=weighted, polar=polar, lmax=lmax, ordering=ordering, nohdr=nohdr, help=help

  ON_ERROR, 2

  IF NOT DEFINED(map) THEN PSM_INFO_MESSAGE, 'Input map not defined'

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_MAP2ALM: procedure to calculate alm coefficients from a map'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_MAP2ALM, map, alm, input_beam=input_beam, output_beam=, lmax= , ordering=, iter=, '
     PSM_INFO_MESSAGE, '                     /nohdr, /double, /overwrite, /weighted, /polar'
     PSM_INFO_MESSAGE, "input_beam is the structure describing the beam of the input map, if any (default is 0)."
     PSM_INFO_MESSAGE, "output_beam is the structure describing the output beam, if any."
     PSM_INFO_MESSAGE, "ordering ['RING' or 'NESTED'] is read from header if map is a fits file."
     PSM_INFO_MESSAGE, "It is assumed to be 'RING' by default if map is a memory array, unless set to 'NESTED'"
     PSM_INFO_MESSAGE, "Set 'iter' to the number of iterations requested for the alm transform (defaut=0)"
     PSM_INFO_MESSAGE, 'set /double to force double precision for output alm'
     PSM_INFO_MESSAGE, 'set /overwrite to overwrite any already existing output'
     PSM_INFO_MESSAGE, 'set /nohdr to skip the writing of the correct header in the alm file'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF
  
  IF GET_DEBUG() GE 2 THEN IF NOT DEFINED(lmax) THEN STOP

  maptype = SIZE(map, /type)

  ;; If input map is in file format
  ;;--------------------------------
  IF maptype EQ 7 THEN BEGIN
     ;; Test that the file exists
     exists = FILE_TEST(map)
     IF NOT exists THEN MESSAGE, 'Input map file not found'
     IF NOT DEFINED(lmax) THEN lmax = FITSFILE_LMAX(map)
     IF lmax EQ -1 THEN BEGIN
        lmax = 2L*GET_NSIDE(map)
     ENDIF
     mapfile = map
     erasemapfile = 0B          ; the input file will NOT be erased at the end
     IF DEFINED(input_beam) THEN BEGIN
        filebeam = FITSFILE_BEAM(mapfile)
        IF ISANUMBER(input_beam) THEN input_beam_struct=GET_BEAM_STRUCT('GAUSSIAN',beamsize=input_beam,sizeunits='arcmin') ELSE input_beam_struct=input_beam
        IF NOT STRUCT_EQUAL(filebeam, input_beam_struct) THEN BEGIN
           IF filebeam.type EQ 'UNKNOWN' THEN filebeam=input_beam_struct ELSE $
              MESSAGE, 'Problem: The input beam provided as a keyword is incompatible with what is read in the input file'
        ENDIF
     ENDIF ELSE BEGIN
        filebeam = FITSFILE_BEAM(mapfile)
     ENDELSE
  ENDIF
  ;; If input map is in array format
  ;;---------------------------------
  IF maptype LE 5 THEN BEGIN
     ;; Write the map
     IF NOT DEFINED(lmax) THEN BEGIN
        IF PSM_CAREFULNESS() EQ 0 THEN BEGIN
           npixin = N_ELEMENTS(map[*,0])
           nsidein = NPIX2NSIDE(npixin)
           lmax = 2L*nsidein
        ENDIF ELSE BEGIN
           PSM_DEBUG_MESSAGE, 'Input value for lmax keyword requested', routine='PSM_MAP2ALM', infolevel=0, stoplevel=0
        ENDELSE
     ENDIF
     mapfile = PSM_TMPFILENAME(extension='.fits')
     arraypolarised = ISPOLARISED(map,/map)
     IF arraypolarised THEN WRITE_HPXMAP, mapfile, beam=input_beam, tqustokes=map, ordering=ordering, lmax=lmax ELSE WRITE_HPXMAP, mapfile, tstokes=map, beam=input_beam, ordering=ordering, lmax=lmax
     erasemapfile = 1B          ; the temporary file will be erased at the end
     IF DEFINED(input_beam) THEN BEGIN
        filebeam = FITSFILE_BEAM(mapfile)
        IF ISANUMBER(input_beam) THEN input_beam_struct=GET_BEAM_STRUCT('GAUSSIAN',beamsize=input_beam,sizeunits='arcmin') ELSE input_beam_struct=input_beam
        IF NOT STRUCT_EQUAL(filebeam, input_beam_struct) THEN BEGIN
           IF filebeam.type EQ 'UNKNOWN' THEN filebeam=input_beam_struct ELSE $
              MESSAGE, 'Problem: The input beam provided as a keyword is incompatible with what is read in the input file'
        ENDIF
     ENDIF ELSE BEGIN
        filebeam = FITSFILE_BEAM(mapfile)
     ENDELSE
  ENDIF

  ;; Test the type of the output alm
  ;;---------------------------------
     defalm = DEFINED(alm)
     IF NOT defalm THEN BEGIN
        outname = PSM_TMPFILENAME(extension='.fits')
        erasealmfile = 1B
        nohdr = 1B
     ENDIF ELSE BEGIN
        almtype = SIZE(alm, /type)
        IF almtype EQ 7 THEN BEGIN
           outname = alm
           erasealmfile = 0B
           IF NOT KEYWORD_SET(nohdr) THEN nohdr = 0B ELSE nohdr = 1B
        ENDIF ELSE BEGIN
           IF NOT KEYWORD_SET(overwrite) THEN BEGIN
              MESSAGE, 'alm variable already exists, set /overwrite to replace it by new alm'
           ENDIF ELSE BEGIN
              DELVARX, alm
              outname = PSM_TMPFILENAME(extension='.fits')
              erasealmfile = 1B
              nohdr = 1B
           ENDELSE
        ENDELSE
     ENDELSE

     ;; Erase old file if it exists and if requested
     ;;----------------------------------------------
     IF KEYWORD_SET(overwrite) THEN FILE_DELETE, outname, /allow_nonexistent, /quiet
     
     ;; Write configuration file for anafast_cxx
     ;;------------------------------------------
     conf = PSM_TMPFILENAME(extension='.conf')
     GET_LUN, unit
     OPENW, unit, conf
     IF NOT keyword_set(weighted) THEN BEGIN
        PRINTF, unit,'weighted = false'       
     ENDIF ELSE BEGIN
        healpix_data=!HEALPIX_DIR+'/data/'
        PRINTF, unit,'weighted = true'
        PRINTF, unit,'healpix_data = '+ healpix_data
     ENDELSE
     
     ;; Find input and output beams, decide whether rebeaming is needed, write a temporary file if so

     IF NOT KEYWORD_SET(output_beam) THEN BEGIN
        output_beam = filebeam
        norebeam = 1B
     ENDIF ELSE BEGIN
        IF ISSTRUCT(filebeam) THEN BEGIN
           IF ISSTRUCT(output_beam) THEN BEGIN
              norebeam = STRUCT_EQUAL(filebeam, output_beam)
           ENDIF ELSE BEGIN
              IF ISANUMBER(output_beam) THEN BEGIN
                 struct_output_beam = GET_BEAM_STRUCT(output_beam)
                 norebeam = STRUCT_EQUAL(filebeam, struct_output_beam)
              ENDIF ELSE MESSAGE, 'Unexpected beam type'
           ENDELSE
        ENDIF ELSE BEGIN
           IF ISSTRUCT(output_beam) THEN BEGIN
                 struct_filebeam = GET_BEAM_STRUCT(filebeam)
                 norebeam = STRUCT_EQUAL(struct_filebeam, output_beam)
           ENDIF ELSE BEGIN
              norebeam = (filebeam EQ output_beam)
           ENDELSE
        ENDELSE
     ENDELSE
     
     IF norebeam THEN outfile = outname ELSE outfile = PSM_TMPFILENAME(extension='.fits')
     
     ;; lmax
     IF NOT DEFINED(lmax) THEN BEGIN
        filelmax = FITSFILE_LMAX(mapfile)
        IF filelmax NE -1 THEN lmax=filelmax ELSE BEGIN
           h = HEADFITS(mapfile,exten=1)
           nside = SXPAR(h,'NSIDE') & lmax = 2*nside
        ENDELSE
     ENDIF
     PRINTF, unit, 'nlmax = '+ NUMBER2STRING(lmax)
     
     ;; File names
     PRINTF, unit, 'infile = '+ mapfile        
     PRINTF, unit, 'outfile_alms = ' + outfile     
     ;; Polarised ?
     IF NOT DEFINED(polar) THEN polar = ISPOLARISED(map,/map)
     IF polar THEN PRINTF, unit, 'polarisation = true' ELSE PRINTF, unit, 'polarisation = false'
     
     IF NOT KEYWORD_SET(iter) THEN iter=0
     PRINTF, unit, 'iter_order = ' + NUMBER2STRING(iter)
     IF KEYWORD_SET(double) THEN PRINTF, unit, 'double_precision = true'
     CLOSE, unit
     FREE_LUN, unit

     ;; Launch the C++ executable
     ;;---------------------------  
     SPAWN, 'anafast_cxx '+conf

     ;; Erase the configuration file
     ;;------------------------------
     FILE_DELETE, conf, /quiet
     
     ;;-------------------------------------------------------------------------------
     ;; Fix the header in all extensions
     ;;-------------------------------------------------------------------------------
     IF nohdr NE 1 THEN BEGIN
        hdmap = HEADFITS(mapfile,exten=1)
        FITS_INFO, outfile, n_ext=nexten, /silent
        
        SPLIT_FITSHDR, hdmap, hout=hout, hend=hend, hpsm=hpsm
        
        ;; If there is already a PSM header in hdmap, fix it, otherwise create it
        ;;------------------------------------------------------------------------
        IF N_ELEMENTS(hpsm) LT 2 THEN hpsm = [PSMHDR_HEADLINE(),PSMHDR_ENDLINE()]
        
        ;; Update the base header block
        EXTRACT_PSMHDRBLOCK, hpsm, blockname='base', whereblock=whereblock ;, /silent
        IF whereblock[0] NE -1 THEN BEGIN
           UPDATEHDRPAR, hpsm, 'PSM_DTFM', 'ALM'
           PSM_ID_KEYGEN, dtkey
           UPDATEHDRPAR, hpsm, 'PSM_DTID', dtkey
           PSM_ID_KEYGEN, flkey
           UPDATEHDRPAR, hpsm, 'PSM_FLID', flkey
        ENDIF ELSE BEGIN
           PSM_DEBUG_MESSAGE, 'Insert a base header block', routine='PSM_MAP2ALM'
           basehdr = PSM_BASE_HDR(dataform='ALM')
           hpsm = MERGE_FITSHDRS(hpsm,basehdr)
        ENDELSE
        
        ;; Replace the map header block by an appropriate alm header block
        EXTRACT_PSMHDRBLOCK, hpsm, blockname='map', wherenotblock=wherenotblock, whereblock=whereblock
        IF whereblock[0] NE -1 THEN BEGIN
           mapblock = hpsm[whereblock]
           writelmax = SXPAR(mapblock, 'PSM_LMAX')
           writepixwin = SXPAR(mapblock, 'PXWIN')
        ENDIF ELSE BEGIN
           writelmax = lmax
           writepixwin = 0
        ENDELSE
        IF wherenotblock[0] NE -1 THEN hpsm = hpsm[wherenotblock]
        almhdrblock = PSM_ALM_HDR(lmax=writelmax)
        UPDATEHDRPAR, almhdrblock, 'PXWIN', writepixwin
        UPDATE_PSMHDRBLOCK, hpsm, almhdrblock
        
        ;; Update the fits header in all extensions of the alm file
        ;;----------------------------------------------------------
        FOR e=1, nexten DO BEGIN
           hdalm = HEADFITS(outfile,exten=e)
           
           ;; Create output header for this extension by merging in the psm header
           hdalm = MERGE_FITSHDRS(hdalm,hpsm)
           
           ;; Update units in standard fits header
           unit = SXPAR(hdmap,'TUNIT1', count=count)
           IF count EQ 1 THEN BEGIN
              SXADDPAR, hdalm, 'TUNIT2', unit, after='TFORM2'
              SXADDPAR, hdalm, 'TUNIT3', unit, after='TFORM3'
           ENDIF
           
           REORDER_FITSHDR, hdalm
           MODFITS, outfile, 0, hdalm, EXTEN_NO = e
        ENDFOR
        
     ENDIF
     
     ;; Change the beam if needed
     ;;---------------------------
     IF NOT norebeam THEN BEGIN
        IF NOT DEFINED(filebeam) THEN STOP
        IF NOT DEFINED(output_beam) THEN STOP
        CHANGE_ALM_RESOLUTION, outfile, outname, beam_in=filebeam, beam_out=output_beam, double=double, lmax=lmax
        FILE_DELETE, outfile, /allow_nonexistent, /quiet
     ENDIF
     
     IF erasemapfile THEN FILE_DELETE, mapfile, /allow_nonexistent, /quiet
     IF erasealmfile THEN BEGIN
        READ_ALM, outname, alm
        FILE_DELETE, outname, /allow_nonexistent, /quiet
     ENDIF
     
     
END
