; -----------------------------------------------------------------------------
;
; Copyright (C) 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SUB_CL
;
; PURPOSE:
;	Function to extract from a cl structure a substructure with only a fraction of the fields
;
; CATEGORY:
;	...
;
; CALLING SEQUENCE:
;	Result = SUB_CL(Cl, FIELDINDEX=)
;
; INPUTS:
;	Cl:	A Cl structure created with the CL_STRUCTURE function
;
; KEYWORD PARAMETERS:
;	FIELDINDEX:	The list of field indices to be kept in the output Cl.
;
; OUTPUTS:
;	Result is a Cl structure extracted from the input one.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	14/09/2012:	Jacques Delabrouille, First version.
;-
FUNCTION SUB_CL, cl, fieldindex=fieldindex, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, "SUB_CL: function to extract from a cl structure a substructure with only a fraction of the fields"
     PSM_INFO_MESSAGE, "SYNTAX: result = SUB_CL(cl, fieldindex=)"
     PSM_INFO_MESSAGE, "'cl' is an output cl_structure"
     PSM_INFO_MESSAGE, "'fieldindex' (required) is an input keyword for specifying which Cl indices (starting at 1) are to be extracted"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  fieldnames = cl.field[fieldindex-1]
  nfield = N_ELEMENTS(fieldindex-1)
  lmax = MAX(cl.l)
  IF N_ELEMENTS(cl.l) NE lmax+1 THEN MESSAGE, 'Error, unexpected number of ell values in input cl structure'
  clout = CL_STRUCTURE(fieldnames=fieldnames, lmax=lmax, double=ISDOUBLE(cl.c_1_1))
  
  FOR i=0, nfield-1 DO BEGIN
     is = NUMBER2STRING(i+1)
     iis = NUMBER2STRING(fieldindex[i])
     FOR j=i, nfield-1 DO BEGIN
        js = NUMBER2STRING(j+1)
        jjs = NUMBER2STRING(fieldindex[j])
        ok = EXECUTE('clout.c_'+is+'_'+js+' = cl.c_'+iis+'_'+jjs)
        IF NOT ok THEN MESSAGE, 'Command could not be executed'
     ENDFOR
  ENDFOR

  RETURN, clout

END
