; -----------------------------------------------------------------------------
;
; Copyright (C) 2008-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_STRING_TO_LENGTH
;
; PURPOSE:
;	Routine to add at the end (or beginning) of a string the
;	number of trailing (or leading) blanks to make its length 
;	match the given input length (useful for writing formatted 
;	strings in a file).
;
; CATEGORY:
;	String handling
;
; CALLING SEQUENCE:
;	PSM_STRING_TO_LENGTH, Strin, Leng
;
; INPUTS:
;	Strin:	A string or string array.
;	Leng:   The desired string length.
;
; OPTIONAL INPUTS:
;	None
;	
; KEYWORD PARAMETERS:
;	STRICT:    Set this keyword for last characters to be cut out if
;	           the input string is greater than length.
;
;	LEADING:   Set this keyword to add characters at beginning
;	           rather than end.
;       
;	CHARACTER: Set this keyword to a one character string, to set
;	           the character added at beginning or end (default=blank
;	           character ' ').
;
; OUTPUTS:
;	The result is the modified string or string array.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple string manipulation.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	02/12/2008:	Jacques Delabrouille, First version
;
;-

FUNCTION SINGLE_STRING_TO_LENGTH, strin, leng, strict=strict, leading=leading, character=character

instrlen = STRLEN(strin)
st=strin

IF NOT KEYWORD_SET(character) THEN character=' '
IF STRLEN(character) EQ 0 THEN character=' '

IF instrlen LT leng THEN BEGIN
  nblanks = leng-instrlen
  IF NOT KEYWORD_SET(leading) THEN FOR i=0, nblanks-1 DO st=st+character ELSE FOR i=0, nblanks-1 DO st=character+st
ENDIF
len = STRLEN(st)

IF KEYWORD_SET(strict) NE 0 THEN BEGIN
   IF NOT KEYWORD_SET(leading) THEN st = STRMID(st,0,leng) ELSE st = STRMID(st,len-leng,leng)
ENDIF

RETURN, st
END

FUNCTION PSM_STRING_TO_LENGTH, strin, leng, strict=strict, leading=leading, character=character, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, 'PSM_STRING_TO_LENGTH: function for formatting a string to a given length'
     PRINT, 'SYNTAX: result = PSM_STRING_TO_LENGTH(strin, leng, strict=, leading=, character=)'
     RETURN, 0
  ENDIF

IF ISARRAY(strin) THEN BEGIN

   nstr = N_ELEMENTS(strin)
   st = strin
   FOR i=0, nstr-1 DO st[i] = SINGLE_STRING_TO_LENGTH(strin[i], leng, strict=strict, leading=leading, character=character)
   RETURN, st

ENDIF ELSE RETURN, SINGLE_STRING_TO_LENGTH(strin, leng, strict=strict, leading=leading, character=character)



END
