; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Marc Betoule
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FILL_STRUCT
;
; PURPOSE:
;	Fills each field of a structure by looking for a corresponding
;	keyword in a table
;
; CATEGORY:
;	Structure handling
;
; CALLING SEQUENCE:
;	FILL_STRUCT, Table, Struct
;
; INPUTS:
;	Table:	Vector of couple (keyword, value)
;	Struct:	The structure to be filled
;
; KEYWORD PARAMETERS:
;	TAGS:   	The tags to be filled (default all tags of the structure)
;
;	CAREFULNESS:	0, 1 or 2, set how careful the program is
;
;	EXPECTED_DIM:	The expected dimension of the array for tags,
;	                if a control is needed. 
;
;	STRICT_DIM:	if not set then a single element is duplicated for output to match EXPECTED_DIM
;
; OUTPUTS:
;	Struct: The structure to be filled, modified according to
;	input values in the input Table
;
; PROCEDURE:
;	Looks in table for keywords matching those present in the
;	structure, modify the values in the structure whenever needed
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	30/01/2007:	Marc Betoule, First version.
;	27/01/2009:	Jacques Delabrouille, Added tags keyword + some rewriting
;	22/05/2009:	JD, Keyword for carefulness added, improved handling of missing or repeated keywords
;
;-
FUNCTION READ_KEYWORD, table, keyword, default=default, carefulness=carefulness, expected_dim=expected_dim, strict_dim=strict_dim

  entries = strupcase(table[0,*])
  entries = strcompress(entries, /remove_all)

  keyword = strupcase(keyword)
  keyword = strcompress(keyword, /remove_all)
  
  index = where(entries eq keyword, noccur)
 
  care = 0
  IF KEYWORD_SET(carefulness) NE 0 THEN care=carefulness
 
  IF noccur EQ 0 THEN BEGIN
     IF care GE 3 THEN BEGIN 
        MESSAGE, "Missing keyword " + keyword + ": STOPPING THE PSM RUN", /info
        STOP
     ENDIF ELSE BEGIN
        IF DEFINED(default) THEN BEGIN
           def_type = SIZE(default, /type)
           IF def_type GE 1 AND def_type LE 5 THEN def = NUMBER2STRING(default) ELSE def=default
           nel = N_ELEMENTS(def)
           IF nel GT 1 THEN def="array of default values" ELSE BEGIN
              IF def_type EQ 7 THEN def = "'"+STRTRIM(def[0])+"'" ELSE def=STRTRIM(def[0])
           ENDELSE
           res = EXECUTE( 'MESSAGE, "Missing keyword '+ keyword +" set to " + def +' by default", /info' )
           RETURN, default
        ENDIF ELSE BEGIN
        MESSAGE, "Missing keyword " + keyword + ' (No default value defined)'
        ENDELSE
     ENDELSE
  ENDIF ELSE BEGIN
     val_list = table[1,index]
     val = val_list(UNIQ(val_list))
     noccur = N_ELEMENTS(val)
     IF noccur EQ 1 THEN BEGIN
        val = table[1,index[0]]
        val = strtrim(val,2)
        val = strcompress(val)
        if strpos(val,' ') NE -1 THEN BEGIN
           pos1=0
           pos2=strpos(val,' ',pos1)
           tab = [strmid(val,pos1, pos2-pos1)]
           pos1 = pos2+1
           pos2 = strpos(val,' ',pos1)
           WHILE pos2 NE -1 DO BEGIN
              tab = [tab,strmid(val,pos1, pos2-pos1)]
              pos1 = pos2+1
              pos2 = strpos(val,' ',pos1)
           ENDWHILE
           tab=[tab, strmid(val, pos1)]
           IF DEFINED(expected_dim) THEN BEGIN
              neltab = N_ELEMENTS(tab)
              IF neltab NE expected_dim THEN MESSAGE, "Keyword '"+ keyword +"' : Unexpected number of elements"
           ENDIF
           RETURN, tab
        ENDIF ELSE BEGIN
           IF DEFINED(expected_dim) THEN BEGIN
              neltab = N_ELEMENTS(val)
              IF neltab NE 1 THEN MESSAGE, 'Case not implemented'
              IF ((neltab NE expected_dim) AND KEYWORD_SET(strict_dim) NE 0) THEN $
                 MESSAGE, "Keyword '"+ keyword +"' : Unexpected number of elements" $
              ELSE RETURN, REPLICATE(val,expected_dim)
           ENDIF
           RETURN, val
        ENDELSE
     ENDIF ELSE BEGIN
        IF care GE 1 THEN BEGIN 
           MESSAGE, "Keyword " + keyword + " occurs more than once"
        ENDIF ELSE BEGIN
           IF DEFINED(default) THEN BEGIN
              def_type = SIZE(default, /type)
              IF def_type GE 1 AND def_type LE 5 THEN def = NUMBER2STRING(default) ELSE def=default
              res = EXECUTE( "MESSAGE, 'Multiply defined keyword "+ keyword +" set to " + def +" by default', /info" )
              RETURN, default
           ENDIF ELSE BEGIN
              MESSAGE, "Keyword " + keyword + " occurs more than once"
           ENDELSE
        ENDELSE
     ENDELSE
  ENDELSE
END

;------------------------------------------------------------------------------------------------------------------------
PRO FILL_STRUCT, table, struct, tags=tags, carefulness=carefulness, expected_dim=expected_dim, strict_dim=strict_dim

IF NOT KEYWORD_SET(tags) THEN tags = tag_names(struct)
FOR i = 0, N_ELEMENTS(tags)-1 DO BEGIN
   ok = EXECUTE('default = struct.'+tags[i])
   value = [READ_KEYWORD(table, tags[i], default=default,carefulness=carefulness,expected_dim=expected_dim,strict_dim=strict_dim)]
   result = EXECUTE( 'struct.'+tags[i]+' = value' )
   IF result EQ 0 THEN BEGIN
      IF PSM_CAREFULNESS() GE 1 THEN BEGIN
         PSM_INFO_MESSAGE, tags[i] + ' cannot be set to input value read from table'
         MESSAGE, 'Process aborted'
      ENDIF ELSE BEGIN
         WARNING_LINE
         PSM_INFO_MESSAGE, tags[i] + ' cannot be set to input value read from table'
         PSM_INFO_MESSAGE, 'Using default value for '+tags[i]
         WARNING_LINE
      ENDELSE
   ENDIF
   other = EXECUTE( 'str = struct.'+tags[i])
   typ = SIZE(str, /type)
   IF ((typ GE 2) AND (typ LE 5)) THEN str = NUMBER2STRING(str)
ENDFOR
END
