; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;   Frederic Guilloux
;   Mathieu Remazeilles
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MAP2WAVELETS
;
; PURPOSE:
;	Compute wavelets coefficients af a healpix map
;
; CATEGORY:
;	Wavelets (needlets) processing
;
; CALLING SEQUENCE:
;	MAP2WAVELETS, Map, Wavelets
;
; INPUTS:
;	Map:    	Input HEALPix map (can be a fits file or an array).
;	Wavelets:	Fits file name for output wavelets.
;
; KEYWORD PARAMETERS:
;	BANDS:  	The wavelets bands to be used.
;	DOUBLE: 	Set this keyword for double precision.
;	OVERWRITE: 	Normally, this code does not overwrite any
;                	existing wavelets file if it already.
;                	exists. Set the OVERWRITE keyword to erase and
;                	replace an existing file.
;	SIGNAL: 	Signal type (can be 'T' for temperature only,
;                  	or 'TP' for temperature and polarisation).  
;                  	Only 'T' case [the default] is implemented for now.
;	WAVELETNSIDE: 	Set this keyword to a usual Healpix NSIDE
;	                (e.g. 512) to force storing the needlet
;	                coefficients at that NSIDE for all scales. 
;	NODEGRADE: 	Set this keyword to store needlet coefficients
;	                at the same NSIDE at all scales.
;	NSIDE_MAX_WIN: 	Set this keyword to define an upper limit to
;                	the HEALPix NSIDE used for storing needlet coefficients.
;	NSIDE_MIN_WIN: 	Set this keyword to define a lower limit to
;                	the HEALPix NSIDE used for storing needlet coefficients.
;	INPUT_BEAM: 	Set this keyword to the beam of the input map
;	                (if you wish to change the resolution). The
;	                beam is either a FWHM in arcminutes, or a
;	                structure crated with the GET_BEAM_STRUCT
;	                function. The default is 0 (no beam, FWHM=0)
;	OUTPUT_BEAM: 	Set this keyword to the beam of the output map
;	                (if you wish to change the resolution). The
;	                beam is either a FWHM in arcminutes, or a
;	                structure crated with the GET_BEAM_STRUCT
;	                function. By default the output beam is equal
;	                to the input beam (no change of resolution).
;	ITER:   	Set this keyword to iterate the HEALPix
;	                harmonic transform. 
;	WEIGHTED:   	Set this keyword to weight the HEALPix
;	                harmonic transform. 
;	ORDERING:   	Set this keyword to the ordering of the input
;	                HEALPix map, if provided in array format. Can
;	                be 'RING' (the default) or 'NESTED'.
;
;
; OUTPUTS:
;	A file is written, which contains the output needlet
;	coefficients and the bands used to compute them.
;
; SIDE EFFECTS:
;	A file is written to disk. The computation of wavelet
;	coefficients requires a spherical harmonic transform, so it is
;	better to use ALM2WAVELETS if you also have a file with the
;	harmonic coefficients available.
;
; PROCEDURE:
;	First compute alm coefficients, and then use ALM2WAVELETS.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/06/2010:	Jacques Delabrouille, First version (adapted
; 	                from my previous implementation of the needlet
; 	                transform).
; 	30/06/2010:	JD, Set default bands.
; 	09/09/2010:	JD, Propagate /double keyword
; 	01/10/2010:	Mathieu Remazeilles, Implement ITER and
; 	                WEIGHTED keywords.
; 	08/02/2011:	Frederic Guilloux, Implement OVERWRITE keyword.
; 	07/07/2011:	JD, Implement NODEGRADE keyword.
; 	02/09/2011:	JD, Small bug fix connected to implementation
; 	                of OVERWRITE keyword.
; 	06/10/2011:	JD, Implement INPUT_BEAM and OUTPUT_BEAM
; 	                keywords, and call ALM2WAVELETS routine.
; 	04/01/2012:	JD, PSM license header.
; 	09/01/2012:	JD, doc header + general clean-up.
; 	
;-
PRO MAP2WAVELETS, map, wavelets, bands=bands, double=double, signal=signal, overwrite=overwrite, nodegrade=nodegrade, nside_max_w=nside_max_w, nside_min_w=nside_min_w, input_beam=input_beam, output_beam=output_beam, iter=iter, weighted=weighted, ordering=ordering, lmax=lmax

  ;; Type of the output (file?), cleanup if necessary
  ;;--------------------------------------------------
  originalnside = GET_NSIDE(map)
  waveletstype = SIZE(wavelets, /type)
  IF waveletstype EQ 7 THEN waveletsfile = wavelets ELSE STOP
  exists = FILE_TEST(waveletsfile)
  IF exists THEN BEGIN
     IF NOT KEYWORD_SET(overwrite) THEN BEGIN
        PSM_INFO_MESSAGE, 'File exists, aborting. Set /overwrite to overwrite file: '+waveletsfile
        GOTO, Closing
     ENDIF ELSE BEGIN
        FILE_DELETE, waveletsfile
     ENDELSE
  ENDIF

  ;; Set lmax
  ;;----------
  IF NOT KEYWORD_SET(bands) THEN BEGIN
     IF NOT KEYWORD_SET(lmax) THEN lmax=2L*originalnside
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(lmax) THEN BEGIN 
        totbands = TOTAL(bands^2,2)
        wh = WHERE(totbands NE 0)
        lmax = MAX(wh)
        lmax = lmax<(3L*originalnside-1)
     ENDIF ELSE BEGIN
        lmax=2L*originalnside
     ENDELSE
  ENDELSE

  ;; Harmonic transform
  ;;--------------------
  PSM_MAP2ALM, map, alm, double=double, overwrite=overwrite, polar=polar, lmax=lmax, iter=iter, $
               weighted=weighted, input_beam=input_beam, output_beam=output_beam, ordering=ordering, /nohdr

  IF NOT KEYWORD_SET(signal) THEN signal='T'  

  ALM2WAVELETS, alm, wavelets, signal=signal, bands=bands, double=double, overwrite=overwrite, $
                waveletnside=waveletnside, nodegrade=nodegrade, nside_max_w=nside_max_w, nside_min_w=nside_min_w


Closing:
END
