; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Soumen Basak
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	QUASIGAUSS_ELL_BANDS
;
; PURPOSE:
;	This function generates quasi gaussian-shaped windows in harmonic mode
;	(ell) space, used for wavelet (needlet) analysis of spherical
;	maps. Bands are truncated in ell for effectivenes in harmonic transforms
;
; CATEGORY:
;	Wavelet (needlet) processing.
;
; CALLING SEQUENCE:
;	Result = QUASIGAUSS_ELL_BANDS()
;
; INPUTS:
;	None (all parameters are passed by keywords)
;
; KEYWORD PARAMETERS:
;	BEAMSIZES:	Size of gaussian beams in arcminutes. An array of Nband numbers.
;	SINGLE: 	By default, the bands are in double precision.
;	                Set this keyword for single floating point precision.
;
; OUTPUTS:
;	The result is an array of dimension
;	[N_ELEMENT(beamsizes)+1,Nband], that gives the shapes of the
;	spectral bands. These bands are such that the sum of their
;	squares is equal to 1 for all ell.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	
;	Create a set of bands:
;	b = QUASIGAUSS_ELL_BANDS(beamsizes=[120,60,20,10,5], /single) 
;	
;	View them using the PLOT_BANDS routine:
;	PLOT_ELL_BANDS, b, yrange=[0,1.2]
;
; MODIFICATION HISTORY:
; 	15/02/2012:	Jacques Delabrouille, First version 
;
;-
FUNCTION QUASIGAUSS_ELL_BANDS, beamsizes=beamsizes, single=single, lmax=lmax, help=help

  IF ((N_PARAMS() NE 0) OR (KEYWORD_SET(help) NE 0)) THEN BEGIN
     PRINT, 'Syntax : bands = GAUSSBEAM_ELL_BANDS( [beamsizes=, single=, lmax=] )'
     GOTO, closing
  ENDIF
  
  IF NOT KEYWORD_SET(beamsizes) THEN BEGIN
     beamsizes = [600,300,120,60,45,32,21,13,10,7.,4.5]
  ENDIF
  
  sortedbeamsizes = beamsizes(SORT(-beamsizes))

  IF NOT KEYWORD_SET(lmax) THEN lmax = LONG(360.*60./MIN(beamsizes))

  nbands = N_ELEMENTS(beamsizes) + 1
  IF NOT KEYWORD_SET(single) THEN BEGIN
     double=1B 
     bands = DBLARR(lmax+1,nbands) 
  ENDIF ELSE BEGIN 
     double=0B
     bands = FLTARR(lmax+1,nbands) 
  ENDELSE

  ;; Threshold
  ;;-----------
  threshold = 1e-6

  ;; First band
  ;;------------
  bands[*,0] = PSM_GET_BL(sortedbeamsizes[0],lmax=lmax, double=double)^2
  wh1 = WHERE((bands[*,0] / MAX(bands[*,0])) GE threshold)
  wh2 = WHERE((bands[*,0] / MAX(bands[*,0])) GE 10*threshold)
  factor = REPLICATE(0.,lmax+1)
  factor[wh2] = 1.
  ntrans = MAX(wh1)-MAX(wh2)
  factor[MAX(wh2):MAX(wh1)] = 1-FINDGEN(ntrans+1)/ntrans
  bands[*,0] = bands[*,0] * (factor>0)^2

  ;; Intermediate bands
  ;;--------------------
  IF nbands GT 2 THEN BEGIN
     FOR i=1, nbands-2 DO BEGIN
        bands[*,i] = (PSM_GET_BL(sortedbeamsizes[i],lmax=lmax, double=double))^2
        wh1 = WHERE((bands[*,i] / MAX(bands[*,i])) GE threshold)
        wh2 = WHERE((bands[*,i] / MAX(bands[*,i])) GE (10*threshold))
        factor = REPLICATE(0.,lmax+1)
        factor[wh2] = 1.
        ntrans = MAX(wh1)-MAX(wh2)
        IF ntrans NE 0 THEN factor[MAX(wh2):MAX(wh1)] = 1-FINDGEN(ntrans+1)/ntrans
        bands[*,i] = bands[*,i] * (factor>0)^2
        IF i EQ 1 THEN bands[*,i] = bands[*,i] - bands[*,0] $
        ELSE bands[*,i] = bands[*,i] - TOTAL(bands[*,0:i-1],2)
     ENDFOR
  ENDIF

  ;; Last band
  ;;-----------
  bands[*,nbands-1] = 1-PSM_GET_BL(sortedbeamsizes[nbands-2],lmax=lmax, double=double)^2

  ;; Take the square root
  ;;----------------------
  bands = SQRT(bands)
  
  IF ((NOT KEYWORD_SET(double)) AND (KEYWORD_SET(single))) THEN RETURN, FLOAT(bands) ELSE RETURN, bands
  
  closing:

END


