; -----------------------------------------------------------------------------
;
; Copyright (C) 2012 
;   Jacques Delabrouille 
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	WHITEN_WAVELETS
;
; PURPOSE:
;	This procedure whitens a set a needlet coefficients.
;
; CATEGORY:
;	Wavelets (needlets) processing
;
; CALLING SEQUENCE:
;	WHITEN_WAVELETS, Wavelets_in, Wavelets_out
;
; INPUTS:
;	Wavelets_in:  The input fits file of needlet coefficients.
;
; OPTIONAL INPUTS:
;	None
;	
; KEYWORD PARAMETERS:
;	OVERWRITE:	Set this keyword for the procedure to erase
;	                and replace any existing output wavelet file.
;	                If not set then the program exits if a file
;	                exists with the name of the expected output.
;
;	Extra keywords that govern the computation of the local
;	standard deviation of the input needlet coefficients with
;	HPX_LOCALSIGMA are passed to that routine through the _EXTRA
;	keyword mechanism. 
;
; OUTPUTS:
;	Wavelets_out: The output fits file of whitened needlet coefficients.
;
; OPTIONAL OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;	Creates (and possibly overwrites) a file.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Computes the local standard deviation of needlet coefficients
;	using HPX_LOCALSIGMA, and then divides the needlet
;	coefficients by the local standard deviation.
;
; EXAMPLE:
;	
;
; MODIFICATION HISTORY:
; 	08/12/2011:	Jacques Delabrouille, first version.
;	03/01/2012:	Added standard IDL documentation header
;-
PRO WHITEN_WAVELETS, wavelets_in, wavelets_out, overwrite=overwrite, _extra=_extra, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WHITEN_WAVELETS: program to whiten a st of needlet coefficients'
     PSM_INFO_MESSAGE, 'SYNTAX: WHITEN_WAVELETS, wavelets_in, wavelets_out, overwrite=, degrade=, niter=, nocentralpix=, minpts='
     PSM_INFO_MESSAGE, "'wavelets_in' is the input file of needlet coefficients"
     PSM_INFO_MESSAGE, "EXTRA keywords used for computing the local standard deviation are passed with the following:"
     PSM_INFO_MESSAGE, "degrade=, niter=, /nocentralpix, minpts=."
     PSM_INFO_MESSAGE, "The HPX_LOCALSIGMA procedure used to calculate the local standard deviation"
     PSM_INFO_MESSAGE, "computes averages over pixels larger by the factor degrade (must be a power of 2, "
     PSM_INFO_MESSAGE, "default is 8), and uses HPX_PIXSMOOTHING to average the over all neighbouring pixels"
     PSM_INFO_MESSAGE, "at degraded resolution"
     PSM_INFO_MESSAGE, "'niter' sets the number of iterations of the HPX_PIXSMOOTHING procedure (done 1+niter times)"
     PSM_INFO_MESSAGE, "Set /nocentralpix for discarding the (degrade x degrade) central pixels in sigma calculation"
     PSM_INFO_MESSAGE, "Set minpts [default = 1] to the minimum number of points required to return the sigma."
     PSM_INFO_MESSAGE, "pixels for which the minimum is not reached are flagged as bad."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ;; Check format of input and output
  ;;----------------------------------
  ok = IS_FITS_FILE(wavelets_in)
  IF NOT ok THEN BEGIN
     STOP
     GOTO, Closing
  ENDIF

  waveletstype = SIZE(wavelets_out, /type)
  IF waveletstype EQ 7 THEN waveletsfile = wavelets ELSE STOP
  exists = FILE_TEST(waveletsfile)
  IF exists THEN BEGIN
     IF NOT KEYWORD_SET(overwrite) THEN BEGIN
        PSM_INFO_MESSAGE, 'File exists, aborting. Set /overwrite to overwrite file: '+waveletsfile
        GOTO, Closing
     ENDIF ELSE BEGIN
        FILE_DELETE, waveletsfile
     ENDELSE
  ENDIF

  ;; Copy the wavelet bands into the wavelets_out fits file
  ;;--------------------------------------------------------

  bands = MRDFITS(wavelets_in)
  MWRFITS, bands, wavelets_out

  ;; For each of the bands whiten the needlet coefficients
  ;;-------------------------------------------------------
  nb = N_ELEMENTS(bands[0,*])

  FOR b=1, nb DO BEGIN

     map = MRDFITS(wavelets_in,b)

     ;; Compute the local standard deviation using HPX_LOCALSIGMA
     sigma = HPX_LOCALSIGMA(map, degrade=degrade, niter=niter, nocentralpix=nocentralpix, minpts=minpts)

     ;; Divide the wavelets coefficients by the local standard deviation
     map = TEMPORARY(map)/sigma

     ;; Write the new wavelets into the wavelets_out file
     MWRFITS, map, wavelets_out 

  ENDFOR 

  Closing:

END
